Package: easyRaschBayes
Title: Bayesian Rasch Analysis Using 'brms'
Version: 0.2.0.1
Authors@R: c(
    person("Magnus", "Johansson",
           role = c("aut", "cre"),
           email = "pgmj@pm.me",
           comment = c(ORCID = "0000-0003-1669-592X")),
    person("Giacomo", "Bignardi",
           role = "ctb",
           comment = "RMU reliability code"))
Description: Reproduces classic Rasch psychometric analysis features
    using Bayesian item response theory models fitted with 'brms' following 
    Bürkner (2021) <doi:10.18637/jss.v100.i05> and Bürkner (2020) <doi:10.3390/jintelligence8010005>.
    Supports both dichotomous and polytomous Rasch models. 
    Features include posterior predictive item fit, conditional infit, item-restscore
    associations, person fit, differential item functioning, local dependence
    assessment via Q3 residual correlations, dimensionality assessment with
    residual principal components analysis, person-item targeting plots,
    item category probability curves, and reliability using relative
    measurement uncertainty following Bignardi et al. (2025) <doi:10.31234/osf.io/h54k8_v1>.
License: GPL (>= 3)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: brms (>= 2.20.0), rlang (>= 1.0.0), dplyr (>= 1.1.0), tidyr
        (>= 1.3.0), tibble (>= 3.0.0), ggdist, stats, grDevices,
        ggplot2 (>= 3.4.0), forcats
Suggests: ggrepel, patchwork, eRm, testthat (>= 3.0.0), knitr,
        rmarkdown
URL: https://github.com/pgmj/easyRaschBayes,
        https://pgmj.github.io/easyRaschBayes/
BugReports: https://github.com/pgmj/easyRaschBayes/issues
VignetteBuilder: knitr
Config/testthat/edition: 3
NeedsCompilation: no
Packaged: 2026-04-23 06:45:31 UTC; magnus.johansson.3
Author: Magnus Johansson [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-1669-592X>),
  Giacomo Bignardi [ctb] (RMU reliability code)
Maintainer: Magnus Johansson <pgmj@pm.me>
Repository: CRAN
Date/Publication: 2026-04-23 07:10:02 UTC
Built: R 4.6.0; ; 2026-04-26 17:49:07 UTC; unix

# Generated by roxygen2: do not edit by hand

export(RMUreliability)
export(dif_statistic)
export(fit_statistic_pcm)
export(fit_statistic_rm)
export(infit_post)
export(infit_statistic)
export(item_parameters)
export(item_restscore_post)
export(item_restscore_statistic)
export(person_parameters)
export(plot_bars)
export(plot_icc)
export(plot_ipf)
export(plot_residual_pca)
export(plot_stackedbars)
export(plot_targeting)
export(plot_tile)
export(posterior_to_prior)
export(q3_post)
export(q3_statistic)
importFrom(brms,as_draws_df)
importFrom(brms,bf)
importFrom(brms,brm)
importFrom(brms,ndraws)
importFrom(brms,posterior_epred)
importFrom(brms,posterior_predict)
importFrom(brms,ranef)
importFrom(brms,set_prior)
importFrom(dplyr,across)
importFrom(dplyr,arrange)
importFrom(dplyr,desc)
importFrom(dplyr,everything)
importFrom(dplyr,filter)
importFrom(dplyr,group_by)
importFrom(dplyr,left_join)
importFrom(dplyr,mutate)
importFrom(dplyr,n)
importFrom(dplyr,row_number)
importFrom(dplyr,slice_head)
importFrom(dplyr,summarise)
importFrom(dplyr,where)
importFrom(forcats,fct_reorder)
importFrom(forcats,fct_rev)
importFrom(ggdist,mean_hdci)
importFrom(ggdist,median_hdi)
importFrom(ggdist,stat_slab)
importFrom(ggdist,stat_slabinterval)
importFrom(ggplot2,aes)
importFrom(ggplot2,after_stat)
importFrom(ggplot2,geom_point)
importFrom(ggplot2,geom_vline)
importFrom(ggplot2,ggplot)
importFrom(ggplot2,labs)
importFrom(ggplot2,scale_fill_brewer)
importFrom(ggplot2,scale_fill_manual)
importFrom(ggplot2,theme)
importFrom(ggplot2,theme_bw)
importFrom(grDevices,colorRampPalette)
importFrom(rlang,"!!")
importFrom(rlang,.data)
importFrom(rlang,as_name)
importFrom(rlang,enquo)
importFrom(rlang,quo_is_null)
importFrom(stats,acf)
importFrom(stats,aggregate)
importFrom(stats,as.formula)
importFrom(stats,complete.cases)
importFrom(stats,cor)
importFrom(stats,family)
importFrom(stats,formula)
importFrom(stats,mad)
importFrom(stats,median)
importFrom(stats,optimize)
importFrom(stats,plogis)
importFrom(stats,quantile)
importFrom(stats,sd)
importFrom(stats,setNames)
importFrom(stats,update)
importFrom(stats,var)
importFrom(tibble,as_tibble)
importFrom(tibble,tibble)
importFrom(tidyr,as_tibble)
importFrom(tidyr,pivot_longer)

# easyRaschBayes

<!-- badges: start -->
[![CRAN Version](https://www.r-pkg.org/badges/version/easyRaschBayes)](https://cran.r-project.org/package=easyRaschBayes)
[![Downloads](https://cranlogs.r-pkg.org/badges/easyRaschBayes?color=brightgreen)](https://CRAN.R-project.org/package=easyRaschBayes)
![Downloads Status](https://cranlogs.r-pkg.org/badges/grand-total/easyRaschBayes)
<a href="https://buymeacoffee.com/pgmj" target="_blank"><img src="https://cdn.buymeacoffee.com/buttons/default-orange.png" alt="Buy Me A Coffee" height="41" width="174"></a>
<!-- badges: end -->

**easyRaschBayes** provides functions to reproduce classic Rasch analysis
features using Bayesian item response theory (IRT) models fitted with
[brms](https://paulbuerkner.com/brms/). It supports both dichotomous
Rasch models and polytomous partial credit models, and exposes
the full posterior distribution for all output.

For more materials on Rasch analysis, see the 
[vignette](https://pgmj.github.io/raschrvignette/RaschRvign.html) for my 
(frequentist) package [`easyRasch`](https://pgmj.github.io/easyRasch/).

## Installation

Install the stable version from CRAN:

```r
install.packages("easyRaschBayes")
```

Or install the development version from GitHub:

```r
install.packages("remotes") # if you don't have `remotes` installed
remotes::install_github("pgmj/easyRaschBayes")
```

## Main functions

| Function | Description |
|---|---|
| `infit_statistic()` | Conditional infit (and outfit) statistics |
| `infit_post()` | Post-processing and plotting of infit statistics |
| `item_restscore_statistic()` | Item–rest score associations with Goodman & Kruskal's gamma |
| `item_restscore_post()` | Post-processing and plotting of item-restscore statistics |
| `plot_icc()` | Conditional item fit curves & DIF analysis |
| `plot_residual_pca()` | Residual PCA contrast plot for dimensionality assessment |
| `q3_statistic()` | Yen's Q3 residual correlations for local dependence evaluation |
| `q3_post()` | Post-processing and plotting of Q3 statistics |
| `plot_ipf()` | Item probability function curves |
| `plot_icc()` | Item category characteristics curves |
| `plot_targeting()` | Person-item map (Wright map) |
| `dif_statistic()` | Differential Item Functioning (DIF) analysis |
| `fit_statistic_pcm()` | Posterior predictive item/person fit for polytomous models |
| `fit_statistic_rm()` | Posterior predictive item/person fit for dichotomous models |
| `RMUreliability()` | Reliability via Relative Measurement Uncertainty (RMU) |
| `person_parameters()` | EAP and WLE latent scores and ordinal sum score transformation table |
| `item_parameters()` | Summarize item threshold locations |
| `plot_tile()` | Plot response data, see also `plot_bars()` and `plot_stackedbars()` |

## Usage

```r
library(brms)
library(easyRaschBayes)

# Fit a Bayesian Rasch model (dichotomous)
fit <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = my_data,
  family = bernoulli(),
  chains = 4, cores = 4
)

# Conditional item infit
infit <- infit_statistic(fit)
infit_post(infit)

# Local dependence with Yen's Q3
q3_results <- q3_statistic(fit)
q3_post(q3_results)

# Response category functioning
plot_ipf(fit)

# Person-item map
plot_targeting(fit)
```

## References

Bürkner, P.-C. (2020). Analysing Standard Progressive Matrices (SPM-LS) with 
Bayesian Item Response Models. *Journal of Intelligence*, *8*(1). 
<doi:10.3390/jintelligence8010005>

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with brms and
Stan. *Journal of Statistical Software*, *100*, 1–54.
<https://doi.org/10.18637/jss.v100.i05>

## Credits

This started as a side project to update the code to assess item fit included in Bürkner (2020).
With the help of Claude Opus 4.6 things expanded to try to 
reproduce core Rasch analysis steps. Most of the code in this package is
produced by the LLM.

[Magnus Johansson](https://ki.se/en/people/magnus-johansson-3) is a licensed psychologist with a PhD in behavior analysis. He works as a research specialist at [Karolinska Institutet](https://ki.se/en/cns/research/centre-for-psychiatry-research), Department of Clinical Neuroscience, Center for Psychiatry Research.

- ORCID: [0000-0003-1669-592X](https://orcid.org/0000-0003-1669-592X)
- Bluesky: [@pgmj.bsky.social](https://bsky.app/profile/pgmj.bsky.social) 

## License

GPL (>= 3)

#' Item Category Probability Function Curves for Polytomous IRT Models
#'
#' Plots item category probability functions (ICPFs) for polytomous
#' Bayesian IRT models fitted with \pkg{brms}. For each item, the
#' probability of endorsing each response category is plotted as a
#' function of the latent variable (theta), with separate colored
#' curves per category. All items are displayed in a combined faceted
#' plot.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from a
#'   polytomous IRT model (e.g., \code{family = acat} for a partial
#'   credit model or \code{family = cumulative} for a graded response
#'   model).
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data (e.g., \code{item}).
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data (e.g., \code{id}).
#' @param items An optional character vector of item names to plot.
#'   If \code{NULL} (the default), all items in the model are plotted.
#' @param theta_range A numeric vector of length 2 specifying the range
#'   of the latent variable (theta) for the x-axis. Default is
#'   \code{c(-4, 4)}.
#' @param n_points Integer. Number of evenly spaced theta values at
#'   which to evaluate the category probabilities. Default is 100.
#' @param ncol Integer. Number of columns in the faceted plot layout.
#'   If \code{NULL} (the default), an appropriate number is chosen
#'   automatically.
#' @param line_size Numeric. Line width for the probability curves.
#'   Default is 0.8.
#' @param ribbon_alpha Numeric in \eqn{[0, 1]}. Transparency of the
#'   credible interval ribbons. Default is 0.15. Set to 0 to hide
#'   ribbons.
#' @param prob Numeric in \eqn{(0, 1)}. Width of the credible interval
#'   for the ribbons. Default is 0.95.
#' @param category_labels An optional character vector of labels for
#'   the response categories. If \code{NULL} (the default), categories
#'   are labelled as integers starting from 1.
#' @param palette An optional character vector of colors, one per
#'   response category. If \code{NULL} (the default), the
#'   \code{viridis} discrete scale from \pkg{ggplot2} is used.
#'
#' @return A \code{\link[ggplot2]{ggplot}} object. The plot can be
#'   further customised using standard \pkg{ggplot2} functions.
#'
#' @details
#' The function computes category probabilities directly from the
#' posterior draws of the item threshold parameters. For the brms
#' \code{acat} (adjacent category / partial credit) family with
#' logit link, the density is:
#' \deqn{P(Y = y | \eta) = \frac{\exp\bigl(\sum_{k=1}^{y}(\eta -
#'   \tau_k)\bigr)}{\sum_{k=0}^{K} \exp\bigl(\sum_{j=1}^{k}(\eta -
#'   \tau_j)\bigr)}}
#' where \eqn{\eta} is the linear predictor (i.e., theta for a Rasch
#' model with no additional fixed effects) and \eqn{\tau_k} are the
#' item thresholds. Analogous formulas are used for the
#' \code{cumulative}, \code{sratio}, and \code{cratio} families.
#'
#' Posterior uncertainty in the thresholds propagates into credible
#' interval ribbons around the category probability curves — a
#' Bayesian advantage over point-estimate-based plots.
#'
#' @references
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
#' brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
#' 1--54. \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link[brms]{posterior_epred}},
#' \code{\link[brms]{conditional_effects}},
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#' library(ggplot2)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' # Plot all items
#' plot_ipf(fit_pcm, item_var = item, person_var = id)
#'
#' # Plot a subset of items
#' plot_ipf(fit_pcm, item_var = item, person_var = id,
#'          items = c("I1", "I2", "I3"))
#'
#' # Customise appearance
#' plot_ipf(fit_pcm, item_var = item, person_var = id,
#'          theta_range = c(-6, 6), ncol = 3, prob = 0.90) +
#'   theme_minimal() +
#'   labs(title = "Item Category Probability Functions")
#' }
#'
#' @importFrom brms as_draws_df
#' @importFrom rlang enquo as_name
#' @importFrom stats formula quantile plogis family
#' @importFrom tidyr as_tibble
#' @export
plot_ipf <- function(
    model,
    item_var = item,
    person_var = id,
    items = NULL,
    theta_range = c(-4, 4),
    n_points = 100,
    ncol = NULL,
    line_size = 0.8,
    ribbon_alpha = 0.15,
    prob = 0.95,
    category_labels = NULL,
    palette = NULL
) {

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required. Please install it.", call. = FALSE)
  }
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }

  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))

  # --- Extract response variable name ---
  resp_var <- as.character(formula(model)$formula[[2]])
  if (length(resp_var) > 1) {
    resp_var <- resp_var[2]
  }

  # --- Validate inputs ---
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }

  unique_items <- sort(unique(model$data[[item_name]]))

  if (!is.null(items)) {
    invalid <- setdiff(items, unique_items)
    if (length(invalid) > 0) {
      stop("Items not found in model data: ",
           paste(invalid, collapse = ", "), call. = FALSE)
    }
    unique_items <- unique_items[unique_items %in% items]
  }

  # --- Create theta grid ---
  theta_grid <- seq(theta_range[1], theta_range[2], length.out = n_points)

  # --- Extract posterior draws ---
  draws <- tidyr::as_tibble(brms::as_draws_df(model))
  n_draws_total <- nrow(draws)
  family_name <- stats::family(model)$family
  is_acat   <- grepl("acat", family_name, ignore.case = TRUE)
  is_cumul  <- grepl("cumul", family_name, ignore.case = TRUE)
  is_sratio <- grepl("sratio", family_name, ignore.case = TRUE)
  is_cratio <- grepl("cratio", family_name, ignore.case = TRUE)

  if (!any(is_acat, is_cumul, is_sratio, is_cratio)) {
    stop("This function is designed for ordinal models (acat, cumulative, ",
         "sratio, cratio). Your model uses family '", family_name, "'.",
         call. = FALSE)
  }

  # Use a subsample of draws for speed
  max_draws <- min(n_draws_total, 500)
  draw_sample <- sample(seq_len(n_draws_total), max_draws)

  lower_prob <- (1 - prob) / 2
  upper_prob <- 1 - lower_prob

  # --- Compute category probabilities per item ---
  plot_data_list <- list()

  for (item_label in unique_items) {
    # Find threshold columns for this item
    # Grouped thresholds: b_Intercept[item_label,1], etc.
    thresh_pattern <- paste0(
      "^b_Intercept\\[",
      gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
      ","
    )
    thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)

    if (length(thresh_cols) == 0) {
      # Ungrouped thresholds: b_Intercept[1], b_Intercept[2], ...
      thresh_cols <- grep("^b_Intercept\\[\\d+\\]$", names(draws),
                          value = TRUE)
    }

    if (length(thresh_cols) == 0) {
      warning("Could not find threshold parameters for item '",
              item_label, "'. Skipping.", call. = FALSE)
      next
    }

    # Sort thresholds numerically
    thresh_nums <- as.numeric(
      gsub(".*,(\\d+)\\]$|.*\\[(\\d+)\\]$", "\\1\\2", thresh_cols)
    )
    thresh_cols <- thresh_cols[order(thresh_nums)]

    n_thresholds <- length(thresh_cols)
    n_cat <- n_thresholds + 1
    thresh_sub <- as.matrix(draws[draw_sample, thresh_cols, drop = FALSE])

    # Preallocate
    prob_mean  <- matrix(NA_real_, nrow = n_points, ncol = n_cat)
    prob_lower <- matrix(NA_real_, nrow = n_points, ncol = n_cat)
    prob_upper <- matrix(NA_real_, nrow = n_points, ncol = n_cat)

    for (t in seq_along(theta_grid)) {
      theta <- theta_grid[t]
      prob_draws <- matrix(NA_real_, nrow = max_draws, ncol = n_cat)

      if (is_acat) {
        # Adjacent category model (brms formula, logit link):
        #   P(Y = y) = exp(sum_{k=1}^{y} (eta - tau_k)) /
        #              sum_{k=0}^{K} exp(sum_{j=1}^{k} (eta - tau_j))
        # where eta = theta (the person parameter)
        for (s in seq_len(max_draws)) {
          tau_s <- thresh_sub[s, ]
          # For y = 0 (first category): cumsum = 0
          # For y = c: cumsum of (theta - tau_1), ..., (theta - tau_c)
          cum_eta <- c(0, cumsum(theta - tau_s))
          # Log-sum-exp for numerical stability
          max_eta <- max(cum_eta)
          log_denom <- max_eta + log(sum(exp(cum_eta - max_eta)))
          prob_draws[s, ] <- exp(cum_eta - log_denom)
        }
      } else if (is_cumul) {
        # Cumulative model:
        #   P(Y <= c) = logistic(tau_c - eta)
        for (s in seq_len(max_draws)) {
          tau_s <- thresh_sub[s, ]
          cum_probs <- stats::plogis(tau_s - theta)
          probs <- diff(c(0, cum_probs, 1))
          probs <- pmax(probs, 0)
          prob_draws[s, ] <- probs / sum(probs)
        }
      } else if (is_sratio) {
        # Stopping ratio model:
        #   f(y) = g(tau_{y+1} - eta) * prod_{k=1}^{y} (1 - g(tau_k - eta))
        for (s in seq_len(max_draws)) {
          tau_s <- thresh_sub[s, ]
          probs <- numeric(n_cat)
          remaining <- 1
          for (cc in seq_len(n_thresholds)) {
            p_stop <- stats::plogis(tau_s[cc] - theta)
            probs[cc] <- remaining * p_stop
            remaining <- remaining * (1 - p_stop)
          }
          probs[n_cat] <- remaining
          prob_draws[s, ] <- probs
        }
      } else if (is_cratio) {
        # Continuation ratio model:
        #   f(y) = (1 - g(eta - tau_{y+1})) * prod_{k=1}^{y} g(eta - tau_k)
        for (s in seq_len(max_draws)) {
          tau_s <- thresh_sub[s, ]
          probs <- numeric(n_cat)
          remaining <- 1
          for (cc in seq_len(n_thresholds)) {
            p_cont <- stats::plogis(theta - tau_s[cc])
            probs[cc] <- remaining * (1 - p_cont)
            remaining <- remaining * p_cont
          }
          probs[n_cat] <- remaining
          prob_draws[s, ] <- probs
        }
      }

      prob_mean[t, ]  <- colMeans(prob_draws)
      prob_lower[t, ] <- apply(prob_draws, 2, stats::quantile,
                               probs = lower_prob)
      prob_upper[t, ] <- apply(prob_draws, 2, stats::quantile,
                               probs = upper_prob)
    }

    # Category labels
    if (is.null(category_labels)) {
      cat_labs <- as.character(seq_len(n_cat))
    } else {
      if (length(category_labels) != n_cat) {
        stop("Length of 'category_labels' (", length(category_labels),
             ") does not match the number of categories (", n_cat,
             ") for item '", item_label, "'.", call. = FALSE)
      }
      cat_labs <- category_labels
    }

    for (cc in seq_len(n_cat)) {
      plot_data_list[[length(plot_data_list) + 1]] <- data.frame(
        item     = item_label,
        theta    = theta_grid,
        category = cat_labs[cc],
        prob     = prob_mean[, cc],
        lower    = prob_lower[, cc],
        upper    = prob_upper[, cc],
        stringsAsFactors = FALSE
      )
    }
  }

  plot_data <- do.call(rbind, plot_data_list)
  plot_data$item <- factor(plot_data$item, levels = unique_items)
  if (is.null(category_labels)) {
    plot_data$category <- factor(
      plot_data$category,
      levels = as.character(seq_len(max(as.integer(plot_data$category))))
    )
  } else {
    plot_data$category <- factor(plot_data$category, levels = category_labels)
  }

  # --- Layout ---
  n_items <- length(unique_items)
  if (is.null(ncol)) {
    ncol <- ceiling(sqrt(n_items))
  }

  # --- Build plot ---
  p <- ggplot2::ggplot(
    plot_data,
    ggplot2::aes(
      x      = .data$theta,
      y      = .data$prob,
      colour = .data$category,
      fill   = .data$category
    )
  )

  if (ribbon_alpha > 0) {
    p <- p + ggplot2::geom_ribbon(
      ggplot2::aes(ymin = .data$lower, ymax = .data$upper),
      alpha = ribbon_alpha,
      linewidth = 0
    )
  }

  p <- p +
    ggplot2::geom_line(linewidth = line_size) +
    ggplot2::facet_wrap(~ item, ncol = ncol) +
    ggplot2::labs(
      x      = expression(theta),
      y      = "Probability",
      colour = "Category",
      fill   = "Category"
    ) +
    ggplot2::ylim(0, 1) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      legend.position = "bottom",
      strip.text       = ggplot2::element_text(face = "bold")
    )

  if (!is.null(palette)) {
    p <- p +
      ggplot2::scale_colour_manual(values = palette) +
      ggplot2::scale_fill_manual(values = palette)
  } else {
    p <- p +
      ggplot2::scale_colour_viridis_d(end = 0.9) +
      ggplot2::scale_fill_viridis_d(end = 0.9)
  }

  p
}


#' Person-Item Map (Targeting Plot) for Bayesian IRT Models
#'
#' Plots a person-item map (also known as a Wright map or targeting
#' plot) for Bayesian IRT models fitted with \pkg{brms}. The plot
#' consists of three vertically stacked panels sharing the same
#' latent variable (theta / logit) x-axis:
#'
#' \enumerate{
#'   \item \strong{Top}: A histogram of person ability estimates,
#'     with a reference line for the mean (or median) and shading
#'     for ±1 SD (or ±1 MAD).
#'   \item \strong{Middle}: An inverted histogram of item threshold
#'     locations, with a reference line for the mean (or median) and
#'     shading for ±1 SD (or ±1 MAD), mirroring the top panel to
#'     visualise the overlap between person abilities and item
#'     difficulties.
#'   \item \strong{Bottom}: A dot-and-whisker plot of item thresholds
#'     by item, with credible intervals and color-coded response
#'     categories.
#' }
#'
#' Together, the top and middle panels form a half-moon (or
#' back-to-back histogram) display that makes it easy to assess
#' whether the test is well-targeted to the sample.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from an
#'   ordinal IRT model (e.g., \code{family = acat}) or a dichotomous
#'   model (\code{family = bernoulli()}).
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data. Default is \code{id}.
#' @param robust Logical. If \code{FALSE} (the default), the
#'   histogram annotations use mean ± SD. If \code{TRUE}, median ±
#'   MAD is used instead.
#' @param center Logical. If \code{TRUE} (the default), the scale is
#'   recentered so that the grand mean of all item threshold
#'   locations is zero, following the convention in frequentist
#'   Rasch analysis. Person estimates are shifted by the same
#'   constant. If \code{FALSE}, the raw brms parameterisation is
#'   used.
#' @param sort_items Character. How to order items on the y-axis of
#'   the bottom panel. \code{"data"} (the default) preserves the
#'   order in which items first appear in the model data, with the
#'   first item at the top. \code{"location"} sorts items by their
#'   mean threshold location (easiest at top, hardest at bottom).
#' @param bins Integer. Number of bins for both histograms. Default
#'   is 30.
#' @param prob Numeric in \eqn{(0, 1)}. Width of the credible
#'   intervals for the item threshold whiskers. Default is 0.95.
#' @param palette An optional character vector of colors for the
#'   response categories. If \code{NULL} (the default), the
#'   \code{viridis} discrete scale is used.
#' @param person_fill Fill color for the person histogram. Default is
#'   \code{"#0072B2"} (blue).
#' @param threshold_fill Fill color for the threshold histogram.
#'   Default is \code{"#D55E00"} (vermillion).
#' @param height_ratios Numeric vector of length 3 specifying the
#'   relative heights of the top (person), middle (threshold), and
#'   bottom (dot-whisker) panels. Default is \code{c(3, 2, 5)}.
#'
#' @return A \code{patchwork} object (combined \code{ggplot}).
#'
#' @details
#' \strong{Person estimates} are obtained as the posterior means of
#' the person random effects from the fitted model via
#' \code{\link[brms]{ranef}}.
#'
#' \strong{Item thresholds} are extracted from the posterior draws.
#' For models with grouped thresholds (\code{thres(gr = item)}),
#' each item has its own set of threshold parameters. For models
#' with a single set of thresholds (e.g., dichotomous Rasch with
#' \code{(1 | item)}), the item random effects are subtracted from
#' the global thresholds to obtain item-specific locations.
#'
#' When \code{center = TRUE} (the default), the grand mean of all
#' item threshold posterior means is computed and subtracted from
#' every threshold estimate, its credible interval bounds, and every
#' person estimate. This is a uniform translation of the entire
#' scale that preserves all relative distances and matches the
#' zero-centered item difficulty convention used in frequentist CML
#' estimation.
#'
#' @references
#' Wright, B. D. & Stone, M. H. (1979). \emph{Best Test Design}.
#' MESA Press.
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
#' brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
#' 1--54. \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{plot_ipf}} for item category probability curves,
#' \code{\link[brms]{ranef}},
#' \code{\link[brms]{as_draws_df}}.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#' library(ggplot2)
#' library(patchwork)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' # Default: centered, mean ± SD, items in data order
#' plot_targeting(fit_pcm)
#'
#' # Uncentered (raw brms parameterisation)
#' plot_targeting(fit_pcm, center = FALSE)
#'
#' # Robust: median ± MAD, items sorted by location
#' plot_targeting(fit_pcm, robust = TRUE, sort_items = "location")
#'
#' # --- Dichotomous Rasch Model ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' plot_targeting(fit_rm, sort_items = "location")
#' }
#'
#' @importFrom brms ranef as_draws_df
#' @importFrom rlang enquo as_name .data
#' @importFrom stats formula quantile median mad sd aggregate family
#' @export
plot_targeting <- function(
    model,
    item_var = item,
    person_var = id,
    robust = FALSE,
    center = TRUE,
    sort_items = c("data", "location"),
    bins = 30,
    prob = 0.95,
    palette = NULL,
    person_fill = "#0072B2",
    threshold_fill = "#D55E00",
    height_ratios = c(3, 2, 5)
) {

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  if (!requireNamespace("patchwork", quietly = TRUE)) {
    stop("Package 'patchwork' is required.", call. = FALSE)
  }
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }

  sort_items  <- match.arg(sort_items)
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))

  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }

  lower_prob <- (1 - prob) / 2
  upper_prob <- 1 - lower_prob

  # ================================================================
  # PERSON ESTIMATES
  # ================================================================
  person_re <- brms::ranef(model)[[person_name]]
  if (is.null(person_re)) {
    stop("No random effects found for '", person_name,
         "'. Check your model formula.", call. = FALSE)
  }
  person_theta <- person_re[, "Estimate", "Intercept"]

  # ================================================================
  # ITEM THRESHOLDS
  # ================================================================
  draws <- brms::as_draws_df(model)
  unique_items_data_order <- unique(model$data[[item_name]])
  unique_items <- sort(unique(model$data[[item_name]]))
  family_name <- stats::family(model)$family
  is_ordinal <- grepl("acat|cumul|sratio|cratio",
                      family_name, ignore.case = TRUE)

  threshold_data <- .extract_threshold_data(
    draws, model, unique_items, item_name, person_name,
    is_ordinal, lower_prob, upper_prob
  )

  # ================================================================
  # CENTERING (optional)
  # ================================================================
  # The entire scale (persons AND thresholds) is shifted by the same

  # constant so that the grand mean of item thresholds = 0.
  # Both move in the same direction to preserve relative positions.
  if (center) {
    shift <- mean(threshold_data$estimate)

    threshold_data$estimate <- threshold_data$estimate - shift
    threshold_data$lower    <- threshold_data$lower - shift
    threshold_data$upper    <- threshold_data$upper - shift

    person_theta <- person_theta - shift
  }

  # ================================================================
  # SUMMARY STATISTICS (computed after centering)
  # ================================================================
  if (robust) {
    p_center <- stats::median(person_theta)
    p_spread <- stats::mad(person_theta)
    center_label <- "Median"
    spread_label <- "MAD"
  } else {
    p_center <- mean(person_theta)
    p_spread <- stats::sd(person_theta)
    center_label <- "Mean"
    spread_label <- "SD"
  }

  thresh_locations <- threshold_data$estimate
  if (robust) {
    t_center <- stats::median(thresh_locations)
    t_spread <- stats::mad(thresh_locations)
  } else {
    t_center <- mean(thresh_locations)
    t_spread <- stats::sd(thresh_locations)
  }

  # ================================================================
  # SHARED X-AXIS LIMITS
  # ================================================================
  all_values <- c(
    person_theta,
    threshold_data$estimate,
    threshold_data$lower,
    threshold_data$upper
  )
  x_range <- range(all_values, na.rm = TRUE)
  x_pad   <- diff(x_range) * 0.05
  x_lim   <- c(x_range[1] - x_pad, x_range[2] + x_pad)

  if (center) {
    x_label <- expression("Centered latent variable" ~ (theta))
  } else {
    x_label <- expression("Latent variable" ~ (theta))
  }

  # ================================================================
  # TOP PANEL: Person histogram
  # ================================================================
  person_df <- data.frame(theta = person_theta)

  p_top <- ggplot2::ggplot(person_df, ggplot2::aes(x = .data$theta)) +
    ggplot2::geom_histogram(
      bins = bins, fill = person_fill, colour = "white", alpha = 0.85
    ) +
    ggplot2::annotate(
      "rect",
      xmin = p_center - p_spread, xmax = p_center + p_spread,
      ymin = -Inf, ymax = Inf,
      fill = person_fill, alpha = 0.12
    ) +
    ggplot2::geom_vline(
      xintercept = p_center, linewidth = 0.8,
      linetype = "dashed", colour = "grey20"
    ) +
    ggplot2::annotate(
      "text",
      x = p_center, y = Inf, vjust = -0.5,
      label = paste0(center_label, " = ", round(p_center, 2),
                     ", ", spread_label, " = ", round(p_spread, 2)),
      size = 3.2, colour = "grey20"
    ) +
    ggplot2::scale_y_continuous(
      breaks = function(lim) {
        seq(0, floor(lim[2]), by = max(1, round(lim[2] / 6)))
      }
    ) +
    ggplot2::coord_cartesian(xlim = x_lim, clip = "off") +
    ggplot2::labs(x = NULL, y = "Persons") +
    ggplot2::theme_bw() +
    ggplot2::theme(
      axis.text.x  = ggplot2::element_blank(),
      axis.ticks.x = ggplot2::element_blank(),
      plot.margin  = ggplot2::margin(5, 5, 0, 5)
    )

  # ================================================================
  # MIDDLE PANEL: Inverted threshold histogram
  # ================================================================
  thresh_hist_df <- data.frame(location = thresh_locations)

  p_mid <- ggplot2::ggplot(
    thresh_hist_df, ggplot2::aes(x = .data$location)
  ) +
    ggplot2::geom_histogram(
      bins = bins, fill = threshold_fill, colour = "white", alpha = 0.85
    ) +
    ggplot2::annotate(
      "rect",
      xmin = t_center - t_spread, xmax = t_center + t_spread,
      ymin = -Inf, ymax = Inf,
      fill = threshold_fill, alpha = 0.12
    ) +
    ggplot2::geom_vline(
      xintercept = t_center, linewidth = 0.8,
      linetype = "dashed", colour = "grey20"
    ) +
    ggplot2::annotate(
      "text",
      x = t_center, y = -Inf, vjust = 1.5,
      label = paste0(center_label, " = ", round(t_center, 2),
                     ", ", spread_label, " = ", round(t_spread, 2)),
      size = 3.2, colour = "grey20"
    ) +
    ggplot2::scale_y_reverse(
      breaks = function(lim) {
        max_val <- abs(floor(lim[1]))
        seq(0, max_val, by = max(1, round(max_val / 4)))
      }
    ) +
    ggplot2::coord_cartesian(xlim = x_lim, clip = "off") +
    ggplot2::labs(x = NULL, y = "Thresholds") +
    ggplot2::theme_bw() +
    ggplot2::theme(
      axis.text.x  = ggplot2::element_blank(),
      axis.ticks.x = ggplot2::element_blank(),
      plot.margin  = ggplot2::margin(0, 5, 0, 5)
    )

  # ================================================================
  # BOTTOM PANEL: Dot-and-whisker by item
  # ================================================================
  if (sort_items == "location") {
    item_means <- stats::aggregate(
      estimate ~ item, data = threshold_data, FUN = mean
    )
    # Reverse: easiest (lowest location) at top, hardest at bottom
    item_order <- item_means$item[order(item_means$estimate)]
  } else {
    # Reverse data order: first item in data at top of y-axis
    # ggplot places the first factor level at the bottom, so we
    # reverse so that the last level (= first item) is at the top
    item_order <- rev(unique_items_data_order)
  }
  threshold_data$item <- factor(threshold_data$item, levels = item_order)

  p_bot <- ggplot2::ggplot(
    threshold_data,
    ggplot2::aes(
      x      = .data$estimate,
      y      = .data$item,
      colour = .data$category,
      xmin   = .data$lower,
      xmax   = .data$upper
    )
  ) +
    ggplot2::geom_errorbarh(
      width = 0.25, linewidth = 0.5,
      position = ggplot2::position_dodge(width = 0.4)
    ) +
    ggplot2::geom_point(
      size = 2.5,
      position = ggplot2::position_dodge(width = 0.4)
    ) +
    ggplot2::coord_cartesian(xlim = x_lim) +
    ggplot2::labs(
      x      = x_label,
      y      = NULL,
      colour = "Threshold"
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      legend.position = "bottom",
      plot.margin     = ggplot2::margin(0, 5, 5, 5)
    )

  if (!is.null(palette)) {
    p_bot <- p_bot + ggplot2::scale_colour_manual(values = palette)
  } else {
    p_bot <- p_bot + ggplot2::scale_colour_viridis_d(end = 0.9)
  }

  # ================================================================
  # COMBINE
  # ================================================================
  combined <- p_top / p_mid / p_bot +
    patchwork::plot_layout(heights = height_ratios)

  combined
}


# ── Internal helper: extract threshold data ──────────────────────
#' @keywords internal
.extract_threshold_data <- function(draws, model, unique_items,
                                    item_name, person_name,
                                    is_ordinal, lower_prob, upper_prob) {

  result_list <- list()

  if (is_ordinal) {
    has_grouped <- any(grepl("^b_Intercept\\[.+,\\d+\\]$", names(draws)))

    for (item_label in unique_items) {
      if (has_grouped) {
        thresh_pattern <- paste0(
          "^b_Intercept\\[",
          gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
          ","
        )
        thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)
      } else {
        thresh_cols <- grep("^b_Intercept\\[\\d+\\]$", names(draws),
                            value = TRUE)
      }

      if (length(thresh_cols) == 0) next

      thresh_nums <- as.numeric(
        gsub(".*,(\\d+)\\]$|.*\\[(\\d+)\\]$", "\\1\\2", thresh_cols)
      )
      thresh_cols <- thresh_cols[order(thresh_nums)]

      for (idx in seq_along(thresh_cols)) {
        vals <- draws[[thresh_cols[idx]]]
        result_list[[length(result_list) + 1]] <- data.frame(
          item     = item_label,
          category = as.character(idx),
          estimate = mean(vals),
          lower    = stats::quantile(vals, probs = lower_prob),
          upper    = stats::quantile(vals, probs = upper_prob),
          stringsAsFactors = FALSE
        )
      }
    }
  } else {
    intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
    if (length(intercept_col) == 0) {
      stop("Could not find intercept parameter.", call. = FALSE)
    }

    for (item_label in unique_items) {
      re_col <- paste0("r_", item_name, "[",  item_label, ",Intercept]")
      if (!re_col %in% names(draws)) {
        re_col_alt <- grep(
          paste0("^r_", item_name, "\\[",
                 gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
                 ",Intercept\\]$"),
          names(draws), value = TRUE
        )
        if (length(re_col_alt) == 1) {
          re_col <- re_col_alt
        } else {
          warning("Could not find random effect for item '",
                  item_label, "'. Skipping.", call. = FALSE)
          next
        }
      }

      item_location <- -(draws[[intercept_col]] + draws[[re_col]])

      result_list[[length(result_list) + 1]] <- data.frame(
        item     = item_label,
        category = "1",
        estimate = mean(item_location),
        lower    = stats::quantile(item_location, probs = lower_prob),
        upper    = stats::quantile(item_location, probs = upper_prob),
        stringsAsFactors = FALSE
      )
    }
  }

  result <- do.call(rbind, result_list)
  rownames(result) <- NULL
  result
}

#' Summarize and Plot Posterior Predictive Infit Statistics
#'
#' Postprocesses the output of \code{\link{infit_statistic}} to produce summary tables
#' of posterior predictive infit statistics and a combined slab +
#' interval plot comparing observed infit values to the posterior
#' predictive distribution.
#'
#' @param infit_draws A data frame (or tibble) as returned by
#'   \code{\link{infit_statistic}} containing at minimum the columns
#'   \code{item}, \code{infit} (observed infit per draw), and
#'   \code{infit_rep} (replicated infit per draw).
#' @param ci Numeric in \eqn{(0, 1)}. Width of the credible
#'   interval used for the posterior predictive HDI and the slab
#'   display. Default is 0.84.
#'
#' @return A list with three elements:
#' \describe{
#'   \item{summary}{A \code{\link[tibble]{tibble}} with one row per
#'     item containing: \code{item}, \code{infit_obs} (posterior
#'     mean of observed infit), \code{infit_rep} (posterior mean of
#'     replicated infit), and \code{infit_ppp} (posterior predictive
#'     p-value: proportion of draws where the replicated infit
#'     exceeds the observed infit). Values near 0.5 indicate good
#'     fit; values near 0 suggest underfit; values near 1 suggest
#'     overfit.}
#'   \item{hdi}{A \code{\link[tibble]{tibble}} with one row per
#'     item containing: \code{item}, \code{underfit} (posterior
#'     probability that the observed infit exceeds the upper HDI
#'     bound of the replicated distribution), and \code{overfit}
#'     (posterior probability that the observed infit falls below
#'     the lower HDI bound).}
#'   \item{plot}{A \code{\link[ggplot2]{ggplot}} object showing
#'     the posterior predictive distribution of replicated infit
#'     (grey filled slab) overlaid with the observed infit
#'     distribution (coloured slab + interval), with a dashed
#'     reference line at 1 (perfect fit).}
#' }
#'
#' @details
#' Two complementary summary tables are provided:
#'
#' \describe{
#'   \item{\code{summary}}{Reports the posterior mean observed and
#'     replicated infit values alongside the posterior predictive
#'     p-value (ppp). The ppp is the proportion of draws where the
#'     replicated infit exceeds the observed infit. Under good fit,
#'     the ppp should be near 0.5. A ppp near 0 indicates the
#'     observed infit is consistently larger than expected
#'     (underfit); a ppp near 1 indicates it is consistently
#'     smaller (overfit).}
#'   \item{\code{hdi}}{Reports the probability that the observed
#'     infit falls above (underfit) or below (overfit) the HDI of
#'     the replicated distribution. This provides a more
#'     distributional assessment than the ppp alone.}
#' }
#'
#' The plot uses two layers from the \pkg{ggdist} package:
#' \describe{
#'   \item{\code{stat_slab}}{Displays the posterior predictive
#'     (replicated) infit distribution as a filled density slab
#'     per item, shaded by credible interval level.}
#'   \item{\code{stat_slabinterval}}{Displays the observed infit
#'     distribution per item as a semi-transparent slab with
#'     point and interval summaries.}
#' }
#'
#' Under good model fit, the observed infit distribution should
#' overlap substantially with the replicated distribution. Items
#' where the observed distribution sits systematically above the
#' replicated HDI indicate underfit (more variation than expected);
#' items below indicate overfit (less variation than expected).
#'
#' @seealso
#' \code{\link{infit_statistic}},
#' \code{\link{item_restscore_post}}.
#'
#' @examples
#' \dontrun{
#' library(brms)
#' library(ggplot2)
#'
#' # Assuming fit_pcm is a fitted brmsfit object
#' infit_draws <- infit_statistic(fit_pcm)
#'
#' result <- infit_post(infit_draws)
#' result$summary
#' result$hdi
#' result$plot
#' }
#'
#' @importFrom dplyr group_by summarise left_join mutate
#' @importFrom ggdist stat_slab stat_slabinterval median_hdi
#' @importFrom ggplot2 ggplot aes after_stat geom_vline labs
#'   theme_bw theme scale_fill_manual scale_fill_brewer
#' @importFrom forcats fct_rev
#' @importFrom rlang .data
#' @export
infit_post <- function(infit_draws, ci = 0.84) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop("Package 'ggdist' is required.", call. = FALSE)
  }
  if (!all(c("item", "infit", "infit_rep") %in% names(infit_draws))) {
    stop("'infit_draws' must contain columns 'item', 'infit', ",
         "and 'infit_rep'.", call. = FALSE)
  }
  
  # --- Summary table: means and ppp ---
  summary_tbl <- infit_draws |>
    dplyr::group_by(.data$item) |>
    dplyr::summarise(
      infit_obs = round(mean(.data$infit), 3),
      infit_rep = round(mean(.data$infit_rep), 3),
      infit_ppp = round(mean(.data$infit_rep > .data$infit), 3),
      .groups = "drop"
    )
  
  # --- HDI table: over/underfit probabilities ---
  infit_rep_hdi <- infit_draws |>
    dplyr::group_by(.data$item) |>
    dplyr::summarise(
      ggdist::median_hdi(.data$infit_rep, .width = ci),
      .groups = "drop"
    )
  
  hdi_tbl <- infit_draws |>
    dplyr::left_join(
      infit_rep_hdi[, c("item", "ymin", "ymax")],
      by = "item"
    ) |>
    dplyr::group_by(.data$item) |>
    dplyr::summarise(
      underfit = round(mean(.data$infit > .data$ymax), 3),
      overfit  = round(mean(.data$infit < .data$ymin), 3),
      .groups = "drop"
    )
  
  # --- Plot ---
  p <- ggplot2::ggplot(
    infit_draws,
    ggplot2::aes(x = .data$infit, y = forcats::fct_rev(.data$item))
  ) +
    ggdist::stat_slab(
      ggplot2::aes(x = .data$infit_rep,
                   fill = ggplot2::after_stat(level)),
      .width = c(ci, 0.95)
    ) +
    ggdist::stat_slabinterval(
      ggplot2::aes(slab_fill = ggplot2::after_stat(level)),
      .width = c(ci, 0.95),
      alpha = 0.8
    ) +
    ggplot2::scale_fill_manual(
      aesthetics = "fill",
      values = c("darkgrey", "lightgrey")
    ) +
    ggplot2::scale_fill_brewer(
      aesthetics = "slab_fill",
      na.value = "lightblue1"
    ) +
    ggplot2::geom_vline(
      xintercept = 1, linetype = 2
    ) +
    ggplot2::labs(
      x = "Observed (blue) and expected conditional item infit",
      y = NULL
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.position = "none")
  
  list(summary = summary_tbl, hdi = hdi_tbl, plot = p)
}


#' Summarize and Plot Posterior Predictive Item-Restscore Associations
#'
#' Postprocesses the output of \code{\link{item_restscore_statistic}}
#' to produce a summary table and a slab plot comparing observed
#' item-restscore gamma associations to the posterior predictive
#' distribution.
#'
#' @param item_restscore A list as returned by
#'   \code{\link{item_restscore_statistic}}, containing at minimum:
#'   \describe{
#'     \item{result}{A data frame with columns including \code{item}
#'       and summary statistics (first 5 columns are used).}
#'     \item{draws}{A data frame with columns \code{item},
#'       \code{gamma} (observed gamma per draw), and
#'       \code{gamma_rep} (replicated gamma per draw).}
#'   }
#'
#' @return A list with two elements:
#' \describe{
#'   \item{summary}{A \code{\link[tibble]{tibble}} with the first 5
#'     columns of the result table, rounded to 3 decimal places and
#'     sorted by item.}
#'   \item{plot}{A \code{\link[ggplot2]{ggplot}} object showing
#'     the posterior predictive distribution of replicated gamma
#'     (grey filled slab) with the observed gamma values overlaid
#'     as orange diamond points.}
#' }
#'
#' @details
#' The item-restscore gamma association measures the strength of
#' the relationship between each item's responses and the rest
#' score (total score excluding that item). Under good fit, the
#' observed gamma should fall within the posterior predictive
#' distribution.
#'
#' The plot displays:
#' \describe{
#'   \item{Grey slab}{The posterior predictive distribution of
#'     replicated gamma values, shaded by 84\% and 95\% credible
#'     interval levels.}
#'   \item{Orange diamonds}{The observed gamma values per draw,
#'     plotted as points on top of the slab.}
#' }
#'
#' Items where the observed gamma (orange) falls consistently
#' outside the replicated distribution (grey) indicate poor fit
#' in terms of item discrimination.
#'
#' @seealso
#' \code{\link{item_restscore_statistic}},
#' \code{\link{infit_post}}.
#'
#' @examples
#' \dontrun{
#' library(brms)
#' library(ggplot2)
#'
#' # Assuming fit_pcm is a fitted brmsfit object
#' irs <- item_restscore_statistic(fit_pcm)
#'
#' result <- item_restscore_post(irs)
#' result$summary
#' result$plot
#' }
#'
#' @importFrom dplyr mutate across where arrange
#' @importFrom ggdist stat_slab
#' @importFrom ggplot2 ggplot aes after_stat geom_point labs
#'   theme_bw theme scale_fill_manual scale_fill_brewer
#' @importFrom forcats fct_rev
#' @export
item_restscore_post <- function(item_restscore) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop("Package 'ggdist' is required.", call. = FALSE)
  }
  if (!is.list(item_restscore) ||
      !all(c("result", "draws") %in% names(item_restscore))) {
    stop("'item_restscore' must be a list with elements 'result' ",
         "and 'draws'.", call. = FALSE)
  }
  if (!all(c("item", "gamma", "gamma_rep") %in%
           names(item_restscore$draws))) {
    stop("'item_restscore$draws' must contain columns 'item', ",
         "'gamma', and 'gamma_rep'.", call. = FALSE)
  }
  
  # --- Summary table ---
  summary_tbl <- item_restscore$result[, 1:5] |>
    dplyr::mutate(
      dplyr::across(dplyr::where(is.numeric), ~ round(.x, 3))
    ) |>
    dplyr::arrange(.data$item)
  
  # --- Plot ---
  draws_df <- item_restscore$draws
  
  p <- ggplot2::ggplot(
    draws_df,
    ggplot2::aes(x = .data$gamma, y = forcats::fct_rev(.data$item))
  ) +
    ggdist::stat_slab(
      ggplot2::aes(x = .data$gamma_rep,
                   fill = ggplot2::after_stat(level)),
      .width = c(0.84, 0.95)
    ) +
    ggplot2::geom_point(
      size = 6, shape = 18, colour = "sienna1",
      position = ggplot2::position_nudge(y = 0.08)
    ) +
    ggplot2::scale_fill_manual(
      aesthetics = "fill",
      values = c("grey", "lightgrey")
    ) +
    ggplot2::scale_fill_brewer(
      aesthetics = "slab_fill",
      na.value = "lightblue1"
    ) +
    ggplot2::labs(
      x = "Observed (orange) and expected item-restscore gamma",
      y = NULL
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.position = "none")
  
  list(summary = summary_tbl, plot = p)
}

#' Summarize and Plot Posterior Predictive Q3 Residual Correlations
#'
#' Postprocesses the output of \code{\link{q3_statistic}} to produce
#' summary tables of posterior predictive Q3 statistics and a combined
#' slab + interval plot comparing observed Q3 residual correlations
#' to the posterior predictive distribution for each item pair.
#'
#' @param q3_draws A data frame (or tibble) as returned by
#'   \code{\link{q3_statistic}} containing at minimum the columns
#'   \code{item_pair}, \code{q3} (observed Q3 per draw), and
#'   \code{q3_rep} (replicated Q3 per draw).
#' @param ci Numeric in \eqn{(0, 1)}. Width of the credible
#'   interval used for the posterior predictive HDI and the slab
#'   display. Default is 0.84.
#' @param n_pairs Integer. Maximum number of item pairs to display
#'   in the plot, selected by largest absolute \code{q3_diff}. If
#'   \code{NULL} (the default), all pairs are shown. Useful when
#'   the number of item pairs is large.
#' @param sort_by Character. How to order item pairs on the y-axis.
#'   \code{"q3_diff"} (the default) sorts by the posterior mean
#'   difference between observed and replicated Q3 (largest at top).
#'   \code{"q3_obs"} sorts by the posterior mean observed Q3.
#'   \code{"ppp"} sorts by the posterior predictive p-value.
#'
#' @return A list with three elements:
#' \describe{
#'   \item{summary}{A \code{\link[tibble]{tibble}} with one row per
#'     item pair containing: \code{item_pair}, \code{item_1},
#'     \code{item_2}, \code{q3_obs} (posterior mean observed Q3),
#'     \code{q3_rep} (posterior mean replicated Q3),
#'     \code{q3_diff} (posterior mean difference), and \code{q3_ppp}
#'     (posterior predictive p-value: proportion of draws where the
#'     observed Q3 exceeds the replicated Q3).}
#'   \item{hdi}{A \code{\link[tibble]{tibble}} with one row per
#'     item pair containing: \code{item_pair}, \code{item_1},
#'     \code{item_2}, \code{ld} (local dependence probability:
#'     proportion of draws where observed Q3 exceeds upper HDI
#'     bound of replicated distribution), and \code{lr} (local
#'     repulsion probability: proportion of draws where observed
#'     Q3 falls below lower HDI bound).}
#'   \item{plot}{A \code{\link[ggplot2]{ggplot}} object showing
#'     the posterior predictive distribution of replicated Q3
#'     (grey filled slab) overlaid with the observed Q3
#'     distribution (coloured slab + interval), with a dashed
#'     reference line at 0.}
#' }
#'
#' @details
#' Two complementary summary tables are provided:
#'
#' \describe{
#'   \item{\code{summary}}{Reports posterior mean observed and
#'     replicated Q3 values alongside the posterior predictive
#'     p-value (ppp). The ppp is the proportion of draws where
#'     the observed Q3 exceeds the replicated Q3. Under good fit
#'     (no local dependence), the ppp should be near 0.5. A ppp
#'     close to 1 indicates the observed correlation is
#'     systematically higher than expected (local dependence);
#'     a ppp close to 0 indicates it is systematically lower
#'     (local repulsion, e.g., speed-accuracy tradeoffs).}
#'   \item{\code{hdi}}{Reports the probability that the observed
#'     Q3 falls above (local dependence) or below (local repulsion)
#'     the HDI of the replicated distribution. This provides a
#'     more distributional assessment than the ppp alone.}
#' }
#'
#' The plot uses two layers from the \pkg{ggdist} package:
#' \describe{
#'   \item{\code{stat_slab}}{Displays the posterior predictive
#'     (replicated) Q3 distribution as a filled density slab per
#'     item pair, shaded by credible interval level.}
#'   \item{\code{stat_slabinterval}}{Displays the observed Q3
#'     distribution per item pair as a semi-transparent slab with
#'     point and interval summaries.}
#' }
#'
#' @seealso
#' \code{\link{q3_statistic}},
#' \code{\link{infit_post}},
#' \code{\link{item_restscore_post}}.
#'
#' @examples
#' \dontrun{
#' library(brms)
#' library(ggplot2)
#'
#' # Assuming fit_pcm is a fitted brmsfit object
#' q3_draws <- q3_statistic(fit_pcm, ndraws_use = 500)
#'
#' result <- q3_post(q3_draws)
#' result$summary
#' result$hdi
#' result$plot
#'
#' # Show only top 10 pairs by Q3 difference
#' result_top <- q3_post(q3_draws, n_pairs = 10)
#' result_top$plot
#' }
#'
#' @importFrom dplyr group_by summarise left_join mutate arrange
#'   desc slice_head filter
#' @importFrom ggdist stat_slab stat_slabinterval median_hdi
#' @importFrom ggplot2 ggplot aes after_stat geom_vline labs
#'   theme_bw theme scale_fill_manual scale_fill_brewer
#' @importFrom forcats fct_rev fct_reorder
#' @importFrom rlang .data
#' @importFrom tibble tibble
#' @export
q3_post <- function(q3_draws,
                    ci = 0.84,
                    n_pairs = NULL,
                    sort_by = c("q3_diff", "q3_obs", "ppp")) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  if (!requireNamespace("ggdist", quietly = TRUE)) {
    stop("Package 'ggdist' is required.", call. = FALSE)
  }
  if (!all(c("item_pair", "q3", "q3_rep") %in% names(q3_draws))) {
    stop("'q3_draws' must contain columns 'item_pair', 'q3', ",
         "and 'q3_rep'.", call. = FALSE)
  }
  
  sort_by <- match.arg(sort_by)
  
  # --- Summary table: means and ppp ---
  summary_tbl <- q3_draws |>
    dplyr::group_by(.data$item_pair) |>
    dplyr::summarise(
      item_1  = .data$item_1[1],
      item_2  = .data$item_2[1],
      q3_obs  = round(mean(.data$q3, na.rm = TRUE), 3),
      q3_rep  = round(mean(.data$q3_rep, na.rm = TRUE), 3),
      q3_diff = round(mean(.data$q3 - .data$q3_rep, na.rm = TRUE), 3),
      q3_ppp  = round(mean(.data$q3 > .data$q3_rep, na.rm = TRUE), 3),
      .groups = "drop"
    ) |>
    dplyr::arrange(dplyr::desc(.data$q3_diff))
  
  # --- HDI table: LD / LR probabilities ---
  q3_rep_hdi <- q3_draws |>
    dplyr::group_by(.data$item_pair) |>
    dplyr::summarise(
      ggdist::median_hdi(.data$q3_rep, .width = ci),
      .groups = "drop"
    )
  
  hdi_tbl <- q3_draws |>
    dplyr::left_join(
      q3_rep_hdi[, c("item_pair", "ymin", "ymax")],
      by = "item_pair"
    ) |>
    dplyr::group_by(.data$item_pair) |>
    dplyr::summarise(
      item_1 = .data$item_1[1],
      item_2 = .data$item_2[1],
      ld     = round(mean(.data$q3 > .data$ymax, na.rm = TRUE), 3),
      lr     = round(mean(.data$q3 < .data$ymin, na.rm = TRUE), 3),
      .groups = "drop"
    ) |>
    dplyr::arrange(dplyr::desc(.data$ld))
  
  # --- Determine which pairs to plot ---
  if (sort_by == "q3_diff") {
    sort_var <- abs(summary_tbl$q3_diff)
  } else if (sort_by == "q3_obs") {
    sort_var <- abs(summary_tbl$q3_obs)
  } else {
    sort_var <- summary_tbl$q3_ppp
  }
  pair_order <- summary_tbl$item_pair[order(sort_var, decreasing = TRUE)]
  
  if (!is.null(n_pairs)) {
    n_pairs <- min(as.integer(n_pairs), length(pair_order))
    pairs_to_plot <- pair_order[seq_len(n_pairs)]
  } else {
    pairs_to_plot <- pair_order
  }
  
  plot_data <- q3_draws |>
    dplyr::filter(.data$item_pair %in% pairs_to_plot) |>
    dplyr::mutate(
      item_pair = factor(.data$item_pair, levels = rev(pairs_to_plot))
    )
  
  # --- Plot ---
  p <- ggplot2::ggplot(
    plot_data,
    ggplot2::aes(x = .data$q3, y = .data$item_pair)
  ) +
    ggdist::stat_slab(
      ggplot2::aes(x = .data$q3_rep,
                   fill = ggplot2::after_stat(level)),
      .width = c(ci, 0.95)
    ) +
    ggdist::stat_slabinterval(
      ggplot2::aes(slab_fill = ggplot2::after_stat(level)),
      .width = c(ci, 0.95),
      alpha = 0.8
    ) +
    ggplot2::scale_fill_manual(
      aesthetics = "fill",
      values = c("darkgrey", "lightgrey")
    ) +
    ggplot2::scale_fill_brewer(
      aesthetics = "slab_fill",
      na.value = "lightblue1"
    ) +
    ggplot2::geom_vline(
      xintercept = 0, linetype = 2
    ) +
    ggplot2::labs(
      x = "Observed and expected Q3 residual correlation",
      y = NULL
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(legend.position = "none")
  
  list(summary = summary_tbl, hdi = hdi_tbl, plot = p)
}

#' Extract Informative Priors from a Fitted Bayesian IRT Model
#'
#' Takes a fitted \code{\link[brms]{brmsfit}} object and constructs a
#' \code{\link[brms]{brmsprior}} object in which each item parameter
#' receives a \code{normal(mean, sd)} prior derived from its posterior
#' distribution. The person-level random effect SD prior is also
#' updated. The returned prior can be passed directly to
#' \code{\link[stats]{update}} (or \code{\link[brms]{brm}}) to refit
#' the model with empirical Bayes / informative priors — useful for
#' anchoring scales, warm-starting a model on new data, or
#' regularising estimation with small samples.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object. Supported
#'   parameterisations:
#'   \describe{
#'     \item{Polytomous ordinal}{e.g., \code{family = acat} with
#'       \code{thres(gr = item)}, producing item-specific thresholds.}
#'     \item{Dichotomous Rasch (random items)}{e.g.,
#'       \code{response ~ 1 + (1 | item) + (1 | id)} with
#'       \code{family = bernoulli()}.}
#'     \item{Dichotomous 1PL (fixed items)}{e.g.,
#'       \code{response ~ 0 + item + (1 | id)} with
#'       \code{family = bernoulli()}.}
#'   }
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data. Default is \code{id}.
#' @param mult Numeric multiplier applied to each posterior SD before
#'   it is used as the prior SD. Values > 1 widen the priors
#'   (less informative); values < 1 tighten them. Default is 1
#'   (use posterior SD directly).
#' @param target_link Character string specifying the link function of
#'   the model the priors will be used with. One of \code{"logit"},
#'   \code{"probit"}, or \code{"source"} (the default). When
#'   \code{"source"}, the link function of the fitted model is used
#'   and no transformation is applied. When different from the source
#'   model's link, all location and scale parameters are rescaled
#'   using the approximation \eqn{\beta_{\text{probit}} \approx
#'   \beta_{\text{logit}} / 1.7}. This is useful when transferring
#'   priors from a logit-fitted model to a probit model or vice versa.
#'
#' @return A \code{\link[brms]{brmsprior}} object that can be supplied
#'   to the \code{prior} argument of \code{\link[brms]{brm}} or
#'   \code{\link[stats]{update}}.
#'
#' @details
#' The function extracts all posterior draws via
#' \code{\link[brms]{as_draws_df}}, computes the mean and SD of each
#' parameter's marginal posterior, and constructs
#' \code{normal(mean, sd * mult)} priors.
#'
#' \strong{Polytomous ordinal models} with grouped thresholds
#' (\code{thres(gr = item)}): each threshold receives its own prior
#' via \code{brms::set_prior("normal(...)", class = "Intercept",
#' group = item, coef = threshold_index)}.
#'
#' \strong{Dichotomous Rasch models} parameterised as
#' \code{response ~ 1 + (1 | item) + (1 | id)}: priors are set on
#' the global intercept (\code{class = "Intercept"}), the item-level
#' SD (\code{class = "sd", group = item_var}), and the person-level
#' SD.
#'
#' \strong{Dichotomous 1PL models} parameterised as
#' \code{response ~ 0 + item + (1 | id)}: each item-specific fixed
#' effect (e.g., \code{b_itemI1}) receives its own
#' \code{normal(mean, sd)} prior via
#' \code{brms::set_prior(..., class = "b", coef = "itemI1")}.
#'
#' In all cases the person-level SD receives a
#' \code{normal(mean, sd * mult)} prior (brms applies the lower
#' bound of zero automatically for SD parameters).
#'
#' \strong{Link function transformation:} When \code{target_link}
#' differs from the source model's link function, all parameters
#' (means and SDs) are rescaled by a factor of approximately 1.7.
#' This uses the well-known approximation that
#' \eqn{\Phi(x) \approx \text{logistic}(1.7 \, x)}, so
#' logit-scale parameters can be converted to probit-scale by
#' dividing by 1.7, and vice versa. The approximation is excellent
#' for parameters in the range \eqn{|\beta| < 3} and adequate
#' beyond that range.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4, cores = 2, iter = 1000 # use more iter (and cores if you have)
#' )
#'
#' # Extract posterior-informed priors (same link)
#' new_priors <- posterior_to_prior(fit_pcm)
#' new_priors
#'
#' # Narrow the prior's sd by a factor of 0.5
#' wide_priors <- posterior_to_prior(fit_pcm, mult = 0.5)
#'
#' # Extract priors for use with a probit model
#' probit_priors <- posterior_to_prior(fit_pcm, target_link = "probit")
#'
#' # --- Dichotomous 1PL (fixed item effects) ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_1pl <- brm(
#'   response ~ 0 + item + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4, cores = 2, iter = 1000 # use more iter (and cores if you have)
#' )
#'
#' priors_1pl <- posterior_to_prior(fit_1pl)
#' priors_1pl
#'
#' # Transfer logit priors to a probit refit
#' priors_probit <- posterior_to_prior(fit_1pl, target_link = "probit")
#' }
#'
#' @importFrom brms as_draws_df set_prior
#' @importFrom rlang enquo as_name
#' @importFrom stats sd family
#' @importFrom tibble as_tibble
#' @export
posterior_to_prior <- function(model,
                               item_var = item,
                               person_var = id,
                               mult = 1,
                               target_link = c("source", "logit", "probit")) {
  
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  if (!is.numeric(mult) || length(mult) != 1 || mult <= 0) {
    stop("'mult' must be a single positive number.", call. = FALSE)
  }
  
  target_link <- match.arg(target_link)
  
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))
  
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  draws       <- tibble::as_tibble(brms::as_draws_df(model))
  family_name <- stats::family(model)$family
  is_ordinal  <- grepl("acat|cumul|sratio|cratio",
                       family_name, ignore.case = TRUE)
  
  # ================================================================
  # DETERMINE LINK TRANSFORMATION FACTOR
  # ================================================================
  source_link <- stats::family(model)$link
  if (is.null(source_link) || source_link == "") {
    source_link <- "logit"  # brms default for bernoulli/acat
  }
  
  if (target_link == "source") {
    link_factor <- 1
  } else if (source_link == target_link) {
    link_factor <- 1
  } else if (source_link == "logit" && target_link == "probit") {
    link_factor <- 1 / 1.7
  } else if (source_link == "probit" && target_link == "logit") {
    link_factor <- 1.7
  } else {
    warning("Link transformation from '", source_link, "' to '",
            target_link, "' is not supported. No rescaling applied.",
            call. = FALSE)
    link_factor <- 1
  }
  
  prior_list <- list()
  
  # ==================================================================
  # ORDINAL MODELS (grouped thresholds: thres(gr = item))
  # ==================================================================
  if (is_ordinal) {
    has_grouped <- any(grepl("^b_Intercept\\[.+,\\d+\\]$", names(draws)))
    
    if (!has_grouped) {
      stop("Ordinal model does not appear to use grouped thresholds ",
           "(thres(gr = ...)). This function currently requires ",
           "item-specific thresholds.", call. = FALSE)
    }
    
    unique_items <- sort(unique(model$data[[item_name]]))
    
    for (item_label in unique_items) {
      thresh_pattern <- paste0(
        "^b_Intercept\\[",
        gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
        ","
      )
      thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)
      
      if (length(thresh_cols) == 0) {
        warning("No threshold parameters found for item '", item_label,
                "'. Skipping.", call. = FALSE)
        next
      }
      
      # Sort numerically
      thresh_nums <- as.numeric(
        gsub(".*,(\\d+)\\]$", "\\1", thresh_cols)
      )
      thresh_cols <- thresh_cols[order(thresh_nums)]
      
      for (idx in seq_along(thresh_cols)) {
        vals   <- draws[[thresh_cols[idx]]]
        p_mean <- mean(vals) * link_factor
        p_sd   <- stats::sd(vals) * mult * abs(link_factor)
        
        prior_list[[length(prior_list) + 1]] <- brms::set_prior(
          paste0("normal(", round(p_mean, 4), ", ", round(p_sd, 4), ")"),
          class = "Intercept",
          coef  = as.character(idx),
          group = item_label
        )
      }
    }
    
    # ==================================================================
    # DICHOTOMOUS MODELS
    # ==================================================================
  } else {
    
    # --- Detect parameterisation ---
    fe_pattern <- paste0("^b_", item_name)
    fe_cols    <- grep(fe_pattern, names(draws), value = TRUE)
    fe_cols    <- setdiff(fe_cols, "b_Intercept")
    
    has_fixed_items  <- length(fe_cols) > 0
    has_random_items <- any(grepl(
      paste0("^r_", item_name, "\\["), names(draws)
    ))
    
    # ── Fixed item effects: response ~ 0 + item + (1 | id) ──
    if (has_fixed_items) {
      
      for (col_name in fe_cols) {
        vals   <- draws[[col_name]]
        p_mean <- mean(vals) * link_factor
        p_sd   <- stats::sd(vals) * mult * abs(link_factor)
        
        coef_name <- sub("^b_", "", col_name)
        
        prior_list[[length(prior_list) + 1]] <- brms::set_prior(
          paste0("normal(", round(p_mean, 4), ", ", round(p_sd, 4), ")"),
          class = "b",
          coef  = coef_name
        )
      }
      
      # ── Random item effects: response ~ 1 + (1 | item) + (1 | id) ──
    } else if (has_random_items) {
      
      # Global intercept
      intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
      if (length(intercept_col) == 1) {
        vals   <- draws[[intercept_col]]
        p_mean <- mean(vals) * link_factor
        p_sd   <- stats::sd(vals) * mult * abs(link_factor)
        
        prior_list[[length(prior_list) + 1]] <- brms::set_prior(
          paste0("normal(", round(p_mean, 4), ", ", round(p_sd, 4), ")"),
          class = "Intercept"
        )
      }
      
      # Item-level SD
      item_sd_col <- grep(
        paste0("^sd_", item_name, "__Intercept$"),
        names(draws), value = TRUE
      )
      if (length(item_sd_col) == 1) {
        vals   <- draws[[item_sd_col]]
        p_mean <- mean(vals) * abs(link_factor)
        p_sd   <- stats::sd(vals) * mult * abs(link_factor)
        
        prior_list[[length(prior_list) + 1]] <- brms::set_prior(
          paste0("normal(", round(p_mean, 4), ", ", round(p_sd, 4), ")"),
          class = "sd",
          group = item_name
        )
      }
      
    } else {
      stop("Could not detect item parameters in the model. ",
           "Supported dichotomous parameterisations:\n",
           "  - response ~ 0 + item + (1 | id)    [fixed items]\n",
           "  - response ~ 1 + (1 | item) + (1 | id)  [random items]",
           call. = FALSE)
    }
  }
  
  # ==================================================================
  # PERSON-LEVEL SD (common to all parameterisations)
  # ==================================================================
  person_sd_col <- grep(
    paste0("^sd_", person_name, "__Intercept$"),
    names(draws), value = TRUE
  )
  if (length(person_sd_col) == 1) {
    vals   <- draws[[person_sd_col]]
    p_mean <- mean(vals) * abs(link_factor)
    p_sd   <- stats::sd(vals) * mult * abs(link_factor)
    
    prior_list[[length(prior_list) + 1]] <- brms::set_prior(
      paste0("normal(", round(p_mean, 4), ", ", round(p_sd, 4), ")"),
      class = "sd",
      group = person_name
    )
  }
  
  # ==================================================================
  # COMBINE INTO A SINGLE brmsprior OBJECT
  # ==================================================================
  if (length(prior_list) == 0) {
    stop("Could not extract any parameters from the model.",
         call. = FALSE)
  }
  
  combined_prior <- prior_list[[1]]
  if (length(prior_list) > 1) {
    for (i in 2:length(prior_list)) {
      combined_prior <- combined_prior + prior_list[[i]]
    }
  }
  
  if (link_factor != 1) {
    message("Priors rescaled from '", source_link, "' to '",
            target_link, "' link (factor = ",
            round(link_factor, 4), ").")
  }
  
  combined_prior
}

#' Differential Item Functioning (DIF) Analysis for Bayesian IRT Models
#'
#' Tests for differential item functioning (DIF) in Bayesian
#' Rasch-family models fitted with \pkg{brms} by comparing item
#' parameters across subgroups defined by an exogenous variable.
#' The function fits a DIF model that includes group-by-item
#' interactions and summarizes the posterior distribution of the
#' DIF effects.
#'
#' For polytomous models, two types of DIF can be tested:
#' \describe{
#'   \item{Uniform DIF (\code{dif_type = "uniform"}, default)}{
#'     A single location shift per item across groups, modelled as
#'     a \code{group:item} fixed-effect interaction. This tests
#'     whether the average item difficulty differs between groups.}
#'   \item{Non-uniform / threshold-level DIF
#'     (\code{dif_type = "non-uniform"})}{Each item receives
#'     group-specific thresholds via
#'     \code{thres(gr = interaction(item, group))}. DIF effects
#'     are computed as the difference in each threshold between
#'     groups, revealing whether DIF affects specific response
#'     categories.}
#' }
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from
#'   the baseline (no-DIF) model.
#' @param group_var An unquoted variable name identifying the
#'   grouping variable for DIF testing (e.g., \code{gender}).
#'   Must be a factor or character variable with exactly 2 levels
#'   in the current implementation.
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the
#'   person grouping variable. Default is \code{id}.
#' @param data An optional data frame containing all variables
#'   needed for the DIF model, including the group variable. If
#'   \code{NULL} (the default), the function attempts to use
#'   \code{model$data}. Since the baseline model formula typically
#'   does not include the group variable, brms will have dropped
#'   it from the stored model data. In that case, you must supply
#'   the original data frame here.
#' @param dif_type Character. For polytomous ordinal models only.
#'   \code{"uniform"} (the default) tests for a uniform location
#'   shift per item via a \code{group:item} fixed-effect
#'   interaction. \code{"non-uniform"} fits group-specific
#'   thresholds per item and computes per-threshold DIF effects
#'   as the difference between groups. Ignored for dichotomous
#'   models.
#' @param prob Numeric in \eqn{(0, 1)}. Width of the credible
#'   intervals. Default is 0.95.
#' @param rope Numeric. Half-width of the Region of Practical
#'   Equivalence (ROPE) around zero for DIF effects, on the logit
#'   scale. Default is 0.5, corresponding to a practically
#'   negligible DIF effect. Set to 0 to skip ROPE analysis.
#' @param refit Logical. If \code{TRUE} (the default), the DIF
#'   model is fitted automatically by updating the baseline model
#'   via \code{\link[stats]{update}}, which reuses the compiled
#'   Stan code for faster sampling. If \code{FALSE}, only the DIF
#'   model formula is returned (useful for manual fitting with
#'   custom settings).
#' @param ... Additional arguments passed to
#'   \code{\link[brms]{update.brmsfit}} when refitting the DIF
#'   model (e.g., \code{cores}, \code{control}).
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{summary}{A \code{\link[tibble]{tibble}} with one row per
#'     item (for uniform DIF) or per item × threshold (for
#'     non-uniform DIF) containing: \code{item}, optionally
#'     \code{threshold}, \code{dif_estimate} (posterior mean),
#'     \code{dif_lower}, \code{dif_upper} (credible interval),
#'     \code{dif_sd} (posterior SD), \code{pd} (probability of
#'     direction), \code{rope_percentage} (proportion inside ROPE),
#'     and \code{flag} (classification).}
#'   \item{dif_draws}{A matrix of posterior draws for the DIF
#'     effects (draws × effects), for further analysis.}
#'   \item{dif_model}{The fitted DIF \code{brmsfit} object (if
#'     \code{refit = TRUE}), or \code{NULL}.}
#'   \item{dif_formula}{The \code{brmsformula} used for the DIF
#'     model.}
#'   \item{baseline_model}{The original baseline model.}
#'   \item{plot}{A \code{\link[ggplot2]{ggplot}} forest plot of
#'     DIF effects with credible intervals and ROPE.}
#' }
#'
#' @details
#' The function constructs a DIF model by adding a group-by-item
#' interaction to the baseline model:
#'
#' \itemize{
#'   \item \strong{Dichotomous models}
#'     (\code{family = bernoulli()}): The baseline
#'     \code{response ~ 1 + (1 | item) + (1 | id)} becomes
#'     \code{response ~ 1 + group + (1 + group | item) + (1 | id)},
#'     where the group slope varying by item captures item-specific
#'     DIF.
#'   \item \strong{Polytomous uniform DIF}
#'     (\code{dif_type = "uniform"}): The baseline
#'     \code{response | thres(gr = item) ~ 1 + (1 | id)} becomes
#'     \code{response | thres(gr = item) ~ 1 + group:item + (1 | id)}.
#'   \item \strong{Polytomous non-uniform DIF}
#'     (\code{dif_type = "non-uniform"}): The baseline becomes
#'     \code{response | thres(gr = item_group) ~ 1 + (1 | id)},
#'     where \code{item_group = interaction(item, group)}. Each
#'     item × group combination gets its own thresholds. DIF
#'     effects are the differences between group-specific
#'     thresholds for each item, computed draw-by-draw from the
#'     posterior.
#' }
#'
#' DIF effects are summarized using:
#' \describe{
#'   \item{Probability of Direction (pd)}{The proportion of the
#'     posterior on the dominant side of zero. Values > 0.975
#'     indicate strong directional evidence.}
#'   \item{ROPE}{The Region of Practical Equivalence (Kruschke,
#'     2018). If > 95\% of the posterior falls inside the ROPE,
#'     the DIF effect is practically negligible. If > 95\% falls
#'     outside, the effect is practically significant.}
#'   \item{Credible Interval}{If the CI excludes zero, there is
#'     evidence of DIF at the specified credibility level.}
#' }
#'
#' @references
#' Kruschke, J. K. (2018). Rejecting or accepting parameter values
#' in Bayesian estimation. \emph{Advances in Methods and Practices
#' in Psychological Science}, \emph{1}(2), 270--280.
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
#' brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
#' 1--54. \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{infit_statistic}} for item fit,
#' \code{\link{q3_statistic}} for local dependence,
#' \code{\link[brms]{brm}},
#' \code{\link[brms]{hypothesis}}.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Dichotomous Rasch with DIF testing ---
#'
#' set.seed(123)
#' df <- expand.grid(id = 1:200, item = paste0("I", 1:10)) %>%
#'   mutate(
#'     gender = rep(sample(c("M", "F"), 200, TRUE), each = 10),
#'     theta  = rep(rnorm(200), each = 10),
#'     delta  = rep(seq(-2, 2, length.out = 10), 200),
#'     dif    = ifelse(item == "I3" & gender == "F", 1.0,
#'              ifelse(item == "I7" & gender == "F", -0.8, 0)),
#'     p      = plogis(theta - delta - dif),
#'     response = rbinom(n(), 1, p)
#'   )
#'
#' fit_base <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df,
#'   family = bernoulli(),
#'   chains = 4, 
#'   cores  = 2, # use more cores if you have
#'   iter   = 1000 # use at least 2000 
#' )
#'
#' dif_result <- dif_statistic(
#'   model     = fit_base,
#'   group_var = gender,
#'   data      = df
#' )
#'
#' dif_result$summary
#' dif_result$plot
#'
#' # --- Partial Credit Model: uniform DIF ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   mutate(gender = sample(c("M", "F"), n(), TRUE)) %>%
#'   pivot_longer(!c(id, gender),
#'                names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4, 
#'   cores  = 2, # use more cores if you have
#'   iter   = 1000 # use at least 2000 
#' )
#'
#' # Uniform DIF (default): one shift per item
#' dif_uni <- dif_statistic(fit_pcm, group_var = gender, data = df_pcm)
#' dif_uni$plot
#'
#' # Non-uniform DIF: threshold-level effects
#' dif_nu <- dif_statistic(fit_pcm, group_var = gender, data = df_pcm,
#'                          dif_type = "non-uniform")
#' dif_nu$summary
#' dif_nu$plot
#' }
#'
#' @importFrom brms brm bf as_draws_df ndraws
#' @importFrom rlang enquo as_name .data
#' @importFrom stats formula quantile sd update family as.formula
#' @importFrom tibble tibble
#' @export
dif_statistic <- function(
    model,
    group_var,
    item_var = item,
    person_var = id,
    data = NULL,
    dif_type = c("uniform", "non-uniform"),
    prob = 0.95,
    rope = 0.5,
    refit = TRUE,
    ...
) {

  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }

  dif_type    <- match.arg(dif_type)
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))
  group_name  <- rlang::as_name(rlang::enquo(group_var))

  # --- Resolve data source ---
  if (is.null(data)) {
    dif_data <- model$data
  } else {
    dif_data <- as.data.frame(data)
  }

  if (!item_name %in% names(dif_data)) {
    stop("Item variable '", item_name, "' not found in data.",
         call. = FALSE)
  }
  if (!person_name %in% names(dif_data)) {
    stop("Person variable '", person_name, "' not found in data.",
         call. = FALSE)
  }
  if (!group_name %in% names(dif_data)) {
    stop("Group variable '", group_name, "' not found in data. ",
         "If '", group_name, "' was not part of the baseline model ",
         "formula, brms will have dropped it from model$data. ",
         "Please supply the original data frame via the 'data' ",
         "argument.", call. = FALSE)
  }

  # Ensure group is factor
  if (!is.factor(dif_data[[group_name]])) {
    dif_data[[group_name]] <- as.factor(dif_data[[group_name]])
  }
  group_vals   <- dif_data[[group_name]]
  group_levels <- levels(group_vals)
  if (length(group_levels) != 2) {
    stop("'", group_name, "' must have exactly 2 levels. Found: ",
         length(group_levels), ".", call. = FALSE)
  }

  lower_prob <- (1 - prob) / 2
  upper_prob <- 1 - lower_prob

  family_name  <- stats::family(model)$family
  is_ordinal   <- grepl("acat|cumul|sratio|cratio",
                        family_name, ignore.case = TRUE)
  unique_items <- unique(dif_data[[item_name]])

  # ================================================================
  # CONSTRUCT DIF FORMULA
  # ================================================================
  base_formula <- formula(model)$formula
  resp_var <- as.character(base_formula[[2]])
  if (length(resp_var) > 1) resp_var <- resp_var[2]

  if (is_ordinal) {
    resp_text <- deparse(base_formula[[2]], width.cutoff = 500)
    rhs_text  <- deparse(base_formula[[3]], width.cutoff = 500)

    if (dif_type == "uniform") {
      dif_formula_text <- paste0(
        resp_text, " ~ ", rhs_text, " + ",
        group_name, ":", item_name
      )
      dif_formula <- brms::bf(stats::as.formula(dif_formula_text))
    } else {
      # Non-uniform: create item_group interaction for thres(gr = ...)
      ig_var <- ".item_group_dif"
      dif_data[[ig_var]] <- interaction(
        dif_data[[item_name]], dif_data[[group_name]],
        drop = TRUE, sep = "___"
      )

      dif_formula_text <- paste0(
        resp_var, " | thres(gr = ", ig_var, ") ~ 1 + (1 | ",
        person_name, ")"
      )
      dif_formula <- brms::bf(stats::as.formula(dif_formula_text))
    }
  } else {
    # Dichotomous: always random slope
    dif_formula <- brms::bf(stats::as.formula(paste0(
      deparse(base_formula[[2]], width.cutoff = 500),
      " ~ 1 + ", group_name,
      " + (1 + ", group_name, " | ", item_name, ")",
      " + (1 | ", person_name, ")"
    )))
  }

  # ================================================================
  # FIT DIF MODEL
  # ================================================================
  dif_model <- NULL

  if (refit) {
    dif_model <- stats::update(
      model,
      formula. = dif_formula,
      newdata  = dif_data,
      ...
    )
  }

  if (is.null(dif_model) && !refit) {
    message("DIF formula constructed but model not fitted ",
            "(refit = FALSE).")
    message("Formula: ",
            deparse(dif_formula$formula, width.cutoff = 500))
    return(list(
      summary        = NULL,
      dif_draws      = NULL,
      dif_model      = NULL,
      dif_formula    = dif_formula,
      baseline_model = model,
      plot           = NULL
    ))
  }

  # ================================================================
  # EXTRACT DIF EFFECTS
  # ================================================================
  draws <- brms::as_draws_df(dif_model)
  group_level <- group_levels[2]

  if (is_ordinal && dif_type == "uniform") {
    dif_info <- .extract_dif_uniform(
      draws, unique_items, item_name, group_name, group_level
    )
  } else if (is_ordinal && dif_type == "non-uniform") {
    dif_info <- .extract_dif_threshold(
      draws, unique_items, group_levels
    )
  } else {
    dif_info <- .extract_dif_dichotomous(
      draws, unique_items, item_name, group_name, group_level
    )
  }

  dif_draws_mat <- dif_info$draws_mat
  effect_labels <- dif_info$labels

  # ================================================================
  # SUMMARIZE DIF EFFECTS
  # ================================================================
  n_effects <- ncol(dif_draws_mat)

  dif_mean  <- colMeans(dif_draws_mat, na.rm = TRUE)
  dif_sd    <- apply(dif_draws_mat, 2, stats::sd, na.rm = TRUE)
  dif_lower <- apply(dif_draws_mat, 2, stats::quantile,
                     probs = lower_prob, na.rm = TRUE)
  dif_upper <- apply(dif_draws_mat, 2, stats::quantile,
                     probs = upper_prob, na.rm = TRUE)

  pd <- apply(dif_draws_mat, 2, function(x) {
    p_pos <- mean(x > 0, na.rm = TRUE)
    max(p_pos, 1 - p_pos)
  })

  if (rope > 0) {
    rope_pct <- apply(dif_draws_mat, 2, function(x) {
      mean(abs(x) <= rope, na.rm = TRUE)
    })
  } else {
    rope_pct <- rep(NA_real_, n_effects)
  }

  flag <- character(n_effects)
  for (i in seq_len(n_effects)) {
    ci_excludes_zero <- (dif_lower[i] > 0) | (dif_upper[i] < 0)
    if (rope > 0) {
      mostly_outside_rope <- rope_pct[i] < 0.05
      mostly_inside_rope  <- rope_pct[i] > 0.95
    } else {
      mostly_outside_rope <- FALSE
      mostly_inside_rope  <- FALSE
    }

    if (ci_excludes_zero && mostly_outside_rope) {
      flag[i] <- "DIF detected"
    } else if (ci_excludes_zero) {
      flag[i] <- "DIF likely"
    } else if (mostly_inside_rope) {
      flag[i] <- "Negligible"
    } else {
      flag[i] <- "Inconclusive"
    }
  }

  summary_df <- tibble::tibble(
    item            = effect_labels$item,
    dif_estimate    = dif_mean,
    dif_sd          = dif_sd,
    dif_lower       = dif_lower,
    dif_upper       = dif_upper,
    pd              = pd,
    rope_percentage = rope_pct,
    flag            = flag
  )

  if ("threshold" %in% names(effect_labels)) {
    summary_df$threshold <- effect_labels$threshold
    summary_df <- summary_df[, c("item", "threshold",
                                 setdiff(names(summary_df),
                                         c("item", "threshold")))]
  }

  # ================================================================
  # FOREST PLOT
  # ================================================================
  has_threshold <- "threshold" %in% names(summary_df)

  if (has_threshold) {
    summary_df$label <- paste0(summary_df$item, " [t",
                               summary_df$threshold, "]")
    summary_df$label <- factor(
      summary_df$label,
      levels = rev(unique(summary_df$label))
    )
    y_var <- "label"

    # Map thresholds to shapes
    unique_thresh <- sort(unique(summary_df$threshold))
    shape_vals <- c(16, 17, 15, 18, 8, 3, 4, 1, 2, 0)
    shape_vals <- shape_vals[seq_along(unique_thresh)]
    names(shape_vals) <- unique_thresh
    summary_df$threshold_f <- factor(summary_df$threshold,
                                     levels = unique_thresh)
  } else {
    summary_df$item_f <- factor(
      summary_df$item,
      levels = summary_df$item[order(summary_df$dif_estimate)]
    )
    y_var <- "item_f"
  }

  flag_colors <- c(
    "DIF detected"  = "#D55E00",
    "DIF likely"    = "#E69F00",
    "Inconclusive"  = "#56B4E9",
    "Negligible"    = "#009E73"
  )

  p <- ggplot2::ggplot(
    summary_df,
    ggplot2::aes(
      x      = .data$dif_estimate,
      y      = .data[[y_var]],
      colour = .data$flag
    )
  )

  # Alternating item background bands
  if (has_threshold) {
    band_items <- unique(summary_df$item)
    label_levels <- levels(summary_df$label)
    for (b in seq_along(band_items)) {
      if (b %% 2 == 0) {
        rows <- summary_df$item == band_items[b]
        positions <- match(summary_df$label[rows], label_levels)
        ymin_b <- min(positions) - 0.5
        ymax_b <- max(positions) + 0.5
        p <- p +
          ggplot2::annotate(
            "rect",
            xmin = -Inf, xmax = Inf,
            ymin = ymin_b, ymax = ymax_b,
            fill = "grey95", alpha = 0.6
          )
      }
    }
  } else {
    item_levels <- levels(summary_df$item_f)
    for (b in seq_along(item_levels)) {
      if (b %% 2 == 0) {
        p <- p +
          ggplot2::annotate(
            "rect",
            xmin = -Inf, xmax = Inf,
            ymin = b - 0.5, ymax = b + 0.5,
            fill = "grey95", alpha = 0.6
          )
      }
    }
  }

  if (rope > 0) {
    p <- p +
      ggplot2::annotate(
        "rect",
        xmin = -rope, xmax = rope,
        ymin = -Inf, ymax = Inf,
        fill = "grey85", alpha = 0.4
      )
  }

  p <- p +
    ggplot2::geom_vline(
      xintercept = 0, linetype = "dashed", colour = "grey50"
    ) +
    ggplot2::geom_linerange(
      ggplot2::aes(xmin = .data$dif_lower, xmax = .data$dif_upper),
      linewidth = 0.5
    )

  if (has_threshold) {
    p <- p +
      ggplot2::geom_point(
        ggplot2::aes(shape = .data$threshold_f),
        size = 2.5
      ) +
      ggplot2::scale_shape_manual(
        values = shape_vals,
        name   = "Threshold"
      )
  } else {
    p <- p +
      ggplot2::geom_point(size = 2.5)
  }

  p <- p +
    ggplot2::scale_colour_manual(
      values = flag_colors,
      name   = "DIF status"
    ) +
    ggplot2::labs(
      x = paste0("DIF effect (", group_levels[2],
                 " vs. ", group_levels[1], ")"),
      y = NULL,
      subtitle = if (rope > 0) {
        paste0(
          if (has_threshold) "Threshold-level DIF | " else "",
          "ROPE = [", -rope, ", ", rope,
          "], shaded region = negligible DIF"
        )
      } else {
        if (has_threshold) "Threshold-level DIF" else NULL
      }
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      panel.background = ggplot2::element_rect(fill = "white"),
      plot.background  = ggplot2::element_rect(fill = "white",
                                               colour = NA),
      legend.position  = "bottom"
    )

  # Clean up temporary plot columns
  summary_df$label       <- NULL
  summary_df$item_f      <- NULL
  summary_df$threshold_f <- NULL

  list(
    summary        = summary_df,
    dif_draws      = dif_draws_mat,
    dif_model      = dif_model,
    dif_formula    = dif_formula,
    baseline_model = model,
    plot           = p
  )
}


# ── Internal: extract uniform DIF (fixed interaction) ────────────
#' @keywords internal
.extract_dif_uniform <- function(draws, unique_items, item_name,
                                 group_name, group_level) {
  n_draws <- nrow(draws)
  k <- length(unique_items)
  dif_draws_mat <- matrix(0, nrow = n_draws, ncol = k)
  colnames(dif_draws_mat) <- unique_items

  for (i in seq_along(unique_items)) {
    item_label <- unique_items[i]
    item_esc <- gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label)

    patterns <- c(
      paste0("^b_", group_name, group_level, ":",
             item_name, item_label, "$"),
      paste0("^b_", item_name, item_label, ":",
             group_name, group_level, "$"),
      paste0("^b_", group_name, group_level, ":",
             item_name, item_esc, "$"),
      paste0("^b_", item_name, item_esc, ":",
             group_name, group_level, "$")
    )

    col_found <- NULL
    for (pat in patterns) {
      candidates <- grep(pat, names(draws), value = TRUE)
      if (length(candidates) == 1) {
        col_found <- candidates
        break
      }
    }

    if (!is.null(col_found)) {
      dif_draws_mat[, i] <- draws[[col_found]]
    }
  }

  list(
    draws_mat = dif_draws_mat,
    labels    = data.frame(item = unique_items,
                           stringsAsFactors = FALSE)
  )
}


# ── Internal: extract threshold-level DIF ────────────────────────
# For non-uniform DIF with thres(gr = interaction(item, group)):
# Each item × group gets its own thresholds. DIF = difference
# between group-specific thresholds for each item, per threshold.
#
# brms names these: b_Intercept[ItemLabel___GroupLevel,ThreshNum]
#' @keywords internal
.extract_dif_threshold <- function(draws, unique_items, group_levels) {

  n_draws <- nrow(draws)
  sep <- "___"

  # Find all threshold columns
  thresh_cols <- grep("^b_Intercept\\[", names(draws), value = TRUE)

  if (length(thresh_cols) == 0) {
    stop("Could not find threshold parameters in DIF model.",
         call. = FALSE)
  }

  # Parse: b_Intercept[ItemLabel___GroupLevel,ThreshNum]
  parsed <- data.frame(
    col   = thresh_cols,
    inner = gsub("^b_Intercept\\[(.+)\\]$", "\\1", thresh_cols),
    stringsAsFactors = FALSE
  )
  parsed$ig_label <- sub(",\\d+$", "", parsed$inner)
  parsed$thresh   <- as.integer(sub("^.+,(\\d+)$", "\\1", parsed$inner))

  # Split ig_label into item and group
  parsed$item  <- sub(paste0(sep, ".*$"), "", parsed$ig_label)
  parsed$group <- sub(paste0("^.*", sep), "", parsed$ig_label)

  # Separate by group
  items_g1 <- parsed[parsed$group == group_levels[1], ]
  items_g2 <- parsed[parsed$group == group_levels[2], ]

  # Build DIF draws: for each item × threshold, group2 minus group1
  result_items  <- character(0)
  result_thresh <- character(0)
  result_draws  <- list()

  for (item_label in unique_items) {
    cols_g1 <- items_g1[items_g1$item == item_label, ]
    cols_g2 <- items_g2[items_g2$item == item_label, ]

    if (nrow(cols_g1) == 0 || nrow(cols_g2) == 0) next

    cols_g1 <- cols_g1[order(cols_g1$thresh), ]
    cols_g2 <- cols_g2[order(cols_g2$thresh), ]

    n_thresh <- min(nrow(cols_g1), nrow(cols_g2))

    for (t in seq_len(n_thresh)) {
      dif_draw <- draws[[cols_g2$col[t]]] - draws[[cols_g1$col[t]]]
      result_items  <- c(result_items, item_label)
      result_thresh <- c(result_thresh, as.character(t))
      result_draws[[length(result_draws) + 1]] <- dif_draw
    }
  }

  if (length(result_draws) == 0) {
    stop("Could not compute threshold-level DIF effects. ",
         "Check that thres(gr = ...) produced item x group ",
         "threshold parameters.", call. = FALSE)
  }

  dif_draws_mat <- do.call(cbind, result_draws)
  colnames(dif_draws_mat) <- paste0(result_items, "_t", result_thresh)

  list(
    draws_mat = dif_draws_mat,
    labels    = data.frame(
      item      = result_items,
      threshold = result_thresh,
      stringsAsFactors = FALSE
    )
  )
}


# ── Internal: extract dichotomous DIF (random slopes) ────────────
#' @keywords internal
.extract_dif_dichotomous <- function(draws, unique_items, item_name,
                                     group_name, group_level) {
  n_draws <- nrow(draws)
  k <- length(unique_items)
  dif_draws_mat <- matrix(0, nrow = n_draws, ncol = k)
  colnames(dif_draws_mat) <- unique_items

  for (i in seq_along(unique_items)) {
    item_label <- unique_items[i]
    item_esc <- gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label)

    re_pattern <- paste0(
      "^r_", item_name, "\\[", item_esc,
      ",", group_name, group_level, "\\]$"
    )
    re_col <- grep(re_pattern, names(draws), value = TRUE)

    if (length(re_col) == 1) {
      dif_draws_mat[, i] <- draws[[re_col]]
    } else {
      re_col2 <- grep(
        paste0("^r_", item_name, "\\[", item_esc, ",.*",
               group_name, ".*\\]$"),
        names(draws), value = TRUE
      )
      if (length(re_col2) == 1) {
        dif_draws_mat[, i] <- draws[[re_col2]]
      }
    }
  }

  list(
    draws_mat = dif_draws_mat,
    labels    = data.frame(item = unique_items,
                           stringsAsFactors = FALSE)
  )
}

#' Item Characteristic Curves with Class Intervals
#'
#' Plots Item Characteristic Curves (ICCs) for Bayesian Rasch-family
#' models fitted with \pkg{brms}. Each item panel shows the
#' model-expected item score curve (with a credible interval ribbon)
#' overlaid with observed average item scores computed within class
#' intervals. Optionally, observed scores can be split by a grouping
#' variable to visually assess differential item functioning (DIF).
#'
#' This is the Bayesian analogue of \code{ICCplot()} from the
#' \pkg{iarm} package, using the class interval method.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from
#'   an ordinal IRT model (e.g., \code{family = acat}) or a
#'   dichotomous model (\code{family = bernoulli()}).
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the
#'   person grouping variable in the model data. Default is
#'   \code{id}.
#' @param items An optional character vector of item names to plot.
#'   If \code{NULL} (the default), all items are plotted.
#' @param n_intervals Integer. The number of class intervals into
#'   which persons are binned along the sum score. Default is 5.
#' @param theta_range A numeric vector of length 2 specifying the
#'   range of theta for the expected curve. Default is
#'   \code{c(-4, 4)}.
#' @param n_points Integer. Number of evenly spaced theta values
#'   for computing the expected curve. Default is 200.
#' @param center Logical. If \code{TRUE} (the default), the scale
#'   is recentered so the grand mean of item thresholds = 0,
#'   consistent with \code{\link{plot_targeting}}.
#' @param prob Numeric in \eqn{(0, 1)}. Width of the credible
#'   interval ribbon around the expected curve. Default is 0.95.
#' @param ncol Integer. Number of columns in the faceted layout.
#'   If \code{NULL}, chosen automatically.
#' @param line_size Numeric. Line width for the expected curve.
#'   Default is 0.8.
#' @param ribbon_alpha Numeric in \eqn{[0, 1]}. Transparency of
#'   the credible interval ribbon. Default is 0.3.
#' @param point_size Numeric. Size of observed score points.
#'   Default is 2.5.
#' @param dif_var An optional unquoted variable name for a grouping
#'   variable to assess DIF visually. If supplied, observed scores
#'   are computed separately per group and coloured accordingly.
#' @param dif_data An optional data frame containing the DIF
#'   variable. Required when \code{dif_var} is specified and the
#'   variable was not part of the model formula (since brms drops
#'   unused variables from \code{model$data}). Must have the same
#'   rows and row order as the original model data.
#' @param dif_labels An optional character vector of labels for
#'   the DIF groups. If \code{NULL}, factor levels are used.
#' @param dif_stats Logical. If \code{TRUE} (the default when
#'   \code{dif_var} is specified), the partial gamma coefficient
#'   for each item is annotated in the plot panel. The partial
#'   gamma measures the strength of DIF, stratified by total score.
#' @param min_n Integer. Minimum number of persons required in a
#'   class interval for the observed mean to be plotted. Intervals
#'   with fewer persons are dropped to avoid unstable estimates.
#'   Default is 5.
#' @param palette An optional character vector of colors. If
#'   \code{NULL}, viridis colors are used.
#'
#' @return A \code{\link[ggplot2]{ggplot}} object.
#'
#' @details
#' \strong{Expected curve:} For each item, the expected item score
#' \eqn{E_i(\theta)} is computed for a dense grid of theta values
#' using the posterior draws of item thresholds. For the adjacent
#' category (PCM/acat) family:
#' \deqn{E_i(\theta) = \sum_{c=0}^{K} c \cdot P_{ic}(\theta)}
#' where \eqn{P_{ic}} are the category probabilities. For
#' dichotomous items, \eqn{E_i(\theta) = P_i(\theta)}.
#'
#' The posterior mean of \eqn{E_i(\theta)} is plotted as a line,
#' with a credible interval ribbon showing the \code{prob} interval
#' across posterior draws.
#'
#' \strong{Class intervals:} Following the approach in
#' \code{iarm::ICCplot()}, persons are binned into class intervals
#' by their ordinal \emph{sum score} (the sufficient statistic in
#' Rasch models), not by their EAP theta estimate. Within each
#' interval, the mean observed item response is computed and
#' plotted at the theta value corresponding to the mean sum score
#' in that interval, obtained by inverting the posterior-mean
#' expected total score function. Persons with extreme sum scores
#' (0 and maximum) are excluded from the class intervals, as their
#' theta positions cannot be estimated.
#'
#' \strong{Uncertainty around observed points:} For each class
#' interval, the standard error of the mean observed response is
#' computed and displayed as error bars showing \eqn{\pm 1.96} SE.
#' These reflect sampling variability of the observed mean within
#' each bin.
#'
#' \strong{DIF overlay:} When \code{dif_var} is specified, observed
#' scores are computed separately for each level of the DIF
#' variable, producing group-specific points connected by lines.
#' Deviations between groups that track alongside each other (but
#' offset from the expected curve) suggest uniform DIF. Crossing
#' group lines suggest non-uniform DIF.
#'
#' \strong{Partial gamma:} When \code{dif_stats = TRUE} (default
#' when DIF is requested), the Goodman-Kruskal partial gamma
#' coefficient is displayed in each item panel. This measures the
#' ordinal association between item response and DIF group,
#' stratified by total score. Values near 0 indicate no DIF;
#' values near \eqn{\pm 1} indicate strong DIF. The computation
#' reuses the concordant/discordant pair counting algorithm from
#' \code{\link{gk_gamma}}.
#'
#' @seealso
#' \code{\link{plot_ipf}} for category probability curves,
#' \code{\link{plot_targeting}} for person-item maps,
#' \code{\link{dif_statistic}} for formal Bayesian DIF testing,
#' \code{\link{gk_gamma}} for the underlying gamma algorithm.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#' library(ggplot2)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4, cores = 2, iter = 1000 # use more iter and cores
#' )
#'
#' # Basic ICC plot with 5 class intervals
#' plot_icc(fit_pcm)
#'
#' # Select specific items, more intervals
#' plot_icc(fit_pcm, items = c("I1", "I2"), n_intervals = 5)
#'
#' # With DIF overlay and partial gamma annotation
#' 
#' # same dataset, adding a `gender` DIF variable
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   mutate(gender = sample(c("M", "F"), nrow(.), TRUE)) %>% 
#'   pivot_longer(!c(id,gender), names_to = "item", values_to = "response")
#'   
#' plot_icc(fit_pcm, dif_var = gender, dif_data = df_pcm,
#'          dif_labels = c("Female", "Male"), n_intervals = 5)
#' }
#'
#' @importFrom brms as_draws_df ranef
#' @importFrom rlang enquo as_name .data quo_is_null
#' @importFrom stats formula quantile plogis family sd aggregate
#'   as.formula complete.cases
#' @importFrom tibble as_tibble
#' @export
plot_icc <- function(
    model,
    item_var     = item,
    person_var   = id,
    items        = NULL,
    n_intervals  = 5,
    theta_range  = c(-4, 4),
    n_points     = 200,
    center       = TRUE,
    prob         = 0.95,
    ncol         = NULL,
    line_size    = 0.8,
    ribbon_alpha = 0.3,
    point_size   = 2.5,
    dif_var      = NULL,
    dif_data     = NULL,
    dif_labels   = NULL,
    dif_stats    = TRUE,
    min_n        = 5,
    palette      = NULL
) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))
  
  # --- DIF variable handling ---
  dif_quo  <- rlang::enquo(dif_var)
  has_dif  <- !rlang::quo_is_null(dif_quo)
  dif_name <- if (has_dif) rlang::as_name(dif_quo) else NULL
  
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  resp_var <- as.character(stats::formula(model)$formula[[2]])
  if (length(resp_var) > 1) resp_var <- resp_var[2]
  
  model_data   <- model$data
  unique_items <- sort(unique(model_data[[item_name]]))
  if (!is.null(items)) {
    invalid <- setdiff(items, unique_items)
    if (length(invalid) > 0) {
      stop("Items not found in model data: ",
           paste(invalid, collapse = ", "), call. = FALSE)
    }
    unique_items <- unique_items[unique_items %in% items]
  }
  all_unique_items <- sort(unique(model_data[[item_name]]))
  
  # ================================================================
  # EXTRACT POSTERIOR DRAWS & DETECT MODEL TYPE
  # ================================================================
  draws       <- tibble::as_tibble(brms::as_draws_df(model))
  family_name <- stats::family(model)$family
  is_ordinal  <- grepl("acat|cumul|sratio|cratio",
                       family_name, ignore.case = TRUE)
  is_acat     <- grepl("acat", family_name, ignore.case = TRUE)
  is_cumul    <- grepl("cumul", family_name, ignore.case = TRUE)
  is_sratio   <- grepl("sratio", family_name, ignore.case = TRUE)
  is_cratio   <- grepl("cratio", family_name, ignore.case = TRUE)
  is_binary   <- grepl("bernoulli|binomial",
                       family_name, ignore.case = TRUE)
  
  if (!is_ordinal && !is_binary) {
    stop("Unsupported model family '", family_name,
         "'. Expected acat, cumulative, sratio, cratio, or ",
         "bernoulli.", call. = FALSE)
  }
  
  # ================================================================
  # CENTERING SHIFT
  # ================================================================
  lower_prob <- (1 - prob) / 2
  upper_prob <- 1 - lower_prob
  
  if (center) {
    if (is_ordinal) {
      all_thresh_means <- c()
      for (il in all_unique_items) {
        tc <- .find_thresh_cols(draws, il)
        if (length(tc) > 0) {
          all_thresh_means <- c(
            all_thresh_means,
            vapply(tc, function(col) mean(draws[[col]]), numeric(1))
          )
        }
      }
      shift <- mean(all_thresh_means)
    } else {
      item_locs <- vapply(all_unique_items, function(il) {
        loc_draws <- .get_item_location_draws_all(
          draws, il, item_name
        )
        mean(loc_draws)
      }, numeric(1))
      shift <- mean(item_locs)
    }
  } else {
    shift <- 0
  }
  
  # ================================================================
  # RESPONSE ADJUSTMENT FOR ORDINAL MODELS
  # ================================================================
  min_resp <- min(model_data[[resp_var]], na.rm = TRUE)
  
  # ================================================================
  # COMPUTE EXPECTED TOTAL SCORE CURVE E(R|theta) FOR CLASS INTERVALS
  # ================================================================
  theta_grid_fine <- seq(theta_range[1], theta_range[2],
                         length.out = 1000)
  
  e_total_score <- rep(0, length(theta_grid_fine))
  
  for (item_label in all_unique_items) {
    if (is_ordinal) {
      thresh_cols <- .find_thresh_cols(draws, item_label)
      if (length(thresh_cols) == 0) next
      tau_mean <- vapply(
        thresh_cols, function(col) mean(draws[[col]]), numeric(1)
      )
      tau_mean <- tau_mean - shift
      n_cat <- length(tau_mean) + 1
      cats  <- seq(0, n_cat - 1)
      
      for (t in seq_along(theta_grid_fine)) {
        p_cat <- .compute_cat_probs(
          theta_grid_fine[t], tau_mean,
          is_acat, is_cumul, is_sratio, is_cratio
        )
        e_total_score[t] <- e_total_score[t] + sum(cats * p_cat)
      }
    } else {
      loc_mean <- mean(.get_item_location_draws_all(
        draws, item_label, item_name
      ))
      loc_mean <- loc_mean - shift
      e_total_score <- e_total_score +
        stats::plogis(theta_grid_fine - loc_mean)
    }
  }
  
  # ================================================================
  # COMPUTE SUM SCORES AND BUILD PERSON-LEVEL DATA
  # ================================================================
  sum_scores_df <- stats::aggregate(
    stats::as.formula(paste(resp_var, "~", person_name)),
    data = model_data,
    FUN  = sum, na.rm = TRUE
  )
  colnames(sum_scores_df) <- c("person_id", "sum_score")
  
  if (min_resp > 0) {
    n_obs_per <- stats::aggregate(
      stats::as.formula(paste(resp_var, "~", person_name)),
      data = model_data, FUN = length
    )
    colnames(n_obs_per) <- c("person_id", "n_obs")
    sum_scores_df <- merge(sum_scores_df, n_obs_per, by = "person_id")
    sum_scores_df$sum_score <- sum_scores_df$sum_score -
      (min_resp * sum_scores_df$n_obs)
    sum_scores_df$n_obs <- NULL
  }
  
  if (is_ordinal) {
    n_thresholds <- vapply(all_unique_items, function(il) {
      length(.find_thresh_cols(draws, il))
    }, integer(1))
    max_score <- sum(n_thresholds)
  } else {
    max_score <- length(all_unique_items)
  }
  
  # ================================================================
  # CLASS INTERVALS (following iarm approach)
  # ================================================================
  person_ci <- sum_scores_df[
    sum_scores_df$sum_score > 0 & sum_scores_df$sum_score < max_score,
  ]
  
  if (nrow(person_ci) == 0) {
    stop("No persons with non-extreme sum scores. ",
         "Cannot create class intervals.", call. = FALSE)
  }
  
  breaks <- stats::quantile(
    person_ci$sum_score,
    probs = seq(0, 1, length.out = n_intervals + 1)
  )
  breaks <- unique(breaks)
  n_actual_intervals <- length(breaks) - 1
  
  if (n_actual_intervals < 2) {
    warning("Could not create enough distinct class intervals. ",
            "Try reducing n_intervals.", call. = FALSE)
  }
  
  person_ci$interval <- cut(
    person_ci$sum_score, breaks = breaks,
    include.lowest = TRUE, labels = FALSE
  )
  
  interval_mean_score <- stats::aggregate(
    sum_score ~ interval, data = person_ci, FUN = mean
  )
  
  interval_mean_score$theta <- vapply(
    interval_mean_score$sum_score,
    function(s) {
      idx <- which.min(abs(e_total_score - s))
      theta_grid_fine[idx]
    },
    numeric(1)
  )
  
  person_ci <- merge(
    person_ci,
    interval_mean_score[, c("interval", "theta")],
    by = "interval"
  )
  
  # ================================================================
  # DIF GROUPING VARIABLE
  # ================================================================
  if (has_dif) {
    if (!is.null(dif_data)) {
      dif_person <- data.frame(
        person_id  = as.character(dif_data[[person_name]]),
        .dif_group = as.character(dif_data[[dif_name]]),
        stringsAsFactors = FALSE
      )
      dif_person <- dif_person[!duplicated(dif_person$person_id), ]
    } else if (dif_name %in% names(model_data)) {
      dif_person <- data.frame(
        person_id  = as.character(model_data[[person_name]]),
        .dif_group = as.character(model_data[[dif_name]]),
        stringsAsFactors = FALSE
      )
      dif_person <- dif_person[!duplicated(dif_person$person_id), ]
    } else {
      stop("DIF variable '", dif_name, "' not found. ",
           "Supply 'dif_data' containing the variable.",
           call. = FALSE)
    }
    
    person_ci <- merge(person_ci, dif_person,
                       by = "person_id", all.x = TRUE)
    
    if (!is.null(dif_labels)) {
      old_levels <- sort(unique(person_ci$.dif_group))
      if (length(dif_labels) != length(old_levels)) {
        warning("Length of dif_labels (", length(dif_labels),
                ") doesn't match number of DIF groups (",
                length(old_levels), "). Using original labels.",
                call. = FALSE)
      } else {
        person_ci$.dif_group <- factor(
          person_ci$.dif_group,
          levels = old_levels, labels = dif_labels
        )
      }
    }
  }
  
  # ================================================================
  # COMPUTE PARTIAL GAMMA FOR DIF (if requested)
  # ================================================================
  gamma_annotations <- NULL
  
  if (has_dif && dif_stats) {
    wide_responses <- .build_wide_responses(
      model_data, resp_var, item_name, person_name,
      all_unique_items, min_resp, is_ordinal
    )
    
    wide_responses <- merge(
      wide_responses, dif_person, by = "person_id", all.x = TRUE
    )
    wide_responses <- wide_responses[
      stats::complete.cases(
        wide_responses[, c(".dif_group", "sum_score")]
      ),
    ]
    
    gamma_df <- data.frame(
      item  = character(),
      gamma = numeric(),
      stringsAsFactors = FALSE
    )
    
    for (item_label in unique_items) {
      if (!item_label %in% names(wide_responses)) next
      
      item_resp  <- wide_responses[[item_label]]
      dif_group  <- as.numeric(as.factor(wide_responses$.dif_group))
      tot_score  <- wide_responses$sum_score
      
      ok <- stats::complete.cases(item_resp, dif_group, tot_score)
      if (sum(ok) < 10) {
        gamma_val <- NA_real_
      } else {
        gamma_val <- .partial_gk_gamma(
          item_resp[ok], dif_group[ok], tot_score[ok]
        )
      }
      
      gamma_df <- rbind(gamma_df, data.frame(
        item  = item_label,
        gamma = gamma_val,
        stringsAsFactors = FALSE
      ))
    }
    
    gamma_annotations <- gamma_df
  }
  
  # ================================================================
  # COMPUTE EXPECTED ITEM SCORE CURVES PER ITEM
  # ================================================================
  n_draws_total <- nrow(draws)
  max_draws     <- min(n_draws_total, 500)
  draw_sample   <- sample(seq_len(n_draws_total), max_draws)
  
  theta_grid <- seq(theta_range[1], theta_range[2],
                    length.out = n_points)
  
  expected_data_list <- list()
  
  for (item_label in unique_items) {
    if (is_ordinal) {
      thresh_cols <- .find_thresh_cols(draws, item_label)
      if (length(thresh_cols) == 0) {
        warning("Could not find threshold parameters for item '",
                item_label, "'. Skipping.", call. = FALSE)
        next
      }
      
      thresh_sub <- as.matrix(
        draws[draw_sample, thresh_cols, drop = FALSE]
      )
      thresh_sub <- thresh_sub - shift
      
      n_thresh <- ncol(thresh_sub)
      n_cat    <- n_thresh + 1
      cats     <- seq(0, n_cat - 1)
      
      escore_draws <- matrix(NA_real_, nrow = max_draws,
                             ncol = n_points)
      
      for (t in seq_along(theta_grid)) {
        theta <- theta_grid[t]
        for (s in seq_len(max_draws)) {
          tau_s <- thresh_sub[s, ]
          p_cat <- .compute_cat_probs(
            theta, tau_s, is_acat, is_cumul, is_sratio, is_cratio
          )
          escore_draws[s, t] <- sum(cats * p_cat)
        }
      }
      
    } else {
      item_loc_draws <- .get_item_location_draws(
        draws, item_label, item_name, draw_sample
      )
      item_loc_draws <- item_loc_draws - shift
      
      escore_draws <- matrix(NA_real_, nrow = max_draws,
                             ncol = n_points)
      for (t in seq_along(theta_grid)) {
        escore_draws[, t] <- stats::plogis(
          theta_grid[t] - item_loc_draws
        )
      }
    }
    
    e_mean  <- colMeans(escore_draws)
    e_lower <- apply(escore_draws, 2, stats::quantile,
                     probs = lower_prob)
    e_upper <- apply(escore_draws, 2, stats::quantile,
                     probs = upper_prob)
    
    expected_data_list[[length(expected_data_list) + 1]] <-
      data.frame(
        item    = item_label,
        theta   = theta_grid,
        e_score = e_mean,
        e_lower = e_lower,
        e_upper = e_upper,
        stringsAsFactors = FALSE
      )
  }
  
  expected_data <- do.call(rbind, expected_data_list)
  
  # ================================================================
  # COMPUTE OBSERVED SCORES BY CLASS INTERVAL PER ITEM
  # ================================================================
  observed_data_list <- list()
  
  for (item_label in unique_items) {
    item_rows      <- which(model_data[[item_name]] == item_label)
    item_responses <- as.numeric(model_data[[resp_var]][item_rows])
    item_persons   <- as.character(
      model_data[[person_name]][item_rows]
    )
    
    if (is_ordinal) {
      item_responses <- item_responses - min_resp
    }
    
    obs_df <- data.frame(
      person_id = item_persons,
      response  = item_responses,
      stringsAsFactors = FALSE
    )
    
    obs_df <- merge(obs_df, person_ci, by = "person_id",
                    all.x = FALSE)
    
    if (nrow(obs_df) == 0) next
    
    if (has_dif) {
      obs_agg <- stats::aggregate(
        response ~ interval + theta + .dif_group,
        data = obs_df, FUN = mean, na.rm = TRUE
      )
      obs_n <- stats::aggregate(
        response ~ interval + theta + .dif_group,
        data = obs_df, FUN = length
      )
      obs_se <- stats::aggregate(
        response ~ interval + theta + .dif_group,
        data = obs_df,
        FUN = function(x) {
          stats::sd(x, na.rm = TRUE) / sqrt(sum(!is.na(x)))
        }
      )
      names(obs_n)[names(obs_n) == "response"]  <- "n"
      names(obs_se)[names(obs_se) == "response"] <- "se"
      obs_agg <- merge(obs_agg, obs_n,
                       by = c("interval", "theta", ".dif_group"))
      obs_agg <- merge(obs_agg, obs_se,
                       by = c("interval", "theta", ".dif_group"))
    } else {
      obs_agg <- stats::aggregate(
        response ~ interval + theta,
        data = obs_df, FUN = mean, na.rm = TRUE
      )
      obs_n <- stats::aggregate(
        response ~ interval + theta,
        data = obs_df, FUN = length
      )
      obs_se <- stats::aggregate(
        response ~ interval + theta,
        data = obs_df,
        FUN = function(x) {
          stats::sd(x, na.rm = TRUE) / sqrt(sum(!is.na(x)))
        }
      )
      names(obs_n)[names(obs_n) == "response"]  <- "n"
      names(obs_se)[names(obs_se) == "response"] <- "se"
      obs_agg <- merge(obs_agg, obs_n, by = c("interval", "theta"))
      obs_agg <- merge(obs_agg, obs_se, by = c("interval", "theta"))
    }
    
    obs_agg <- obs_agg[obs_agg$n >= min_n, ]
    obs_agg$item <- item_label
    
    observed_data_list[[length(observed_data_list) + 1]] <- obs_agg
  }
  
  observed_data <- do.call(rbind, observed_data_list)
  
  # ================================================================
  # BUILD PLOT
  # ================================================================
  expected_data$item <- factor(expected_data$item,
                               levels = unique_items)
  observed_data$item <- factor(observed_data$item,
                               levels = unique_items)
  
  n_items_plot <- length(unique_items)
  if (is.null(ncol)) {
    ncol <- ceiling(sqrt(n_items_plot))
  }
  
  p <- ggplot2::ggplot()
  
  # Expected score curve with ribbon
  if (ribbon_alpha > 0) {
    p <- p + ggplot2::geom_ribbon(
      data    = expected_data,
      mapping = ggplot2::aes(
        x = .data$theta, ymin = .data$e_lower, ymax = .data$e_upper
      ),
      fill = "grey70", alpha = ribbon_alpha
    )
  }
  
  p <- p + ggplot2::geom_line(
    data    = expected_data,
    mapping = ggplot2::aes(x = .data$theta, y = .data$e_score),
    colour  = "grey40", linewidth = line_size
  )
  
  # Observed score points with error bars
  if (has_dif) {
    p <- p +
      ggplot2::geom_errorbar(
        data    = observed_data,
        mapping = ggplot2::aes(
          x      = .data$theta,
          ymin   = pmax(.data$response - (1.96 * .data$se), 0),
          ymax   = .data$response + (1.96 * .data$se),
          colour = .data$.dif_group
        ),
        width = 0.05, linewidth = 0.4
      ) +
      ggplot2::geom_line(
        data    = observed_data,
        mapping = ggplot2::aes(
          x      = .data$theta,
          y      = .data$response,
          colour = .data$.dif_group
        ),
        linewidth = 0.5
      ) +
      ggplot2::geom_point(
        data    = observed_data,
        mapping = ggplot2::aes(
          x      = .data$theta,
          y      = .data$response,
          colour = .data$.dif_group
        ),
        size = point_size
      )
    
    if (!is.null(palette)) {
      p <- p +
        ggplot2::scale_colour_manual(values = palette, name = "")
    } else {
      p <- p +
        ggplot2::scale_colour_viridis_d(end = 0.8, name = "")
    }
  } else {
    p <- p +
      ggplot2::geom_errorbar(
        data    = observed_data,
        mapping = ggplot2::aes(
          x    = .data$theta,
          ymin = pmax(.data$response - (1.96 * .data$se), 0),
          ymax = .data$response + (1.96 * .data$se)
        ),
        colour = "#1F78B4", width = 0.05, linewidth = 0.4
      ) +
      ggplot2::geom_point(
        data    = observed_data,
        mapping = ggplot2::aes(
          x = .data$theta, y = .data$response
        ),
        colour = "#1F78B4", size = point_size
      )
  }
  
  # Add partial gamma annotation if DIF is active
  if (has_dif && dif_stats && !is.null(gamma_annotations)) {
    gamma_annotations$item <- factor(gamma_annotations$item,
                                     levels = unique_items)
    gamma_annotations$label <- ifelse(
      is.na(gamma_annotations$gamma),
      "gamma == NA",
      paste0("gamma == ",
             format(round(gamma_annotations$gamma, 2), nsmall = 2))
    )
    
    p <- p +
      ggplot2::geom_text(
        data    = gamma_annotations,
        mapping = ggplot2::aes(
          x     = -Inf,
          y     = Inf,
          label = .data$label
        ),
        hjust   = -0.1,
        vjust   = 1.5,
        size    = 3.5,
        colour  = "black",
        parse   = TRUE,
        inherit.aes = FALSE
      )
  }
  
  p <- p +
    ggplot2::facet_wrap(~ item, ncol = ncol, scales = "free_y") +
    ggplot2::scale_y_continuous(limits = c(0, NA)) +
    ggplot2::labs(
      x = expression(theta),
      y = "Item score"
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      legend.position = "bottom",
      strip.text = ggplot2::element_text(face = "bold")
    )
  
  p
}


# ══════════════════════════════════════════════════════════════════
# INTERNAL HELPERS
# ══════════════════════════════════════════════════════════════════

# ── Internal: find threshold columns for an item ────────────────
#' @keywords internal
.find_thresh_cols <- function(draws, item_label) {
  thresh_pattern <- paste0(
    "^b_Intercept\\[",
    gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
    ","
  )
  thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)
  
  if (length(thresh_cols) == 0) {
    thresh_cols <- grep("^b_Intercept\\[\\d+\\]$", names(draws),
                        value = TRUE)
  }
  
  if (length(thresh_cols) == 0) return(character(0))
  
  thresh_nums <- as.numeric(
    gsub(".*,(\\d+)\\]$|.*\\[(\\d+)\\]$", "\\1\\2", thresh_cols)
  )
  thresh_cols[order(thresh_nums)]
}


# ── Internal: compute category probabilities ────────────────────
#' @keywords internal
.compute_cat_probs <- function(theta, tau,
                               is_acat, is_cumul,
                               is_sratio, is_cratio) {
  n_cat <- length(tau) + 1
  
  if (is_acat) {
    cum_eta   <- c(0, cumsum(theta - tau))
    max_eta   <- max(cum_eta)
    log_denom <- max_eta + log(sum(exp(cum_eta - max_eta)))
    probs     <- exp(cum_eta - log_denom)
    
  } else if (is_cumul) {
    cum_probs <- stats::plogis(tau - theta)
    probs     <- diff(c(0, cum_probs, 1))
    probs     <- pmax(probs, 0)
    probs     <- probs / sum(probs)
    
  } else if (is_sratio) {
    probs     <- numeric(n_cat)
    remaining <- 1
    for (cc in seq_along(tau)) {
      p_stop       <- stats::plogis(tau[cc] - theta)
      probs[cc]    <- remaining * p_stop
      remaining    <- remaining * (1 - p_stop)
    }
    probs[n_cat] <- remaining
    
  } else if (is_cratio) {
    probs     <- numeric(n_cat)
    remaining <- 1
    for (cc in seq_along(tau)) {
      p_cont       <- stats::plogis(theta - tau[cc])
      probs[cc]    <- remaining * (1 - p_cont)
      remaining    <- remaining * p_cont
    }
    probs[n_cat] <- remaining
  }
  
  probs
}


# ── Internal: get dichotomous item location draws (subsampled) ──
#' @keywords internal
.get_item_location_draws <- function(draws, item_label, item_name,
                                     draw_sample) {
  col_pattern <- paste0("^b_", item_name,
                        gsub("([.|()\\^{}+$*?])", "\\\\\\1",
                             item_label), "$")
  fe_col <- grep(col_pattern, names(draws), value = TRUE)
  
  if (length(fe_col) > 0) {
    return(-as.numeric(draws[[fe_col[1]]][draw_sample]))
  }
  
  intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
  intercept_draws <- if (length(intercept_col) == 1) {
    as.numeric(draws[[intercept_col]][draw_sample])
  } else {
    rep(0, length(draw_sample))
  }
  
  re_col <- paste0("r_", item_name, "[", item_label, ",Intercept]")
  if (!re_col %in% names(draws)) {
    re_col_alt <- grep(
      paste0("^r_", item_name, "\\[",
             gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
             ",Intercept\\]$"),
      names(draws), value = TRUE
    )
    if (length(re_col_alt) == 1) re_col <- re_col_alt
    else stop("Could not find parameters for item '", item_label,
              "'.", call. = FALSE)
  }
  
  -(intercept_draws + as.numeric(draws[[re_col]][draw_sample]))
}


# ── Internal: get ALL dichotomous item location draws ────────────
#' @keywords internal
.get_item_location_draws_all <- function(draws, item_label,
                                         item_name) {
  col_pattern <- paste0("^b_", item_name,
                        gsub("([.|()\\^{}+$*?])", "\\\\\\1",
                             item_label), "$")
  fe_col <- grep(col_pattern, names(draws), value = TRUE)
  
  if (length(fe_col) > 0) {
    return(-as.numeric(draws[[fe_col[1]]]))
  }
  
  intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
  intercept_draws <- if (length(intercept_col) == 1) {
    as.numeric(draws[[intercept_col]])
  } else {
    rep(0, nrow(draws))
  }
  
  re_col <- paste0("r_", item_name, "[", item_label, ",Intercept]")
  if (!re_col %in% names(draws)) {
    re_col_alt <- grep(
      paste0("^r_", item_name, "\\[",
             gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
             ",Intercept\\]$"),
      names(draws), value = TRUE
    )
    if (length(re_col_alt) == 1) re_col <- re_col_alt
    else stop("Could not find parameters for item '", item_label,
              "'.", call. = FALSE)
  }
  
  -(intercept_draws + as.numeric(draws[[re_col]]))
}


# ── Internal: build wide-format response matrix per person ──────
#' @keywords internal
.build_wide_responses <- function(model_data, resp_var, item_name,
                                  person_name, all_items,
                                  min_resp, is_ordinal) {
  wide <- stats::reshape(
    model_data[, c(person_name, item_name, resp_var)],
    idvar     = person_name,
    timevar   = item_name,
    v.names   = resp_var,
    direction = "wide",
    sep       = "___"
  )
  
  colnames(wide) <- gsub(
    paste0("^", resp_var, "___"), "", colnames(wide)
  )
  colnames(wide)[1] <- "person_id"
  
  item_cols <- intersect(all_items, colnames(wide))
  if (is_ordinal && min_resp > 0) {
    for (col in item_cols) {
      wide[[col]] <- wide[[col]] - min_resp
    }
  }
  
  wide$sum_score <- rowSums(
    wide[, item_cols, drop = FALSE], na.rm = TRUE
  )
  
  wide
}


# ── Internal: Partial Goodman-Kruskal gamma ─────────────────────
# Computes partial gamma between x and group, stratified by score.
# Reuses the concordant/discordant pair counting algorithm from
# gk_gamma_helpers.R (.gk_gamma_from_tab engine), accumulating
# C and D across strata before computing the ratio. This is the
# same statistic as iarm::partgam_DIF().
#' @keywords internal
.partial_gk_gamma <- function(x, group, score) {
  C_total <- 0
  D_total <- 0
  
  unique_scores <- sort(unique(score))
  
  for (s in unique_scores) {
    idx <- which(score == s)
    if (length(idx) < 2L) next
    
    x_s <- x[idx]
    g_s <- group[idx]
    
    # Build item-response x DIF-group contingency table
    ux <- sort.int(unique.default(x_s), method = "quick")
    ug <- sort.int(unique.default(g_s), method = "quick")
    nr <- length(ux)
    nc <- length(ug)
    if (nr < 2L || nc < 2L) next
    
    xi <- match(x_s, ux)
    gi <- match(g_s, ug)
    
    tab <- matrix(0L, nrow = nr, ncol = nc)
    for (k in seq_along(xi)) {
      tab[xi[k], gi[k]] <- tab[xi[k], gi[k]] + 1L
    }
    
    # Count concordant/discordant pairs for this stratum
    # (same suffix/prefix sum algorithm as .gk_gamma_from_tab)
    suffix <- matrix(0, nrow = nr + 1L, ncol = nc + 1L)
    for (i in seq.int(nr, 1L)) {
      for (j in seq.int(nc, 1L)) {
        suffix[i, j] <- tab[i, j] +
          suffix[i + 1L, j] +
          suffix[i, j + 1L] -
          suffix[i + 1L, j + 1L]
      }
    }
    
    pcol <- matrix(0, nrow = nr + 1L, ncol = nc + 1L)
    for (i in seq.int(nr, 1L)) {
      for (j in seq.int(2L, nc + 1L)) {
        pcol[i, j] <- tab[i, j - 1L] +
          pcol[i + 1L, j] +
          pcol[i, j - 1L] -
          pcol[i + 1L, j - 1L]
      }
    }
    
    for (i in seq_len(nr - 1L)) {
      for (j in seq_len(nc)) {
        n_ij <- tab[i, j]
        if (n_ij == 0L) next
        if (j < nc) C_total <- C_total + n_ij * suffix[i + 1L, j + 1L]
        if (j > 1L) D_total <- D_total + n_ij * pcol[i + 1L, j]
      }
    }
  }
  
  denom <- C_total + D_total
  if (denom == 0) return(NA_real_)
  (C_total - D_total) / denom
}

#' Extract Person Parameters from a Bayesian Rasch Model
#'
#' Extracts person ability estimates from a fitted Bayesian Rasch
#' model. Returns both Bayesian EAP (expected a posteriori) estimates
#' with posterior SDs and frequentist WLE (Warm's weighted likelihood)
#' estimates with standard errors, plus a lookup table mapping ordinal
#' sum scores to both scales.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object. Supported
#'   parameterisations:
#'   \describe{
#'     \item{Polytomous ordinal (PCM)}{e.g., \code{family = acat} with
#'       \code{thres(gr = item)}, producing item-specific thresholds.}
#'     \item{Dichotomous Rasch (random items)}{e.g.,
#'       \code{response ~ 1 + (1 | item) + (1 | id)} with
#'       \code{family = bernoulli()}.}
#'     \item{Dichotomous 1PL (fixed items)}{e.g.,
#'       \code{response ~ 0 + item + (1 | id)} with
#'       \code{family = bernoulli()}.}
#'   }
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data. Default is \code{id}.
#' @param draws Logical. If \code{TRUE}, a matrix of full posterior
#'   draws (persons x draws) is included in the output. Default is
#'   \code{FALSE}.
#' @param center Logical. If \code{TRUE} (the default), person
#'   parameters and item difficulties are recentered so that the
#'   mean item difficulty is zero, matching the convention in
#'   frequentist CML Rasch estimation. If \code{FALSE}, raw brms
#'   parameterisation is used.
#' @param theta_range A numeric vector of length 2 specifying the
#'   range for the Newton-Raphson WLE search. Default is
#'   \code{c(-7, 7)}.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{person_estimates}{A \code{\link[tibble]{tibble}} with one
#'     row per person containing: the person ID, \code{sum_score},
#'     \code{eap} (posterior mean of person random effect),
#'     \code{eap_se} (posterior SD), \code{wle} (Warm's weighted
#'     likelihood estimate), and \code{wle_se} (asymptotic SE
#'     of the WLE). Rows are ordered to match the original person
#'     order in the model data (i.e., the order of first appearance
#'     of each person ID).}
#'   \item{score_table}{A \code{\link[tibble]{tibble}} mapping each
#'     observed ordinal sum score to its mean EAP, mean EAP SE,
#'     WLE, and WLE SE. Extreme scores (0 and maximum) receive
#'     WLE estimates at the boundary of \code{theta_range} with
#'     \code{NA} standard errors.}
#'   \item{draws_matrix}{(Only if \code{draws = TRUE}) A numeric
#'     matrix with rows = persons and columns = posterior draws.
#'     Row names are person IDs. Rows are ordered to match the
#'     original person order in the model data. Can be passed
#'     directly to \code{\link{RMUreliability}}.}
#' }
#'
#' @details
#' \strong{EAP estimates} are extracted as the posterior means of
#' the person random effects (\code{r_id[j, Intercept]}) from
#' \code{\link[brms]{as_draws_df}}, with the posterior SD serving
#' as the standard error. These are the standard Bayesian point
#' estimates and reflect shrinkage toward the population mean.
#'
#' \strong{WLE estimates} (Warm, 1989) are computed from the
#' posterior mean item parameters using Newton-Raphson iteration
#' with adaptive step damping on the Warm-corrected likelihood.
#' WLE adds a bias correction term \eqn{J(\theta) / (2 I(\theta))}
#' to the score equations, where \eqn{I(\theta)} is the test
#' information and \eqn{J(\theta) = \sum_i \sum_c (c - E_i)^3 P_{ic}}
#' is the sum of third central moments. This produces estimates with
#' reduced finite-sample bias compared to MLE, especially at
#' extreme scores (Warm, 1989).
#'
#' The Newton-Raphson algorithm uses adaptive step damping following
#' the approach in \pkg{iarm} (Mueller): the maximum allowed step
#' size shrinks by a factor of 1.05 each iteration, preventing
#' overshoot and ensuring convergence for near-extreme scores.
#'
#' For extreme scores (sum score = 0 or maximum possible), the
#' WLE is not well-defined. These cases are assigned the boundary
#' values of \code{theta_range} with \code{NA} standard errors.
#'
#' When \code{center = TRUE} (the default), item difficulty
#' parameters are shifted so their mean is zero, and EAP person
#' parameters are shifted by the same constant. WLE is computed
#' from the centered item parameters. This matches the convention
#' in frequentist CML Rasch estimation.
#'
#' \strong{Row ordering:} Both \code{person_estimates} and
#' \code{draws_matrix} preserve the original person order from
#' the model data (order of first appearance of each person ID).
#' This allows direct row-binding with the source data without
#' re-matching.
#'
#' @references
#' Warm, T. A. (1989). Weighted likelihood estimation of ability
#' in item response theory. \emph{Psychometrika}, \emph{54}(3),
#' 427--450. \doi{10.1007/BF02294627}
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R
#' with brms and Stan. \emph{Journal of Statistical Software},
#' \emph{100}, 1--54. \doi{10.18637/jss.v100.i05}
#'
#' Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
#' \emph{Rasch Models in Health}. Iste and Wiley, pp. 63--70.
#'
#' @seealso
#' \code{\link{RMUreliability}}, \code{\link{plot_targeting}},
#' \code{\link[brms]{ranef}}, \code{\link[brms]{as_draws_df}}.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Dichotomous Rasch Model (random items) ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4, cores = 2, iter = 1000 # use more iter and cores
#' )
#'
#' # Basic usage — rows match original person order
#' pp <- person_parameters(fit_rm)
#' pp$person_estimates
#' pp$score_table
#'
#' # With full posterior draws (e.g., for RMUreliability)
#' pp_draws <- person_parameters(fit_rm, draws = TRUE)
#' RMUreliability(pp_draws$draws_matrix)
#'
#' # --- Polytomous PCM (acat) ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4, cores = 2, iter = 1000 # use more iter and cores
#' )
#'
#' pp_pcm <- person_parameters(fit_pcm)
#' pp_pcm$score_table
#' }
#'
#' @importFrom brms as_draws_df ranef
#' @importFrom rlang enquo as_name .data
#' @importFrom stats family sd var plogis as.formula
#' @importFrom tibble tibble as_tibble
#' @importFrom dplyr left_join group_by summarise arrange mutate n
#' @export
person_parameters <- function(
    model,
    item_var    = item,
    person_var  = id,
    draws       = FALSE,
    center      = TRUE,
    theta_range = c(-7, 7)
) {
  
  # ================================================================
  # INPUT VALIDATION
  # ================================================================
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))
  
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  resp_var <- as.character(stats::formula(model)$formula[[2]])
  if (length(resp_var) > 1) resp_var <- resp_var[2]
  
  all_draws   <- tibble::as_tibble(brms::as_draws_df(model))
  family_name <- stats::family(model)$family
  is_ordinal  <- grepl("acat|cumul|sratio|cratio",
                       family_name, ignore.case = TRUE)
  
  # ================================================================
  # ORIGINAL PERSON ORDER (first appearance in model data)
  # ================================================================
  person_col_data   <- as.character(model$data[[person_name]])
  original_order    <- unique(person_col_data)
  
  # ================================================================
  # DETECT MODEL PARAMETERISATION
  # ================================================================
  fe_pattern <- paste0("^b_", item_name)
  fe_cols    <- grep(fe_pattern, names(all_draws), value = TRUE)
  fe_cols    <- setdiff(fe_cols, "b_Intercept")
  
  has_fixed_items  <- !is_ordinal && length(fe_cols) > 0
  has_random_items <- !is_ordinal && any(grepl(
    paste0("^r_", item_name, "\\["), names(all_draws)
  ))
  
  if (!is_ordinal && !has_fixed_items && !has_random_items) {
    stop("Could not detect item parameters in the model. ",
         "Supported parameterisations:\n",
         "  - response ~ 0 + item + (1 | id)    [fixed items]\n",
         "  - response ~ 1 + (1 | item) + (1 | id)  [random items]\n",
         "  - response | thres(gr = item) ~ 1 + (1 | id) [PCM/acat]",
         call. = FALSE)
  }
  
  # ================================================================
  # EXTRACT ITEM PARAMETERS (posterior means)
  # ================================================================
  unique_items <- sort(unique(model$data[[item_name]]))
  n_items <- length(unique_items)
  
  if (is_ordinal) {
    item_params <- .extract_item_params_ordinal(
      all_draws, unique_items, item_name
    )
  } else if (has_fixed_items) {
    item_params <- .extract_item_params_fixed(
      all_draws, fe_cols, unique_items, item_name
    )
  } else {
    item_params <- .extract_item_params_random(
      all_draws, unique_items, item_name
    )
  }
  
  # ================================================================
  # CENTERING
  # ================================================================
  if (center && !is_ordinal) {
    shift <- mean(item_params$difficulties)
    item_params$difficulties <- item_params$difficulties - shift
  } else if (center && is_ordinal) {
    all_thresholds <- unlist(item_params$thresholds)
    shift <- mean(all_thresholds)
    item_params$thresholds <- lapply(
      item_params$thresholds, function(x) x - shift
    )
  } else {
    shift <- 0
  }
  
  # ================================================================
  # EAP PERSON PARAMETERS
  # ================================================================
  person_re <- brms::ranef(model)[[person_name]]
  if (is.null(person_re)) {
    stop("No random effects found for '", person_name,
         "'. Check your model formula.", call. = FALSE)
  }
  
  # ranef() returns persons in its own order (typically alphabetical)
  ranef_ids   <- rownames(person_re[, , "Intercept"])
  eap_raw     <- person_re[, "Estimate", "Intercept"]
  eap_se_raw  <- person_re[, "Est.Error", "Intercept"]
  
  if (center) {
    eap <- eap_raw - shift
  } else {
    eap <- eap_raw
  }
  eap_se <- eap_se_raw
  
  # ================================================================
  # COMPUTE SUM SCORES
  # ================================================================
  dat <- model$data
  sum_scores <- stats::aggregate(
    stats::as.formula(paste(resp_var, "~", person_name)),
    data = dat,
    FUN  = sum,
    na.rm = TRUE
  )
  colnames(sum_scores) <- c("person_id", "sum_score")
  
  # For ordinal models, the minimum response category may be > 0
  # (brms acat uses 1-based coding). Adjust sum scores to be
  # 0-based for the WLE computation.
  min_resp <- min(dat[[resp_var]], na.rm = TRUE)
  if (min_resp > 0) {
    n_obs_per_person <- stats::aggregate(
      stats::as.formula(paste(resp_var, "~", person_name)),
      data = dat,
      FUN  = length
    )
    colnames(n_obs_per_person) <- c("person_id", "n_obs")
    sum_scores <- merge(sum_scores, n_obs_per_person, by = "person_id")
    sum_scores$sum_score <- sum_scores$sum_score -
      (min_resp * sum_scores$n_obs)
    sum_scores$n_obs <- NULL
  }
  
  # ================================================================
  # WLE COMPUTATION
  # ================================================================
  if (is_ordinal) {
    max_score <- sum(vapply(
      item_params$thresholds, length, integer(1)
    ))
    wle_results <- .compute_wle_polytomous(
      item_params$thresholds, max_score, theta_range
    )
  } else {
    max_score <- n_items
    wle_results <- .compute_wle_dichotomous(
      item_params$difficulties, theta_range
    )
  }
  
  # ================================================================
  # BUILD PERSON-LEVEL TABLE (in ranef order first, then reorder)
  # ================================================================
  person_df <- tibble::tibble(
    person_id = ranef_ids,
    sum_score = sum_scores$sum_score[
      match(ranef_ids, as.character(sum_scores$person_id))
    ],
    eap    = round(as.numeric(eap), 4),
    eap_se = round(as.numeric(eap_se), 4)
  )
  
  # Match WLE to persons via sum score
  wle_lookup <- tibble::tibble(
    sum_score = wle_results$scores,
    wle       = round(wle_results$wle, 4),
    wle_se    = round(wle_results$wle_se, 4)
  )
  
  person_df <- dplyr::left_join(person_df, wle_lookup, by = "sum_score")
  
  # Rename the person_id column to the actual variable name
  colnames(person_df)[1] <- person_name
  
  # Reorder to match original person order in model data
  orig_idx <- match(original_order, person_df[[person_name]])
  person_df <- person_df[orig_idx, ]
  
  # ================================================================
  # BUILD SCORE TABLE (sum score -> EAP and WLE)
  # ================================================================
  score_table <- person_df |>
    dplyr::group_by(.data$sum_score) |>
    dplyr::summarise(
      n       = dplyr::n(),
      eap     = round(mean(.data$eap, na.rm = TRUE), 4),
      eap_se  = round(mean(.data$eap_se, na.rm = TRUE), 4),
      wle     = round(mean(.data$wle, na.rm = TRUE), 4),
      wle_se  = round(mean(.data$wle_se, na.rm = TRUE), 4),
      .groups = "drop"
    )
  
  # ================================================================
  # FULL POSTERIOR DRAWS (optional)
  # ================================================================
  out <- list(
    person_estimates = person_df,
    score_table      = score_table
  )
  
  if (draws) {
    re_pattern <- paste0("^r_", person_name, "\\[")
    re_cols <- grep(re_pattern, names(all_draws), value = TRUE)
    
    if (length(re_cols) == 0) {
      warning("Could not find posterior draws for person random effects.",
              call. = FALSE)
    } else {
      draws_mat <- t(as.matrix(all_draws[, re_cols]))
      if (center) {
        draws_mat <- draws_mat - shift
      }
      
      # Clean row names: "r_id[person1,Intercept]" -> "person1"
      rn <- rownames(draws_mat)
      rn <- gsub(paste0("^r_", person_name, "\\["), "", rn)
      rn <- gsub(",Intercept\\]$", "", rn)
      rownames(draws_mat) <- rn
      
      # Reorder rows to match original person order in model data
      draws_idx <- match(original_order, rn)
      draws_mat <- draws_mat[draws_idx, , drop = FALSE]
      
      out$draws_matrix <- draws_mat
    }
  }
  
  out
}


# ══════════════════════════════════════════════════════════════════
# ITEM PARAMETER EXTRACTION HELPERS
# ══════════════════════════════════════════════════════════════════

# ── Internal: extract item parameters for ordinal (acat/PCM) ─────
#' @keywords internal
.extract_item_params_ordinal <- function(draws, unique_items,
                                         item_name) {
  thresholds <- list()
  
  for (item_label in unique_items) {
    thresh_pattern <- paste0(
      "^b_Intercept\\[",
      gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
      ","
    )
    thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)
    
    if (length(thresh_cols) == 0) {
      warning("No threshold parameters found for item '",
              item_label, "'.", call. = FALSE)
      next
    }
    
    thresh_nums <- as.numeric(
      gsub(".*,(\\d+)\\]$", "\\1", thresh_cols)
    )
    thresh_cols <- thresh_cols[order(thresh_nums)]
    
    thresholds[[item_label]] <- vapply(
      thresh_cols,
      function(col) mean(draws[[col]]),
      numeric(1)
    )
  }
  
  list(thresholds = thresholds)
}


# ── Internal: extract item params for fixed-effects dichotomous ──
#' @keywords internal
.extract_item_params_fixed <- function(draws, fe_cols, unique_items,
                                       item_name) {
  difficulties <- vapply(
    fe_cols,
    function(col) -mean(draws[[col]]),
    numeric(1)
  )
  names(difficulties) <- unique_items
  list(difficulties = difficulties)
}


# ── Internal: extract item params for random-effects dichotomous ─
#' @keywords internal
.extract_item_params_random <- function(draws, unique_items,
                                        item_name) {
  intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
  intercept_mean <- if (length(intercept_col) == 1) {
    mean(draws[[intercept_col]])
  } else {
    0
  }
  
  difficulties <- vapply(unique_items, function(item_label) {
    re_col <- paste0("r_", item_name, "[", item_label, ",Intercept]")
    if (!re_col %in% names(draws)) {
      re_col_alt <- grep(
        paste0("^r_", item_name, "\\[",
               gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
               ",Intercept\\]$"),
        names(draws), value = TRUE
      )
      if (length(re_col_alt) == 1) re_col <- re_col_alt
      else return(NA_real_)
    }
    -(intercept_mean + mean(draws[[re_col]]))
  }, numeric(1))
  
  names(difficulties) <- unique_items
  list(difficulties = difficulties)
}


# ══════════════════════════════════════════════════════════════════
# WLE COMPUTATION HELPERS
# ══════════════════════════════════════════════════════════════════
#
# Newton-Raphson with adaptive step damping following the approach
# in iarm::persons_mle (Mueller): the maximum allowed step size
# starts at maxdelta_start and shrinks by a factor of
# maxdelta_shrink each iteration, preventing overshoot and ensuring
# convergence for near-extreme scores.
# ══════════════════════════════════════════════════════════════════

# ── Internal: WLE for dichotomous Rasch ──────────────────────────
#' @keywords internal
.compute_wle_dichotomous <- function(difficulties, theta_range,
                                     max_iter = 100, tol = 1e-4,
                                     maxdelta_start = 3,
                                     maxdelta_shrink = 1.05) {
  n_items  <- length(difficulties)
  scores   <- 0:n_items
  n_scores <- length(scores)
  maxval   <- max(abs(theta_range))
  
  wle    <- rep(NA_real_, n_scores)
  wle_se <- rep(NA_real_, n_scores)
  
  for (idx in seq_along(scores)) {
    r <- scores[idx]
    
    # Extreme scores: boundary values, no SE
    if (r == 0) {
      wle[idx]    <- theta_range[1]
      wle_se[idx] <- NA_real_
      next
    }
    if (r == n_items) {
      wle[idx]    <- theta_range[2]
      wle_se[idx] <- NA_real_
      next
    }
    
    # Newton-Raphson with adaptive step damping
    theta_hat <- 0
    maxdelta  <- maxdelta_start
    
    for (iter in seq_len(max_iter)) {
      theta0 <- theta_hat
      
      p_i <- stats::plogis(theta_hat - difficulties)
      q_i <- 1 - p_i
      
      # Score function and derivatives
      dll  <- r - sum(p_i)                    # first derivative
      d2ll <- -sum(p_i * q_i)                 # second derivative
      d3ll <- -sum(p_i * q_i * (q_i - p_i))  # third derivative
      
      # WLE step: NR + Warm bias correction
      delta <- -dll / d2ll - d3ll / (2 * d2ll^2)
      
      # Adaptive damping: shrink maxdelta each iteration
      maxdelta <- maxdelta / maxdelta_shrink
      if (abs(delta) > maxdelta) {
        delta <- sign(delta) * maxdelta
      }
      
      theta_hat <- theta_hat + delta
      
      # Safety clamp at boundary
      if (abs(theta_hat) > maxval) {
        theta_hat <- sign(theta_hat) * maxval
      }
      
      if (abs(theta_hat - theta0) < tol) break
    }
    
    wle[idx] <- theta_hat
    
    # SE = 1/sqrt(I) at convergence; NA if at boundary
    p_i     <- stats::plogis(theta_hat - difficulties)
    q_i     <- 1 - p_i
    I_final <- sum(p_i * q_i)
    
    if (abs(theta_hat) >= maxval || I_final < 1e-12) {
      wle_se[idx] <- NA_real_
    } else {
      wle_se[idx] <- 1 / sqrt(I_final)
    }
  }
  
  list(scores = scores, wle = wle, wle_se = wle_se)
}


# ── Internal: WLE for polytomous (acat/PCM) ─────────────────────
#' @keywords internal
.compute_wle_polytomous <- function(thresholds, max_score, theta_range,
                                    max_iter = 100, tol = 1e-4,
                                    maxdelta_start = 3,
                                    maxdelta_shrink = 1.05) {
  scores   <- 0:max_score
  n_scores <- length(scores)
  n_items  <- length(thresholds)
  maxval   <- max(abs(theta_range))
  
  wle    <- rep(NA_real_, n_scores)
  wle_se <- rep(NA_real_, n_scores)
  
  for (idx in seq_along(scores)) {
    r <- scores[idx]
    
    # Extreme scores: boundary values, no SE
    if (r == 0) {
      wle[idx]    <- theta_range[1]
      wle_se[idx] <- NA_real_
      next
    }
    if (r == max_score) {
      wle[idx]    <- theta_range[2]
      wle_se[idx] <- NA_real_
      next
    }
    
    # Newton-Raphson with adaptive step damping
    theta_hat <- 0
    maxdelta  <- maxdelta_start
    
    for (iter in seq_len(max_iter)) {
      theta0 <- theta_hat
      
      # Accumulate derivatives across items
      dll  <- r   # will subtract E_i for each item
      d2ll <- 0
      d3ll <- 0
      
      for (i in seq_len(n_items)) {
        tau_i <- thresholds[[i]]
        n_cat <- length(tau_i) + 1
        cats  <- seq(0, n_cat - 1)
        
        # Adjacent category model probabilities (log-sum-exp stable)
        cum_eta   <- c(0, cumsum(theta_hat - tau_i))
        max_eta   <- max(cum_eta)
        log_denom <- max_eta + log(sum(exp(cum_eta - max_eta)))
        p_cat     <- exp(cum_eta - log_denom)
        
        E_i <- sum(cats * p_cat)              # expected score
        V_i <- sum((cats - E_i)^2 * p_cat)   # variance = info
        K_i <- sum((cats - E_i)^3 * p_cat)   # third central moment
        
        dll  <- dll - E_i
        d2ll <- d2ll - V_i
        d3ll <- d3ll + K_i
      }
      
      # Guard against zero information
      if (abs(d2ll) < 1e-12) break
      
      # WLE step: NR + Warm bias correction
      delta <- -dll / d2ll - d3ll / (2 * d2ll^2)
      
      # Adaptive damping: shrink maxdelta each iteration
      maxdelta <- maxdelta / maxdelta_shrink
      if (abs(delta) > maxdelta) {
        delta <- sign(delta) * maxdelta
      }
      
      theta_hat <- theta_hat + delta
      
      # Safety clamp at boundary
      if (abs(theta_hat) > maxval) {
        theta_hat <- sign(theta_hat) * maxval
      }
      
      if (abs(theta_hat - theta0) < tol) break
    }
    
    wle[idx] <- theta_hat
    
    # Recompute information at convergence for SE
    I_final <- 0
    for (i in seq_len(n_items)) {
      tau_i <- thresholds[[i]]
      n_cat <- length(tau_i) + 1
      cats  <- seq(0, n_cat - 1)
      cum_eta   <- c(0, cumsum(theta_hat - tau_i))
      max_eta   <- max(cum_eta)
      log_denom <- max_eta + log(sum(exp(cum_eta - max_eta)))
      p_cat     <- exp(cum_eta - log_denom)
      E_i <- sum(cats * p_cat)
      V_i <- sum((cats - E_i)^2 * p_cat)
      I_final <- I_final + V_i
    }
    
    if (abs(theta_hat) >= maxval || I_final < 1e-12) {
      wle_se[idx] <- NA_real_
    } else {
      wle_se[idx] <- 1 / sqrt(I_final)
    }
  }
  
  list(scores = scores, wle = wle, wle_se = wle_se)
}

#' Tile Plot of Item Response Distributions
#'
#' Creates a tile (heat map) plot showing the distribution of
#' responses across all items and response categories. Each cell
#' displays the count (or percentage) of responses, with optional
#' conditional highlighting for cells with low counts. This is a
#' descriptive data visualization tool intended for use before
#' model fitting.
#'
#' @param data A data frame in wide format containing only the item
#'   response columns. Each column is one item, each row is one
#'   person. All columns must be numeric (integer-valued). Response
#'   categories may be coded starting from 0 or 1. Do not include
#'   person IDs, grouping variables, or other non-item columns.
#' @param cutoff Integer. Cells with counts below this value are
#'   highlighted (when \code{highlight = TRUE}). Default is 10.
#' @param highlight Logical. If \code{TRUE} (the default), cell
#'   labels with counts below \code{cutoff} are displayed in red.
#'   This includes cells with zero responses (empty categories),
#'   which is useful for identifying gaps in the response
#'   distribution.
#' @param percent Logical. If \code{TRUE}, cell labels show
#'   percentages instead of raw counts. Default is \code{FALSE}.
#' @param text_color Character. Color for cell label text (when
#'   not highlighted). Default is \code{"orange"}.
#' @param item_labels An optional character vector of descriptive
#'   labels for the items (y-axis). Must be the same length as
#'   \code{ncol(data)}. If \code{NULL} (the default), column names
#'   are used.
#' @param category_labels An optional character vector of labels
#'   for the response categories (x-axis). Must be the same length
#'   as the number of response categories spanning from the minimum
#'   to the maximum observed value. If \code{NULL} (the default),
#'   numeric category values are used.
#'
#' @return A \code{\link[ggplot2]{ggplot}} object.
#'
#' @details
#' The plot displays items on the y-axis (in the same order as the
#' columns in \code{data}, from top to bottom) and response
#' categories on the x-axis. Cell shading represents the count
#' of responses (darker = more responses). Cell labels show either
#' raw counts or percentages.
#'
#' Categories with zero responses are explicitly shown (as cells
#' with \code{n = 0}), which helps identify gaps in the response
#' distribution — one of the primary purposes of this plot.
#'
#' \strong{Input requirements:}
#' \itemize{
#'   \item All columns must be numeric (integer-valued).
#'   \item The data frame must contain at least 2 columns (items)
#'     and at least 1 row (person).
#' }
#'
#' @examples
#' library(ggplot2)
#' if (requireNamespace("eRm", quietly = TRUE))
#'
#' # Basic tile plot
#' plot_tile(eRm::pcmdat2)
#'
#' # With custom item labels
#' plot_tile(
#'   eRm::pcmdat2,
#'   item_labels = c("Mood", "Sleep", "Appetite", "Energy")
#' )
#'
#' # With custom category labels and percentages
#' plot_tile(
#'   eRm::pcmdat2,
#'   category_labels = c("Never", "Sometimes", "Often"),
#'   percent = TRUE
#' )
#'
#' # Adjust cutoff for highlighting
#' plot_tile(eRm::pcmdat2, cutoff = 20, highlight = TRUE)
#'
#' @importFrom rlang .data
#' @export
plot_tile <- function(
    data,
    cutoff          = 10,
    highlight       = TRUE,
    percent         = FALSE,
    text_color      = "orange",
    item_labels     = NULL,
    category_labels = NULL
) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  
  # ================================================================
  # INPUT VALIDATION
  # ================================================================
  if (!is.data.frame(data)) {
    stop("'data' must be a data frame in wide format ",
         "(one column per item, one row per person).",
         call. = FALSE)
  }
  
  if (ncol(data) < 2) {
    stop("'data' must contain at least 2 columns (items). ",
         "Found ", ncol(data), " column(s).", call. = FALSE)
  }
  
  if (nrow(data) < 1) {
    stop("'data' must contain at least 1 row (person). ",
         "Found 0 rows.", call. = FALSE)
  }
  
  # Check all columns are numeric
  non_numeric <- names(data)[!vapply(data, is.numeric, logical(1))]
  if (length(non_numeric) > 0) {
    stop("All columns must be numeric. Non-numeric column(s): ",
         paste(non_numeric, collapse = ", "), ". ",
         "Remove non-item columns (e.g., person IDs, grouping ",
         "variables) before passing to plot_tile().",
         call. = FALSE)
  }
  
  # Check values are integer-valued
  all_vals <- unlist(data, use.names = FALSE)
  all_vals <- all_vals[!is.na(all_vals)]
  if (length(all_vals) == 0) {
    stop("'data' contains only NA values.", call. = FALSE)
  }
  if (!all(all_vals == round(all_vals))) {
    stop("All response values must be integers. ",
         "Found non-integer values in the data.", call. = FALSE)
  }
  
  # Validate item_labels
  if (!is.null(item_labels)) {
    if (length(item_labels) != ncol(data)) {
      stop("'item_labels' must have the same length as the number ",
           "of columns in 'data'. Expected ", ncol(data),
           ", got ", length(item_labels), ".", call. = FALSE)
    }
  }
  
  # Determine category range across all items
  min_val <- min(all_vals)
  max_val <- max(all_vals)
  all_categories <- seq(min_val, max_val)
  n_categories <- length(all_categories)
  
  # Validate category_labels
  if (!is.null(category_labels)) {
    if (length(category_labels) != n_categories) {
      stop("'category_labels' must have the same length as the ",
           "number of response categories (", min_val, " to ",
           max_val, " = ", n_categories, " categories). Got ",
           length(category_labels), ".", call. = FALSE)
    }
  }
  
  # ================================================================
  # PREPARE DATA
  # ================================================================
  item_names <- names(data)
  if (!is.null(item_labels)) {
    label_map <- stats::setNames(item_labels, item_names)
  } else {
    label_map <- stats::setNames(item_names, item_names)
  }
  
  # Count responses per item x category, including zeros
  count_list <- list()
  for (item in item_names) {
    item_vals <- data[[item]]
    item_vals <- item_vals[!is.na(item_vals)]
    tab <- table(factor(item_vals, levels = all_categories))
    count_list[[item]] <- data.frame(
      item     = item,
      category = as.integer(names(tab)),
      n        = as.integer(tab),
      stringsAsFactors = FALSE
    )
  }
  count_df <- do.call(rbind, count_list)
  rownames(count_df) <- NULL
  
  # Compute percentage within each item
  item_totals <- stats::aggregate(n ~ item, data = count_df, FUN = sum)
  colnames(item_totals)[2] <- "total"
  count_df <- merge(count_df, item_totals, by = "item")
  count_df$percentage <- round(count_df$n / count_df$total * 100, 1)
  
  # Apply item labels
  count_df$item_label <- label_map[count_df$item]
  
  # Set factor levels: items in reverse column order (so first item
  # appears at the top of the y-axis)
  count_df$item_label <- factor(
    count_df$item_label,
    levels = rev(label_map[item_names])
  )
  
  # ================================================================
  # BUILD PLOT
  # ================================================================
  p <- ggplot2::ggplot(
    count_df,
    ggplot2::aes(
      x = .data$category,
      y = .data$item_label,
      fill = .data$n
    )
  ) +
    ggplot2::geom_tile() +
    ggplot2::scale_fill_viridis_c(
      expression(italic(n)),
      limits = c(0, NA)
    ) +
    ggplot2::labs(y = "Items") +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      axis.text.x = ggplot2::element_text(size = 8),
      panel.grid  = ggplot2::element_blank()
    )
  
  # X-axis: category labels or numeric breaks
  if (!is.null(category_labels)) {
    p <- p + ggplot2::scale_x_continuous(
      "Response category",
      expand = c(0, 0),
      breaks = all_categories,
      labels = category_labels
    )
  } else {
    p <- p + ggplot2::scale_x_continuous(
      "Response category",
      expand = c(0, 0),
      breaks = all_categories
    )
  }
  
  # Cell labels: count or percentage, with optional highlighting
  if (percent) {
    if (highlight) {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(
            label = paste0(.data$percentage, "%"),
            color = ifelse(.data$n < cutoff, "red", text_color)
          )
        ) +
        ggplot2::guides(color = "none") +
        ggplot2::scale_color_identity()
    } else {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = paste0(.data$percentage, "%")),
          color = text_color
        )
    }
  } else {
    if (highlight) {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(
            label = .data$n,
            color = ifelse(.data$n < cutoff, "red", text_color)
          )
        ) +
        ggplot2::guides(color = "none") +
        ggplot2::scale_color_identity()
    } else {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = .data$n),
          color = text_color
        )
    }
  }
  
  p
}

#' Item Response Distribution Bar Chart
#'
#' Creates a faceted bar chart showing the response distribution for
#' each item, with counts and percentages displayed on each bar.
#' Each item gets its own panel, with response categories on the
#' x-axis and percentage of responses on the y-axis. This is a
#' descriptive data visualization tool intended for use before
#' model fitting.
#'
#' @param data A data frame in wide format containing only the item
#'   response columns. Each column is one item, each row is one
#'   person. All columns must be numeric (integer-valued). Response
#'   categories may be coded starting from 0 or 1. Do not include
#'   person IDs, grouping variables, or other non-item columns.
#' @param item_labels An optional character vector of descriptive
#'   labels for the items (facet strips). Must be the same length
#'   as \code{ncol(data)}. If \code{NULL} (the default), column
#'   names are used. Labels are displayed as
#'   \code{"column_name - label"}.
#' @param category_labels An optional character vector of labels
#'   for the response categories (x-axis). Must be the same length
#'   as the number of response categories spanning from the minimum
#'   to the maximum observed value. If \code{NULL} (the default),
#'   numeric category values are used.
#' @param ncol Integer. Number of columns in the faceted layout.
#'   Default is 1.
#' @param label_wrap Integer. Number of characters per line in
#'   facet strip labels before wrapping. Default is 25.
#' @param text_y Numeric. Vertical position (in percent units) for
#'   the count labels on each bar. Adjust upward if bars are tall.
#'   Default is 6.
#' @param viridis_option Character. Viridis palette option for the
#'   count text color. One of \code{"A"} through \code{"H"}.
#'   Default is \code{"A"}.
#' @param viridis_end Numeric in \eqn{[0, 1]}. End point of the
#'   viridis color scale for count text. Adjust if text is hard to
#'   read against the bar colors. Default is 0.9.
#' @param font Character. Font family for all text. Default is
#'   \code{"sans"}.
#'
#' @return A \code{\link[ggplot2]{ggplot}} object.
#'
#' @details
#' Each item is displayed as a separate facet panel with the item
#' label in the strip on the left side. Bars are colored by response
#' category using the viridis palette. Each bar shows the count
#' (\code{n = X}) as text.
#'
#' \strong{Input requirements:}
#' \itemize{
#'   \item All columns must be numeric (integer-valued).
#'   \item The data frame must contain at least 2 columns (items)
#'     and at least 1 row (person).
#' }
#'
#' @examples
#' library(ggplot2)
#' if (requireNamespace("eRm", quietly = TRUE))
#' 
#' # Basic response distribution plot
#' plot_bars(eRm::pcmdat2)
#'
#' # With custom item labels
#' plot_bars(
#'   eRm::pcmdat2,
#'   item_labels = c("Mood", "Sleep", "Appetite", "Energy")
#' )
#'
#' # Two-column layout with wrapped labels
#' plot_bars(
#'   eRm::pcmdat2,
#'   item_labels = c(
#'     "General mood and emotional wellbeing",
#'     "Quality of sleep at night",
#'     "Appetite and eating habits",
#'     "Overall energy level during the day"
#'   ),
#'   ncol = 2, label_wrap = 20
#' )
#'
#' # With custom category labels
#' plot_bars(
#'   eRm::pcmdat2,
#'   category_labels = c("Never", "Sometimes", "Often")
#' )
#'
#' @importFrom rlang .data
#' @export
plot_bars <- function(
    data,
    item_labels     = NULL,
    category_labels = NULL,
    ncol            = 1,
    label_wrap      = 25,
    text_y          = 6,
    viridis_option  = "A",
    viridis_end     = 0.9,
    font            = "sans"
) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  
  # ================================================================
  # INPUT VALIDATION
  # ================================================================
  if (!is.data.frame(data)) {
    stop("'data' must be a data frame in wide format ",
         "(one column per item, one row per person).",
         call. = FALSE)
  }
  
  if (ncol(data) < 2) {
    stop("'data' must contain at least 2 columns (items). ",
         "Found ", ncol(data), " column(s).", call. = FALSE)
  }
  
  if (nrow(data) < 1) {
    stop("'data' must contain at least 1 row (person). ",
         "Found 0 rows.", call. = FALSE)
  }
  
  non_numeric <- names(data)[!vapply(data, is.numeric, logical(1))]
  if (length(non_numeric) > 0) {
    stop("All columns must be numeric. Non-numeric column(s): ",
         paste(non_numeric, collapse = ", "), ". ",
         "Remove non-item columns (e.g., person IDs, grouping ",
         "variables) before passing to plot_bars().",
         call. = FALSE)
  }
  
  all_vals <- unlist(data, use.names = FALSE)
  all_vals <- all_vals[!is.na(all_vals)]
  if (length(all_vals) == 0) {
    stop("'data' contains only NA values.", call. = FALSE)
  }
  if (!all(all_vals == round(all_vals))) {
    stop("All response values must be integers. ",
         "Found non-integer values in the data.", call. = FALSE)
  }
  
  item_names <- names(data)
  
  if (!is.null(item_labels)) {
    if (length(item_labels) != length(item_names)) {
      stop("'item_labels' must have the same length as the number ",
           "of columns in 'data'. Expected ", length(item_names),
           ", got ", length(item_labels), ".", call. = FALSE)
    }
  }
  
  min_val <- min(all_vals)
  max_val <- max(all_vals)
  all_categories <- seq(min_val, max_val)
  n_categories <- length(all_categories)
  
  if (!is.null(category_labels)) {
    if (length(category_labels) != n_categories) {
      stop("'category_labels' must have the same length as the ",
           "number of response categories (", min_val, " to ",
           max_val, " = ", n_categories, " categories). Got ",
           length(category_labels), ".", call. = FALSE)
    }
  }
  
  # ================================================================
  # PREPARE DATA
  # ================================================================
  # Build facet labels: "itemnr - description" or just "itemnr"
  if (!is.null(item_labels)) {
    facet_labels <- paste0(item_names, " - ", item_labels)
  } else {
    facet_labels <- item_names
  }
  
  # Count responses per item x category, including zeros
  count_list <- list()
  for (i in seq_along(item_names)) {
    item <- item_names[i]
    item_vals <- data[[item]]
    item_vals <- item_vals[!is.na(item_vals)]
    tab <- table(factor(item_vals, levels = all_categories))
    total <- sum(tab)
    
    count_list[[item]] <- data.frame(
      item_name   = item,
      facet_label = facet_labels[i],
      category    = as.integer(names(tab)),
      n           = as.integer(tab),
      percent     = if (total > 0) {
        round(as.integer(tab) / total * 100, 1)
      } else {
        rep(0, length(tab))
      },
      stringsAsFactors = FALSE
    )
  }
  plot_df <- do.call(rbind, count_list)
  rownames(plot_df) <- NULL
  
  # Factor levels preserve column order
  plot_df$facet_label <- factor(
    plot_df$facet_label,
    levels = facet_labels
  )
  plot_df$category_fct <- factor(
    plot_df$category,
    levels = all_categories
  )
  
  # Apply category labels to the factor if provided
  if (!is.null(category_labels)) {
    levels(plot_df$category_fct) <- category_labels
  }
  
  # ================================================================
  # BUILD PLOT
  # ================================================================
  p <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes(x = .data$category_fct, y = .data$percent)
  ) +
    ggplot2::geom_col(
      ggplot2::aes(fill = .data$category_fct)
    ) +
    ggplot2::geom_text(
      ggplot2::aes(
        label = paste0("n = ", .data$n),
        color = .data$category_fct
      ),
      y = text_y
    ) +
    ggplot2::facet_wrap(
      ~ facet_label,
      ncol           = ncol,
      strip.position = "left",
      labeller       = ggplot2::labeller(
        facet_label = ggplot2::label_wrap_gen(label_wrap)
      )
    ) +
    ggplot2::scale_fill_viridis_d(guide = "none") +
    ggplot2::scale_color_viridis_d(
      guide     = "none",
      direction = -1,
      option    = viridis_option,
      end       = viridis_end
    ) +
    ggplot2::scale_y_continuous(position = "right") +
    ggplot2::labs(
      x = "Response category",
      y = "% of responses"
    ) +
    ggplot2::theme_bw(base_family = font) +
    ggplot2::theme(
      legend.position    = "none",
      strip.text.y.left  = ggplot2::element_text(angle = 0)
    )
  
  p
}

#' Stacked Bar Chart of Item Response Distributions
#'
#' Creates a horizontal stacked bar chart showing the response
#' distribution for all items. Each bar represents one item, with
#' segments colored by response category. Counts are displayed as
#' text labels within each segment. This is a descriptive data
#' visualization tool intended for use before model fitting.
#'
#' @param data A data frame in wide format containing only the item
#'   response columns. Each column is one item, each row is one
#'   person. All columns must be numeric (integer-valued). Response
#'   categories may be coded starting from 0 or 1. Do not include
#'   person IDs, grouping variables, or other non-item columns.
#' @param item_labels An optional character vector of descriptive
#'   labels for the items (y-axis). Must be the same length as
#'   \code{ncol(data)}. If \code{NULL} (the default), column names
#'   are used.
#' @param category_labels An optional character vector of labels
#'   for the response categories (legend). Must be the same length
#'   as the number of response categories spanning from the minimum
#'   to the maximum observed value, ordered from lowest to highest
#'   category. If \code{NULL} (the default), numeric category
#'   values are used.
#' @param show_n Logical. If \code{TRUE} (the default), the count
#'   of responses is displayed as a text label inside each bar
#'   segment.
#' @param show_percent Logical. If \code{TRUE}, the percentage of
#'   responses is displayed instead of (or in addition to) counts.
#'   Default is \code{FALSE}.
#' @param text_color Character. Color for the count/percentage
#'   labels. Default is \code{"sienna1"}.
#' @param text_size Numeric. Size of the count/percentage labels.
#'   Default is 3.
#' @param min_label_n Integer. Minimum count required for a label
#'   to be displayed within a bar segment. Segments with fewer
#'   responses are left unlabelled to avoid clutter. Default is 0
#'   (all segments labelled).
#' @param viridis_option Character. Viridis palette option. One of
#'   \code{"A"} through \code{"H"}. Default is \code{"D"}.
#' @param viridis_end Numeric in \eqn{[0, 1]}. End point of the
#'   viridis color scale. Default is 0.99.
#' @param title Character. Plot title. Default is
#'   \code{"Item responses"}.
#'
#' @return A \code{\link[ggplot2]{ggplot}} object.
#'
#' @details
#' Items are displayed on the y-axis in the same order as the
#' columns in \code{data} (first column at the top). Each bar is
#' divided into segments representing response categories, with
#' the lowest category on the left and the highest on the right.
#' The total bar length equals the number of non-missing responses
#' for that item.
#'
#' Categories with zero responses still appear in the legend but
#' produce no visible bar segment, which helps identify gaps in
#' the response distribution.
#'
#' \strong{Input requirements:}
#' \itemize{
#'   \item All columns must be numeric (integer-valued).
#'   \item The data frame must contain at least 2 columns (items)
#'     and at least 1 row (person).
#' }
#'
#' @examples
#' library(ggplot2)
#' if (requireNamespace("eRm", quietly = TRUE))
#'
#' # Basic stacked bar chart
#' plot_stackedbars(eRm::pcmdat2)
#'
#' # With custom item and category labels
#' plot_stackedbars(
#'   eRm::pcmdat2,
#'   item_labels = c("Mood", "Sleep", "Appetite", "Energy"),
#'   category_labels = c("Never", "Sometimes", "Often")
#' )
#'
#' # Show percentages, suppress small segments
#' plot_stackedbars(
#'   eRm::pcmdat2,
#'   show_percent = TRUE,
#'   show_n       = FALSE,
#'   min_label_n  = 5
#' )
#'
#' @importFrom rlang .data
#' @export
plot_stackedbars <- function(
    data,
    item_labels     = NULL,
    category_labels = NULL,
    show_n          = TRUE,
    show_percent    = FALSE,
    text_color      = "sienna1",
    text_size       = 3,
    min_label_n     = 0,
    viridis_option  = "D",
    viridis_end     = 0.99,
    title           = "Item responses"
) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  
  # ================================================================
  # INPUT VALIDATION
  # ================================================================
  if (!is.data.frame(data)) {
    stop("'data' must be a data frame in wide format ",
         "(one column per item, one row per person).",
         call. = FALSE)
  }
  
  if (ncol(data) < 2) {
    stop("'data' must contain at least 2 columns (items). ",
         "Found ", ncol(data), " column(s).", call. = FALSE)
  }
  
  if (nrow(data) < 1) {
    stop("'data' must contain at least 1 row (person). ",
         "Found 0 rows.", call. = FALSE)
  }
  
  non_numeric <- names(data)[!vapply(data, is.numeric, logical(1))]
  if (length(non_numeric) > 0) {
    stop("All columns must be numeric. Non-numeric column(s): ",
         paste(non_numeric, collapse = ", "), ". ",
         "Remove non-item columns (e.g., person IDs, grouping ",
         "variables) before passing to plot_stackedbars().",
         call. = FALSE)
  }
  
  all_vals <- unlist(data, use.names = FALSE)
  all_vals <- all_vals[!is.na(all_vals)]
  if (length(all_vals) == 0) {
    stop("'data' contains only NA values.", call. = FALSE)
  }
  if (!all(all_vals == round(all_vals))) {
    stop("All response values must be integers. ",
         "Found non-integer values in the data.", call. = FALSE)
  }
  
  item_names <- names(data)
  
  if (!is.null(item_labels)) {
    if (length(item_labels) != length(item_names)) {
      stop("'item_labels' must have the same length as the number ",
           "of columns in 'data'. Expected ", length(item_names),
           ", got ", length(item_labels), ".", call. = FALSE)
    }
  }
  
  min_val <- min(all_vals)
  max_val <- max(all_vals)
  all_categories <- seq(min_val, max_val)
  n_categories <- length(all_categories)
  
  if (!is.null(category_labels)) {
    if (length(category_labels) != n_categories) {
      stop("'category_labels' must have the same length as the ",
           "number of response categories (", min_val, " to ",
           max_val, " = ", n_categories, " categories). Got ",
           length(category_labels), ".", call. = FALSE)
    }
  }
  
  # ================================================================
  # PREPARE DATA
  # ================================================================
  # Build y-axis labels
  if (!is.null(item_labels)) {
    y_labels <- stats::setNames(item_labels, item_names)
  } else {
    y_labels <- stats::setNames(item_names, item_names)
  }
  
  # Count responses per item x category, including zeros
  count_list <- list()
  for (item in item_names) {
    item_vals <- data[[item]]
    item_vals <- item_vals[!is.na(item_vals)]
    tab <- table(factor(item_vals, levels = all_categories))
    total <- sum(tab)
    
    count_list[[item]] <- data.frame(
      item_name = item,
      category  = as.integer(names(tab)),
      n         = as.integer(tab),
      percent   = if (total > 0) {
        round(as.integer(tab) / total * 100, 1)
      } else {
        rep(0, length(tab))
      },
      stringsAsFactors = FALSE
    )
  }
  plot_df <- do.call(rbind, count_list)
  rownames(plot_df) <- NULL
  
  # Item factor: reversed so first column appears at top of y-axis
  plot_df$item_fct <- factor(
    plot_df$item_name,
    levels = rev(item_names),
    labels = rev(y_labels[item_names])
  )
  
  # Category factor: reversed for stacking order (lowest category
  # on the left = bottom of stack, which means it must be the
  # last factor level in geom_col's default stacking)
  plot_df$category_fct <- factor(
    plot_df$category,
    levels = rev(all_categories)
  )
  
  # Build text labels
  plot_df$label <- ""
  show_mask <- plot_df$n >= min_label_n
  
  if (show_n && show_percent) {
    plot_df$label[show_mask] <- paste0(
      plot_df$n[show_mask], " (", plot_df$percent[show_mask], "%)"
    )
  } else if (show_n) {
    plot_df$label[show_mask] <- as.character(plot_df$n[show_mask])
  } else if (show_percent) {
    plot_df$label[show_mask] <- paste0(
      plot_df$percent[show_mask], "%"
    )
  }
  
  # Suppress labels for zero-count segments (no visible bar)
  plot_df$label[plot_df$n == 0] <- ""
  
  # ================================================================
  # BUILD LEGEND LABELS
  # ================================================================
  # Legend should show categories from lowest to highest (top to
  # bottom), which is the reverse of the stacking factor
  if (!is.null(category_labels)) {
    legend_labels <- stats::setNames(
      rev(category_labels),
      rev(as.character(all_categories))
    )
  } else {
    legend_labels <- stats::setNames(
      rev(as.character(all_categories)),
      rev(as.character(all_categories))
    )
  }
  
  # ================================================================
  # BUILD PLOT
  # ================================================================
  p <- ggplot2::ggplot(
    plot_df,
    ggplot2::aes(
      x    = .data$n,
      y    = .data$item_fct,
      fill = .data$category_fct
    )
  ) +
    ggplot2::geom_col() +
    ggplot2::scale_fill_viridis_d(
      "Response\ncategory",
      direction = -1,
      option    = viridis_option,
      end       = viridis_end,
      labels    = legend_labels
    ) +
    ggplot2::labs(
      title = title,
      x     = "Number of responses",
      y     = NULL
    ) +
    ggplot2::theme_minimal() +
    ggplot2::theme(
      legend.position = "right"
    )
  
  # Add text labels if any content
  if (show_n || show_percent) {
    p <- p +
      ggplot2::geom_text(
        ggplot2::aes(label = .data$label),
        color    = text_color,
        size     = text_size,
        position = ggplot2::position_stack(vjust = 0.5)
      )
  }
  
  p
}

### The code below is borrowed from https://github.com/giac01/gbtoolbox/blob/main/R/reliability.R
### and slightly modified to be used with `RIreliabilityRMU()`.
### version used: https://github.com/giac01/gbtoolbox/commit/0e5c5d1547abc61c44f7c72c74c36d829241a33a

#' Estimate reliability (Relative Measurement Uncertainty) from Bayesian measurement models
#'
#' This function measures reliability using posterior draws from a fitted Bayesian model.
#'
#' To use this function, you will need to provide a matrix (input_draws) that 
#' contains the posterior draws for the parameter you wish to calculate reliability. 
#' The function assumes that rows of input_draws represent subjects and columns represent posterior draws.
#'
#' For an example of how to apply this function to calculate mean score reliability 
#' using brms, see 
#' \href{https://www.bignardi.co.uk/8_bayes_reliability/tutorial_rmu_sum_score_reliability.html}{this tutorial}.
#'
#' For an example of how to apply this function to go/go-no task data using brms, see 
#' \href{https://www.bignardi.co.uk/8_bayes_reliability/tutorial_calculating_rmu_gonogo.html}{this tutorial}.
#'
#' @param input_draws A matrix or data frame of posterior draws. Rows represent subjects and columns represent draws.
#' @param verbose Logical. Print detailed information about the input data. Default is TRUE.
#' @param level Numeric. Credibility level for the highest density continuous interval. Default is 0.95.
#'
#' @return A list containing:
#' \itemize{
#'   \item hdci: A data frame with a point-estimate (posterior mean) and highest density continuous interval for reliability, calculated using the ggdist::mean_hdci function
#'   \item reliability_posterior_draws: A numeric vector of posterior draws for reliability, of length K/2 (K = number of columns/draws in your input_draws matrix)
#' }
#'
#' @references
#' Bignardi, G., Kievit, R., & Bürkner, P. C. (2025). A general method for estimating reliability using Bayesian Measurement Uncertainty. PsyArXiv. \href{https://osf.io/preprints/psyarxiv/h54k8}{doi:10.31234/osf.io/h54k8}
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Dichotomous Rasch Model ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' # Extract posterior draws from brms model
#'
#' posterior_draws = fit_rm %>%
#'   as_draws_df() %>%
#'   as_tibble() %>% 
#'   select(starts_with("r_id")) %>%
#'   t()
#'
#' # Calculate RMU
#'
#' RMUreliability(posterior_draws)
#' }
#'
#' @importFrom ggdist mean_hdci
#' @export
RMUreliability <- function(
    input_draws,
    verbose = FALSE,
    level   = .95
) {
  if (!is.numeric(level) || level <= 0 || level >= 1) stop("level must be a numeric value between 0 and 1")

  input_draws <- as.matrix(input_draws)

  # Check for columns with zero SD
  sds <- apply(input_draws, 2, stats::sd, na.rm = TRUE)
  zero_sd_cols <- which(sds == 0)
  if (length(zero_sd_cols) > 0) {
    warning(sprintf("Found %d column(s) with zero standard deviation (columns: %s)",
                    length(zero_sd_cols),
                    paste(zero_sd_cols, collapse = ", ")))
  }

  # Check for NAs in input
  na_count <- base::sum(base::is.na(input_draws))
  if (na_count > 0) {
    warning(sprintf("Found %d NA value(s) in input_draws", na_count))
  }

  if (verbose) {
    base::print(paste0("Number of subjects: ", nrow(input_draws)))
    base::print(paste0("Number of posterior draws: ", ncol(input_draws)))
  }

  col_select <- base::sample(1:ncol(input_draws), replace = FALSE)
  input_draws_1 <- input_draws[, col_select[1:floor(length(col_select) / 2)]]
  input_draws_2 <- input_draws[, col_select[(floor(length(col_select) / 2) + 1):length(col_select)]]

  # Calculate correlations and handle NAs
  reliability_posterior_draws <- sapply(1:ncol(input_draws_1), function(i) {
    x <- input_draws_1[, i]
    y <- input_draws_2[, i]

    # Return 0 if either column has zero variance
    if (stats::var(x, na.rm = TRUE) == 0 || stats::var(y, na.rm = TRUE) == 0) {
      return(0)
    }

    # Calculate correlation and handle NA
    cor_val <- stats::cor(x, y, method = "pearson")
    return(cor_val)
  })

  hdci <- ggdist::mean_hdci(reliability_posterior_draws, .width = level)

  colnames(hdci)[1] = "rmu_estimate"
  colnames(hdci)[2] = "hdci_lowerbound"
  colnames(hdci)[3] = "hdci_upperbound"

  return(hdci)
}

#' Residual PCA Contrast Plot for Bayesian IRT Models
#'
#' Assesses dimensionality of Bayesian IRT models by performing a
#' principal component analysis (PCA) of standardized residuals for
#' each posterior draw. The item loadings on the first residual
#' contrast are plotted against item locations, with posterior
#' uncertainty displayed as 2D density contours, crosshairs, or
#' both. A posterior predictive p-value for the first-contrast
#' eigenvalue tests whether the observed residual structure exceeds
#' what the model predicts under unidimensionality.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from an
#'   ordinal IRT model (e.g., \code{family = acat}) or a dichotomous
#'   model (\code{family = bernoulli()}).
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data. Default is \code{id}.
#' @param center Logical. If \code{TRUE} (the default), item
#'   locations are mean-centered to zero, matching the convention
#'   used in \code{\link{plot_targeting}}.
#' @param prob Numeric in \eqn{(0, 1)}. Width of the credible
#'   intervals for both loading and location whiskers. Default is
#'   0.95.
#' @param ndraws_use Optional positive integer. Number of posterior
#'   draws to use. If \code{NULL} (the default), up to 500 draws
#'   are used.
#' @param style Character. Visual style for displaying uncertainty.
#'   \code{"density"} (the default) overlays filled 2D density
#'   contours per item computed from the draw-level location and
#'   loading values, showing the full joint posterior uncertainty.
#'   \code{"crosshair"} shows point estimates with horizontal and
#'   vertical credible interval bars.
#'   \code{"both"} displays density contours with crosshairs on top.
#' @param density_alpha Numeric in \eqn{[0, 1]}. Maximum opacity
#'   of the density contours when \code{style} is \code{"density"}
#'   or \code{"both"}. Default is 0.3.
#' @param density_bins Integer. Number of contour bins for
#'   \code{\link[ggplot2]{geom_density_2d_filled}}. Default is 6.
#' @param density_palette An optional character vector of colors for
#'   the density contour fills (from low to high density). If
#'   \code{NULL} (the default), a blue sequential ramp is used. The
#'   length should match \code{density_bins}; the lowest
#'   (background) level is always transparent.
#' @param label_items Logical. If \code{TRUE} (the default), item
#'   names are displayed next to points.
#' @param point_size Numeric. Size of the item points. Default is
#'   2.5.
#' @param point_color Color for the item points and error bars.
#'   Default is \code{"#0072B2"}.
#'
#' @return A list with three elements:
#' \describe{
#'   \item{plot}{A \code{\link[ggplot2]{ggplot}} object showing item
#'     loadings on the first residual contrast (y-axis) versus item
#'     locations (x-axis).}
#'   \item{data}{A \code{\link[tibble]{tibble}} with columns:
#'     \code{item}, \code{location} (posterior mean item location),
#'     \code{location_lower}, \code{location_upper} (location CI),
#'     \code{loading} (posterior mean loading on first contrast),
#'     \code{loading_lower}, \code{loading_upper} (loading CI).}
#'   \item{eigenvalue}{A \code{\link[tibble]{tibble}} with columns:
#'     \code{eigenvalue_obs} (posterior mean observed eigenvalue),
#'     \code{eigenvalue_rep} (posterior mean replicated eigenvalue),
#'     \code{eigenvalue_diff} (posterior mean difference),
#'     \code{ppp} (posterior predictive p-value),
#'     \code{var_explained_obs}, \code{var_explained_rep} (posterior
#'     mean proportions of residual variance explained).}
#' }
#'
#' @details
#' The procedure for each posterior draw \eqn{s} is:
#'
#' \enumerate{
#'   \item Obtain category probabilities from
#'     \code{\link[brms]{posterior_epred}}. Compute expected values
#'     \eqn{E^{(s)}_{vi}} and variances \eqn{Var^{(s)}_{vi}}.
#'   \item Compute standardized residuals for observed data:
#'     \deqn{z^{obs(s)}_{vi} = \frac{X_{vi} - E^{(s)}_{vi}}
#'       {\sqrt{Var^{(s)}_{vi}}}}
#'   \item Generate replicated data \eqn{Y^{rep(s)}} from
#'     \code{\link[brms]{posterior_predict}} and compute standardized
#'     residuals:
#'     \deqn{z^{rep(s)}_{vi} = \frac{Y^{rep(s)}_{vi} -
#'       E^{(s)}_{vi}}{\sqrt{Var^{(s)}_{vi}}}}
#'   \item Reshape both sets of residuals into person \eqn{\times}
#'     item matrices and perform SVD on each.
#'   \item Extract the first-contrast eigenvalue and item loadings
#'     from both observed and replicated SVDs.
#'   \item Compare eigenvalues across draws: the posterior predictive
#'     p-value \code{ppp = mean(eigenvalue_obs > eigenvalue_rep)}
#'     tests whether the observed residual structure is stronger than
#'     what the model produces under its own assumptions.
#' }
#'
#' When \code{style = "density"} or \code{style = "both"}, the
#' draw-level (location, loading) pairs for each item are used to
#' construct filled 2D kernel density contours via
#' \code{\link[ggplot2]{geom_density_2d_filled}}. The lowest
#' contour level (outside all contours) is set to transparent so
#' the white panel background shows through. Higher density regions
#' use progressively darker fills.
#'
#' Item loadings are aligned across draws using majority-sign
#' alignment to resolve the sign indeterminacy of eigenvectors.
#'
#' @references
#' Smith, E. V. (2002). Detecting and evaluating the impact of
#' multidimensionality using item fit statistics and principal
#' component analysis of residuals. \emph{Journal of Applied
#' Measurement}, \emph{3}, 205--231.
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
#' brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
#' 1--54. \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{plot_targeting}} for person-item maps,
#' \code{\link{plot_ipf}} for item category probability curves,
#' \code{\link{q3_statistic}} for Q3 residual correlations (another
#' local dependence / dimensionality diagnostic).
#'
#' @examples
#' \dontrun{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#' library(ggplot2)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4,
#'   cores  = 4,
#'   iter   = 2000
#' )
#'
#' # 2D density contours (default)
#' result <- plot_residual_pca(fit_pcm)
#' result$plot
#'
#' # Crosshair style
#' result_c <- plot_residual_pca(fit_pcm, style = "crosshair")
#' result_c$plot
#'
#' # Both combined
#' result_b <- plot_residual_pca(fit_pcm, style = "both")
#' result_b$plot
#'
#' # Custom warm palette
#' result_w <- plot_residual_pca(
#'   fit_pcm,
#'   density_palette = c("#FEE8C8", "#FDBB84", "#E34A33",
#'                       "#B30000", "#7F0000", "#4A0000"),
#'   point_color = "#B30000"
#' )
#' result_w$plot
#' }
#'
#' @importFrom brms posterior_epred posterior_predict ndraws as_draws_df
#' @importFrom rlang enquo as_name .data
#' @importFrom stats formula quantile complete.cases aggregate family
#' @importFrom tibble tibble
#' @importFrom grDevices colorRampPalette
#' @importFrom tibble as_tibble
#' @export
plot_residual_pca <- function(
    model,
    item_var = item,
    person_var = id,
    center = TRUE,
    prob = 0.95,
    ndraws_use = NULL,
    style = c("both", "density", "crosshair"),
    density_alpha = 0.3,
    density_bins = 6,
    density_palette = NULL,
    label_items = TRUE,
    point_size = 2.5,
    point_color = "#0072B2"
) {
  
  if (!requireNamespace("ggplot2", quietly = TRUE)) {
    stop("Package 'ggplot2' is required.", call. = FALSE)
  }
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  
  style <- match.arg(style)
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))
  
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  resp_var <- as.character(formula(model)$formula[[2]])
  if (length(resp_var) > 1) resp_var <- resp_var[2]
  
  lower_prob <- (1 - prob) / 2
  upper_prob <- 1 - lower_prob
  
  # --- Draw subset ---
  if (is.null(ndraws_use)) {
    ndraws_use <- min(brms::ndraws(model), 500)
  } else {
    ndraws_use <- min(as.integer(ndraws_use), brms::ndraws(model))
  }
  draw_ids <- sample(seq_len(brms::ndraws(model)), ndraws_use)
  
  # --- Posterior predictions ---
  epred_array <- brms::posterior_epred(model, draw_ids = draw_ids)
  yrep_mat    <- brms::posterior_predict(model, draw_ids = draw_ids)
  
  n_draws <- dim(epred_array)[1]
  n_obs   <- dim(epred_array)[2]
  obs_response <- model$data[[resp_var]]
  
  # =================================================================
  # Compute E_mat and Var_mat — VECTORIZED
  # =================================================================
  if (length(dim(epred_array)) == 3) {
    n_cat <- dim(epred_array)[3]
    cat_values <- seq_len(n_cat)
    dim_orig <- dim(epred_array)
    
    # Reshape [S x N x C] -> [S*N x C], then matrix-multiply by cat_values
    ep_2d <- matrix(epred_array, nrow = dim_orig[1] * dim_orig[2],
                    ncol = dim_orig[3])
    E_vec <- ep_2d %*% cat_values                # [S*N x 1]
    E_mat <- matrix(E_vec, nrow = n_draws, ncol = n_obs)
    
    # Var = sum_c (c - E)^2 * P(c) — fully vectorized
    E_expanded <- E_vec[, rep(1L, n_cat)]         # [S*N x C]
    cat_mat <- matrix(cat_values, nrow = nrow(ep_2d), ncol = n_cat,
                      byrow = TRUE)
    Var_vec <- rowSums((cat_mat - E_expanded)^2 * ep_2d)
    Var_mat <- matrix(Var_vec, nrow = n_draws, ncol = n_obs)
  } else {
    E_mat   <- epred_array
    Var_mat <- epred_array * (1 - epred_array)
  }
  
  Var_mat[Var_mat < 1e-12] <- 1e-12
  sd_mat <- sqrt(Var_mat)
  
  # =================================================================
  # Standardized residuals [S x N] — vectorized
  # =================================================================
  obs_mat       <- matrix(obs_response, nrow = n_draws, ncol = n_obs,
                          byrow = TRUE)
  std_resid_obs <- (obs_mat - E_mat) / sd_mat
  std_resid_rep <- (yrep_mat - E_mat) / sd_mat
  
  # --- Map observations to person x item positions ---
  items   <- model$data[[item_name]]
  persons <- model$data[[person_name]]
  unique_items   <- unique(items)
  unique_persons <- unique(persons)
  k <- length(unique_items)
  n_persons <- length(unique_persons)
  
  person_idx <- match(persons, unique_persons)
  item_idx   <- match(items, unique_items)
  
  # Pre-compute linear index for fast scatter into person x item matrix
  lin_idx <- person_idx + (item_idx - 1L) * n_persons
  
  # =================================================================
  # PCA per draw — vectorized scatter + SVD
  # =================================================================
  loadings_obs_draws <- matrix(NA_real_, nrow = n_draws, ncol = k)
  eigen_obs_draws    <- rep(NA_real_, n_draws)
  eigen_rep_draws    <- rep(NA_real_, n_draws)
  varexp_obs_draws   <- rep(NA_real_, n_draws)
  varexp_rep_draws   <- rep(NA_real_, n_draws)
  
  for (s in seq_len(n_draws)) {
    # Scatter residuals into person x item matrix via linear index
    resid_obs_wide <- matrix(NA_real_, nrow = n_persons, ncol = k)
    resid_rep_wide <- matrix(NA_real_, nrow = n_persons, ncol = k)
    resid_obs_wide[lin_idx] <- std_resid_obs[s, ]
    resid_rep_wide[lin_idx] <- std_resid_rep[s, ]
    
    # Drop rows with any NA (persons who didn't answer all items)
    complete <- stats::complete.cases(resid_obs_wide)
    if (sum(complete) < 3) next
    
    resid_obs_c <- resid_obs_wide[complete, , drop = FALSE]
    resid_rep_c <- resid_rep_wide[complete, , drop = FALSE]
    
    # SVD (nu = 0, nv = 1 for speed — we only need first right singular vector)
    sv_obs <- svd(resid_obs_c, nu = 0, nv = 1)
    sv_rep <- svd(resid_rep_c, nu = 0, nv = 1)
    
    loadings_obs_draws[s, ] <- sv_obs$v[, 1]
    eigen_obs_draws[s] <- sv_obs$d[1]^2 / (nrow(resid_obs_c) - 1)
    total_var_obs <- sum(sv_obs$d^2) / (nrow(resid_obs_c) - 1)
    varexp_obs_draws[s] <- eigen_obs_draws[s] / total_var_obs
    
    eigen_rep_draws[s] <- sv_rep$d[1]^2 / (nrow(resid_rep_c) - 1)
    total_var_rep <- sum(sv_rep$d^2) / (nrow(resid_rep_c) - 1)
    varexp_rep_draws[s] <- eigen_rep_draws[s] / total_var_rep
  }
  
  # =================================================================
  # Resolve sign indeterminacy
  # =================================================================
  ref_idx <- which(!is.na(loadings_obs_draws[, 1]))[1]
  if (is.na(ref_idx)) {
    stop("Could not compute residual PCA for any draw.", call. = FALSE)
  }
  ref_sign <- sign(loadings_obs_draws[ref_idx, ])
  
  for (s in seq_len(n_draws)) {
    if (is.na(loadings_obs_draws[s, 1])) next
    agreement <- sum(sign(loadings_obs_draws[s, ]) == ref_sign,
                     na.rm = TRUE)
    if (agreement < k / 2) {
      loadings_obs_draws[s, ] <- -loadings_obs_draws[s, ]
    }
  }
  
  # =================================================================
  # Item locations with full posterior (draw-level)
  # =================================================================
  draws_df <- tibble::as_tibble(brms::as_draws_df(model))
  family_name <- stats::family(model)$family
  is_ordinal <- grepl("acat|cumul|sratio|cratio",
                      family_name, ignore.case = TRUE)
  
  loc_draws_list <- .extract_item_location_draws(
    draws_df, model, unique_items, item_name, person_name, is_ordinal
  )
  
  loc_draws_mat <- matrix(NA_real_, nrow = n_draws, ncol = k)
  for (i in seq_len(k)) {
    loc_draws_mat[, i] <- loc_draws_list[[i]][draw_ids]
  }
  
  if (center) {
    shift_per_draw <- rowMeans(loc_draws_mat, na.rm = TRUE)
    loc_draws_mat <- loc_draws_mat - shift_per_draw
  }
  
  # Summarize locations
  loc_mean  <- colMeans(loc_draws_mat, na.rm = TRUE)
  loc_lower <- apply(loc_draws_mat, 2, stats::quantile,
                     probs = lower_prob, na.rm = TRUE)
  loc_upper <- apply(loc_draws_mat, 2, stats::quantile,
                     probs = upper_prob, na.rm = TRUE)
  
  # --- Summarize loadings ---
  loading_mean  <- colMeans(loadings_obs_draws, na.rm = TRUE)
  loading_lower <- apply(loadings_obs_draws, 2, stats::quantile,
                         probs = lower_prob, na.rm = TRUE)
  loading_upper <- apply(loadings_obs_draws, 2, stats::quantile,
                         probs = upper_prob, na.rm = TRUE)
  
  # =================================================================
  # Eigenvalue posterior predictive comparison
  # =================================================================
  valid <- !is.na(eigen_obs_draws) & !is.na(eigen_rep_draws) &
    eigen_obs_draws > 0 & eigen_rep_draws > 0
  ppp_eigen <- mean(eigen_obs_draws[valid] > eigen_rep_draws[valid])
  
  eigenvalue_summary <- tibble::tibble(
    eigenvalue_obs    = mean(eigen_obs_draws[valid]),
    eigenvalue_rep    = mean(eigen_rep_draws[valid]),
    eigenvalue_diff   = mean(eigen_obs_draws[valid] -
                               eigen_rep_draws[valid]),
    ppp               = ppp_eigen,
    var_explained_obs = mean(varexp_obs_draws[valid]),
    var_explained_rep = mean(varexp_rep_draws[valid])
  )
  
  # =================================================================
  # Build summary output data
  # =================================================================
  plot_df <- tibble::tibble(
    item           = unique_items,
    location       = loc_mean,
    location_lower = loc_lower,
    location_upper = loc_upper,
    loading        = loading_mean,
    loading_lower  = loading_lower,
    loading_upper  = loading_upper
  )
  
  # --- Build draw-level long data for density plots ---
  valid_draws <- which(!is.na(loadings_obs_draws[, 1]))
  draws_long_list <- vector("list", k)
  for (i in seq_len(k)) {
    draws_long_list[[i]] <- data.frame(
      item     = unique_items[i],
      location = loc_draws_mat[valid_draws, i],
      loading  = loadings_obs_draws[valid_draws, i],
      stringsAsFactors = FALSE
    )
  }
  draws_long <- do.call(rbind, draws_long_list)
  
  # =================================================================
  # Density fill palette
  # =================================================================
  # First value is always transparent (region outside all contours)
  # Remaining values are the actual contour fills (low -> high density)
  if (is.null(density_palette)) {
    contour_colors <- grDevices::colorRampPalette(
      c("#DCEEF8", "#88BEDC", "#3A8EC2", "#0A5C96", "#003560")
    )(density_bins)
  } else {
    if (length(density_palette) < density_bins) {
      contour_colors <- grDevices::colorRampPalette(
        density_palette
      )(density_bins)
    } else {
      contour_colors <- density_palette[seq_len(density_bins)]
    }
  }
  fill_values <- c("transparent", contour_colors)
  
  # =================================================================
  # Build plot
  # =================================================================
  subtitle_text <- paste0(
    "First contrast eigenvalue: observed = ",
    round(eigenvalue_summary$eigenvalue_obs, 2),
    ", replicated = ",
    round(eigenvalue_summary$eigenvalue_rep, 2),
    ", ppp = ",
    round(eigenvalue_summary$ppp, 3)
  )
  
  p <- ggplot2::ggplot(plot_df, ggplot2::aes(x = .data$location,
                                             y = .data$loading))
  
  # Density layer
  if (style %in% c("density", "both")) {
    p <- p +
      ggplot2::geom_density_2d_filled(
        data = draws_long,
        ggplot2::aes(x = .data$location, y = .data$loading,
                     group = .data$item),
        alpha = density_alpha,
        bins = density_bins,
        show.legend = FALSE
      ) +
      ggplot2::scale_fill_manual(
        values = fill_values,
        guide  = "none"
      )
  }
  
  # Zero lines
  p <- p +
    ggplot2::geom_hline(
      yintercept = 0, linetype = "dashed", colour = "grey50"
    ) +
    ggplot2::geom_vline(
      xintercept = 0, linetype = "dashed", colour = "grey50"
    )
  
  # Crosshair layer
  if (style %in% c("crosshair", "both")) {
    crosshair_alpha <- 1
    p <- p +
      ggplot2::geom_errorbar(
        ggplot2::aes(xmin = .data$location_lower,
                     xmax = .data$location_upper),
        width = 0, colour = point_color, alpha = crosshair_alpha,
        linewidth = 0.4,
        orientation = "y"
      ) +
      ggplot2::geom_linerange(
        ggplot2::aes(ymin = .data$loading_lower,
                     ymax = .data$loading_upper),
        colour = point_color, alpha = crosshair_alpha,
        linewidth = 0.4
      )
  }
  
  # Points
  p <- p +
    ggplot2::geom_point(size = point_size, colour = point_color)
  
  # Labels
  if (label_items) {
    if (requireNamespace("ggrepel", quietly = TRUE)) {
      p <- p +
        ggrepel::geom_text_repel(
          ggplot2::aes(label = .data$item),
          size = 3, colour = "grey30",
          max.overlaps = Inf,
          seed = 42
        )
    } else {
      p <- p +
        ggplot2::geom_text(
          ggplot2::aes(label = .data$item),
          size = 3, colour = "grey30",
          nudge_y = 0.02
        )
    }
  }
  
  p <- p +
    ggplot2::labs(
      x        = "Item location (logits)",
      y        = "Loading on first residual contrast",
      subtitle = subtitle_text
    ) +
    ggplot2::theme_bw() +
    ggplot2::theme(
      panel.background = ggplot2::element_rect(fill = "white"),
      plot.background  = ggplot2::element_rect(fill = "white",
                                               colour = NA),
      legend.position  = "none"
    )
  
  list(
    plot       = p,
    data       = plot_df,
    eigenvalue = eigenvalue_summary
  )
}


# ── Internal: extract draw-level item locations ──────────────────
#' @keywords internal
.extract_item_location_draws <- function(draws, model, unique_items,
                                         item_name, person_name,
                                         is_ordinal) {
  
  result <- vector("list", length(unique_items))
  names(result) <- unique_items
  
  if (is_ordinal) {
    has_grouped <- any(grepl("^b_Intercept\\[.+,\\d+\\]$", names(draws)))
    
    for (i in seq_along(unique_items)) {
      item_label <- unique_items[i]
      
      if (has_grouped) {
        thresh_pattern <- paste0(
          "^b_Intercept\\[",
          gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
          ","
        )
        thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)
      } else {
        thresh_cols <- grep("^b_Intercept\\[\\d+\\]$", names(draws),
                            value = TRUE)
      }
      
      if (length(thresh_cols) == 0) {
        result[[i]] <- rep(NA_real_, nrow(draws))
        next
      }
      
      thresh_mat <- as.matrix(draws[, thresh_cols, drop = FALSE])
      result[[i]] <- rowMeans(thresh_mat)
    }
  } else {
    intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
    intercept_draws <- draws[[intercept_col]]
    
    for (i in seq_along(unique_items)) {
      item_label <- unique_items[i]
      
      re_col <- paste0("r_", item_name, "[", item_label, ",Intercept]")
      if (!re_col %in% names(draws)) {
        re_col_alt <- grep(
          paste0("^r_", item_name, "\\[",
                 gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
                 ",Intercept\\]$"),
          names(draws), value = TRUE
        )
        if (length(re_col_alt) == 1) re_col <- re_col_alt
        else {
          result[[i]] <- rep(NA_real_, nrow(draws))
          next
        }
      }
      
      result[[i]] <- -(intercept_draws + draws[[re_col]])
    }
  }
  
  result
}


# ── Internal: extract threshold data (used by plot_targeting) ──────────
#' @keywords internal
.extract_threshold_data <- function(draws, model, unique_items,
                                    item_name, person_name,
                                    is_ordinal, lower_prob, upper_prob) {
  
  result_list <- list()
  
  if (is_ordinal) {
    has_grouped <- any(grepl("^b_Intercept\\[.+,\\d+\\]$", names(draws)))
    
    for (item_label in unique_items) {
      if (has_grouped) {
        thresh_pattern <- paste0(
          "^b_Intercept\\[",
          gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
          ","
        )
        thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)
      } else {
        thresh_cols <- grep("^b_Intercept\\[\\d+\\]$", names(draws),
                            value = TRUE)
      }
      
      if (length(thresh_cols) == 0) next
      
      thresh_nums <- as.numeric(
        gsub(".*,(\\d+)\\]$|.*\\[(\\d+)\\]$", "\\1\\2", thresh_cols)
      )
      thresh_cols <- thresh_cols[order(thresh_nums)]
      
      for (idx in seq_along(thresh_cols)) {
        vals <- draws[[thresh_cols[idx]]]
        result_list[[length(result_list) + 1]] <- data.frame(
          item     = item_label,
          category = as.character(idx),
          estimate = mean(vals),
          lower    = stats::quantile(vals, probs = lower_prob),
          upper    = stats::quantile(vals, probs = upper_prob),
          stringsAsFactors = FALSE
        )
      }
    }
  } else {
    intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
    if (length(intercept_col) == 0) {
      stop("Could not find intercept parameter.", call. = FALSE)
    }
    
    for (item_label in unique_items) {
      re_col <- paste0("r_", item_name, "[", item_label, ",Intercept]")
      if (!re_col %in% names(draws)) {
        re_col_alt <- grep(
          paste0("^r_", item_name, "\\[",
                 gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
                 ",Intercept\\]$"),
          names(draws), value = TRUE
        )
        if (length(re_col_alt) == 1) {
          re_col <- re_col_alt
        } else {
          warning("Could not find random effect for item '",
                  item_label, "'. Skipping.", call. = FALSE)
          next
        }
      }
      
      item_location <- -(draws[[intercept_col]] + draws[[re_col]])
      
      result_list[[length(result_list) + 1]] <- data.frame(
        item     = item_label,
        category = "1",
        estimate = mean(item_location),
        lower    = stats::quantile(item_location, probs = lower_prob),
        upper    = stats::quantile(item_location, probs = upper_prob),
        stringsAsFactors = FALSE
      )
    }
  }
  
  result <- do.call(rbind, result_list)
  rownames(result) <- NULL
  result
}

#' Posterior Predictive Q3 Residual Correlations for Bayesian IRT Models
#'
#' Computes a Bayesian analogue of Yen's Q3 statistic (Yen, 1984) for
#' detecting local dependence between item pairs in Rasch-family models
#' fitted with \pkg{brms}. For each posterior draw, residual correlations
#' are computed for both observed and replicated data, yielding draw-level
#' Q3 values that can be summarized and visualized via
#' \code{\link{q3_post}}.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from an
#'   ordinal IRT model (e.g., \code{family = acat} for a partial credit
#'   model) or a dichotomous model (\code{family = bernoulli()}).
#' @param item_var An unquoted variable name identifying the item grouping
#'   variable in the model data. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data. Default is \code{id}.
#' @param ndraws_use Optional positive integer. If specified, a random
#'   subset of posterior draws of this size is used. If \code{NULL} (the
#'   default), all draws are used.
#'
#' @return A \code{\link[tibble]{tibble}} in long format with one row
#'   per draw per item pair, containing:
#' \describe{
#'   \item{draw}{Integer draw index.}
#'   \item{item_pair}{Character label of the item pair
#'     (\code{"item1 : item2"}).}
#'   \item{item_1}{First item in the pair.}
#'   \item{item_2}{Second item in the pair.}
#'   \item{q3}{Observed Q3 residual correlation for this draw.}
#'   \item{q3_rep}{Replicated Q3 residual correlation for this draw.}
#' }
#'
#' This long-format output parallels the structure of
#' \code{\link{infit_statistic}} and can be passed directly to
#' \code{\link{q3_post}} for summary tables and plots.
#'
#' @details
#' The procedure works as follows for each posterior draw \eqn{s}:
#'
#' \enumerate{
#'   \item Compute expected values \eqn{E^{(s)}_{vi}} from the category
#'     probabilities returned by \code{\link[brms]{posterior_epred}}.
#'     For ordinal models: \eqn{E^{(s)}_{vi} = \sum_c c \cdot
#'     P^{(s)}(X_{vi} = c)}. For binary models: \eqn{E^{(s)}_{vi} =
#'     P^{(s)}(X_{vi} = 1)}.
#'   \item Compute observed residuals: \eqn{d^{(s)}_{vi} = X_{vi} -
#'     E^{(s)}_{vi}}.
#'   \item Compute replicated residuals: \eqn{d^{rep(s)}_{vi} =
#'     Y^{rep(s)}_{vi} - E^{(s)}_{vi}}, where \eqn{Y^{rep}} is drawn
#'     via \code{\link[brms]{posterior_predict}}.
#'   \item For each item pair \eqn{(i, j)}, compute Q3 as the Pearson
#'     correlation of residuals across all persons who responded to
#'     both items.
#' }
#'
#' @references
#' Yen, W. M. (1984). Effects of local item dependence on the fit and
#' equating performance of the three-parameter logistic model.
#' \emph{Applied Psychological Measurement}, \emph{8}(2), 125--145.
#' \doi{10.1177/014662168400800201}
#'
#' Christensen, K. B., Makransky, G. & Horton, M. (2017). Critical
#' values for Yen's Q3: Identification of local dependence in the
#' Rasch model using residual correlations.
#' \emph{Applied Psychological Measurement}, \emph{41}(3), 178--194.
#' \doi{10.1177/0146621616677520}
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
#' brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
#' 1--54. \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{q3_post}} for postprocessing summaries and plots,
#' \code{\link{infit_statistic}} for Bayesian infit/outfit,
#' \code{\link[brms]{posterior_epred}},
#' \code{\link[brms]{posterior_predict}}.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' # Q3 residual correlations
#' q3_draws <- q3_statistic(fit_pcm, ndraws_use = 500)
#'
#' # Postprocess
#' result <- q3_post(q3_draws)
#' result$summary
#' result$hdi
#' result$plot
#'
#' # --- Dichotomous Rasch Model ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' q3_draws <- q3_statistic(fit_rm, ndraws_use = 500)
#'
#' # Postprocess
#' result <- q3_post(q3_draws)
#' result$summary
#' result$hdi
#' result$plot
#' }
#'
#' @importFrom brms posterior_epred posterior_predict ndraws
#' @importFrom rlang enquo as_name .data
#' @importFrom stats formula cor
#' @importFrom tibble tibble
#' @export
q3_statistic <- function(model, item_var = item, person_var = id,
                         ndraws_use = NULL) {
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))
  
  resp_var <- as.character(formula(model)$formula[[2]])
  if (length(resp_var) > 1) resp_var <- resp_var[2]
  
  if (!resp_var %in% names(model$data)) {
    stop("Response variable '", resp_var, "' not found in model data.",
         call. = FALSE)
  }
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  # --- Determine posterior draw subset ---
  draw_ids <- NULL
  total_draws <- brms::ndraws(model)
  if (!is.null(ndraws_use)) {
    ndraws_use <- as.integer(ndraws_use)
    if (ndraws_use < 1) {
      stop("'ndraws_use' must be a positive integer.", call. = FALSE)
    }
    if (ndraws_use > total_draws) {
      warning("'ndraws_use' (", ndraws_use, ") exceeds available draws (",
              total_draws, "). Using all draws.", call. = FALSE)
      ndraws_use <- total_draws
    }
    draw_ids <- sample(seq_len(total_draws), ndraws_use)
  }
  
  # --- Posterior predictions ---
  epred_array <- brms::posterior_epred(model, draw_ids = draw_ids)
  yrep_mat    <- brms::posterior_predict(model, draw_ids = draw_ids)
  
  n_draws <- dim(epred_array)[1]
  n_obs   <- dim(epred_array)[2]
  obs_response <- model$data[[resp_var]]
  
  # --- Expected values ---
  if (length(dim(epred_array)) == 3) {
    n_cat <- dim(epred_array)[3]
    cat_values <- seq_len(n_cat)
    dim_orig <- dim(epred_array)
    ep_2d <- matrix(epred_array, nrow = dim_orig[1] * dim_orig[2],
                    ncol = dim_orig[3])
    E_vec <- ep_2d %*% cat_values
    E_mat <- matrix(E_vec, nrow = n_draws, ncol = n_obs)
  } else {
    E_mat <- epred_array
  }
  
  # --- Residuals ---
  obs_mat   <- matrix(obs_response, nrow = n_draws, ncol = n_obs,
                      byrow = TRUE)
  resid_obs <- obs_mat - E_mat
  resid_rep <- yrep_mat - E_mat
  
  # --- Person x item mapping ---
  items   <- model$data[[item_name]]
  persons <- model$data[[person_name]]
  unique_items   <- sort(unique(items))
  unique_persons <- sort(unique(persons))
  k <- length(unique_items)
  n_persons <- length(unique_persons)
  
  person_idx <- match(persons, unique_persons)
  item_idx   <- match(items, unique_items)
  lin_idx <- person_idx + (item_idx - 1L) * n_persons
  
  # --- Item pair indices (upper triangle) ---
  pair_grid <- expand.grid(j = seq_len(k), i = seq_len(k))
  pair_grid <- pair_grid[pair_grid$i < pair_grid$j, ]
  pair_grid <- pair_grid[order(pair_grid$i, pair_grid$j), ]
  n_pairs <- nrow(pair_grid)
  
  pair_labels <- paste0(unique_items[pair_grid$i], " : ",
                        unique_items[pair_grid$j])
  
  # --- Q3 per draw ---
  q3_obs_draws <- matrix(NA_real_, nrow = n_draws, ncol = n_pairs)
  q3_rep_draws <- matrix(NA_real_, nrow = n_draws, ncol = n_pairs)
  
  for (s in seq_len(n_draws)) {
    resid_obs_wide <- matrix(NA_real_, nrow = n_persons, ncol = k)
    resid_rep_wide <- matrix(NA_real_, nrow = n_persons, ncol = k)
    resid_obs_wide[lin_idx] <- resid_obs[s, ]
    resid_rep_wide[lin_idx] <- resid_rep[s, ]
    
    cor_obs <- stats::cor(resid_obs_wide, use = "pairwise.complete.obs")
    cor_rep <- stats::cor(resid_rep_wide, use = "pairwise.complete.obs")
    
    for (p in seq_len(n_pairs)) {
      q3_obs_draws[s, p] <- cor_obs[pair_grid$i[p], pair_grid$j[p]]
      q3_rep_draws[s, p] <- cor_rep[pair_grid$i[p], pair_grid$j[p]]
    }
  }
  
  # --- Build long-format output ---
  result_list <- vector("list", n_pairs)
  for (p in seq_len(n_pairs)) {
    result_list[[p]] <- tibble::tibble(
      draw      = seq_len(n_draws),
      item_pair = pair_labels[p],
      item_1    = unique_items[pair_grid$i[p]],
      item_2    = unique_items[pair_grid$j[p]],
      q3        = q3_obs_draws[, p],
      q3_rep    = q3_rep_draws[, p]
    )
  }
  
  result <- do.call(rbind, result_list)
  tibble::as_tibble(result)
}

#' Posterior Predictive Item Fit Statistic for Bayesian IRT Models
#'
#' Computes posterior predictive item (or person) fit statistics for
#' Bayesian IRT models fitted with \pkg{brms}. For each posterior draw,
#' observed and replicated data are compared via a user-supplied criterion
#' function, grouped by item, person, or any other variable. Posterior
#' predictive p-values can then be derived from the output to assess fit.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object.
#' @param criterion A function with signature \code{function(y, p)} that
#'   computes a pointwise fit criterion. For ordinal and categorical models,
#'   \code{y} is the observed (or replicated) response category and \code{p}
#'   is the model-predicted probability of that category. For binary models,
#'   \code{y} is the binary response and \code{p} is the predicted probability
#'   of success.
#' @param group An unquoted variable name (e.g., \code{item} or \code{id})
#'   indicating the grouping variable over which the fit statistic is
#'   aggregated. Typically \code{item} for item fit or \code{id} for person
#'   fit.
#' @param ndraws_use Optional positive integer. If specified, a random subset
#'   of posterior draws of this size is used, which can speed up computation
#'   for large models. If \code{NULL} (the default), all draws are used.
#'
#' @return A \code{\link[tibble]{tibble}} with the following columns:
#' \describe{
#'   \item{\code{group}}{The grouping variable (e.g., item name or person id).}
#'   \item{draw}{Integer index of the posterior draw.}
#'   \item{crit}{The observed fit statistic (criterion applied to observed
#'     data) summed within each group and draw.}
#'   \item{crit_rep}{The replicated fit statistic (criterion applied to
#'     posterior predicted data) summed within each group and draw.}
#'   \item{crit_diff}{The difference \code{crit_rep - crit}.}
#' }
#' The output is grouped by the grouping variable. Posterior predictive
#' p-values can be obtained by computing
#' \code{mean(crit_rep > crit)} within each group.
#'
#' @details
#' The function implements the posterior predictive checking approach for
#' item fit described in Bürkner (2020). The procedure works as follows:
#'
#' \enumerate{
#'   \item Draw posterior expected category probabilities via
#'     \code{\link[brms]{posterior_epred}} and posterior predicted responses
#'     via \code{\link[brms]{posterior_predict}}.
#'   \item For ordinal or categorical models (3D array output from
#'     \code{posterior_epred}), extract the probability assigned to the
#'     observed response category and to the replicated response category
#'     for each draw and observation.
#'   \item Apply the user-supplied \code{criterion} function to compute
#'     pointwise fit values for both observed and replicated data.
#'   \item Aggregate (sum) the criterion values within each level of
#'     \code{group} and each posterior draw.
#' }
#'
#' A common choice for ordinal IRT models is the categorical
#' log-likelihood criterion \code{function(y, p) log(p)}. For binary
#' (e.g., dichotomous Rasch) models, the Bernoulli log-likelihood
#' \code{function(y, p) y * log(p) + (1 - y) * log(1 - p)} may be used
#' instead.
#'
#' @references
#' Bürkner, P.-C. (2020). Analysing Standard Progressive Matrices (SPM-LS)
#' with Bayesian Item Response Models. \emph{Journal of Intelligence},
#' \emph{8}(1). \doi{10.3390/jintelligence8010005}
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with brms and
#' Stan. \emph{Journal of Statistical Software}, \emph{100}, 1--54.
#' \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{fit_statistic_rm}} for dichotomous Rasch models,
#' \code{\link[brms]{posterior_epred}} for expected predictions,
#' \code{\link[brms]{posterior_predict}} for posterior predictive samples,
#' \code{\link[brms]{pp_check}} for graphical posterior predictive checks.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Polytomous Rasch (Partial Credit Model) ---
#'
#' # Prepare data in long format
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' # Fit a Partial Credit Model using the adjacent category family
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data    = df_pcm,
#'   family  = acat,
#'   chains  = 4,
#'   cores   = 1, # use more cores if you have
#'   iter    = 500 # use at least 2000 
#' )
#'
#' # Categorical log-likelihood criterion (for polytomous models)
#' ll_categorical <- function(y, p) log(p)
#'
#' # Compute item fit statistics
#' item_fit <- fit_statistic_pcm(
#'   model      = fit_pcm,
#'   criterion  = ll_categorical,
#'   group      = item,
#'   ndraws_use = 100 # use at least 500
#' )
#'
#' # Summarise: posterior predictive p-values per item
#' item_fit %>%
#'   group_by(item) %>%
#'   summarise(
#'     observed   = mean(crit),
#'     replicated = mean(crit_rep),
#'     ppp        = mean(crit_rep > crit)
#'   )
#'
#' # Use ggplot2 to make a histogram
#' library(ggplot2)
#' item_fit %>%
#'   ggplot(aes(crit_diff)) +
#'   geom_histogram(aes(fill = ifelse(crit_diff > 0, "above","below"))) +
#'   facet_wrap("item") +
#'   theme_bw() +
#'   theme(legend.position = "none")
#'
#' # Compute person fit statistics
#' person_fit <- fit_statistic_pcm(
#'   model      = fit_pcm,
#'   criterion  = ll_categorical,
#'   group      = id,
#'   ndraws_use = 100 # use at least 500
#' )
#'}
#'
#' @importFrom brms posterior_epred posterior_predict ndraws
#' @importFrom dplyr mutate left_join group_by summarise arrange
#' @importFrom tidyr pivot_longer
#' @importFrom rlang enquo !! .data
#' @importFrom stats formula setNames
#' @export
fit_statistic_pcm <- function(model, criterion, group, ndraws_use = NULL) {
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  if (!is.function(criterion)) {
    stop("'criterion' must be a function with signature function(y, p).",
         call. = FALSE)
  }

  group <- rlang::enquo(group)


  # --- Extract response variable name from the model formula ---
  resp_var <- as.character(formula(model)$formula[[2]])
  # For formulas with addition terms (e.g., response | thres(gr = item)),

  # the LHS is a call and as.character() returns a vector; the actual
  # variable name is the second element.
  if (length(resp_var) > 1) {
    resp_var <- resp_var[2]
  }
  if (!resp_var %in% names(model$data)) {
    stop("Response variable '", resp_var, "' not found in model data.",
         call. = FALSE)
  }


  # --- Determine posterior draw subset ---
  draw_ids <- NULL
  if (!is.null(ndraws_use)) {
    ndraws_use <- as.integer(ndraws_use)
    if (ndraws_use < 1) {
      stop("'ndraws_use' must be a positive integer.", call. = FALSE)
    }
    total_draws <- brms::ndraws(model)
    if (ndraws_use > total_draws) {
      warning("'ndraws_use' (", ndraws_use, ") exceeds available draws (",
              total_draws, "). Using all draws.", call. = FALSE)
      ndraws_use <- total_draws
    }
    draw_ids <- sample(seq_len(total_draws), ndraws_use)
  }

  # --- Posterior expected predictions and predicted responses ---
  epred_array <- brms::posterior_epred(model, draw_ids = draw_ids)
  yrep_mat    <- brms::posterior_predict(model, draw_ids = draw_ids)

  n_draws <- dim(epred_array)[1]
  n_obs   <- dim(epred_array)[2]

  # --- Compute pointwise predicted probabilities ---
  if (length(dim(epred_array)) == 3) {
    # Ordinal/categorical model: epred_array is S x N x C
    obs_response <- model$data[[resp_var]]

    # p(observed category) per draw and observation
    ppe_mat <- matrix(NA_real_, nrow = n_draws, ncol = n_obs)
    for (n in seq_len(n_obs)) {
      ppe_mat[, n] <- epred_array[, n, obs_response[n]]
    }

    # p(replicated category) per draw and observation
    yrep_ppe_mat <- matrix(NA_real_, nrow = n_draws, ncol = n_obs)
    for (n in seq_len(n_obs)) {
      yrep_ppe_mat[, n] <- epred_array[
        cbind(seq_len(n_draws), n, yrep_mat[, n])
      ]
    }
  } else {
    # Binary/continuous model: epred_array is S x N
    ppe_mat      <- epred_array
    yrep_ppe_mat <- epred_array
  }

  # --- Reshape matrices to long format ---
  obs_names <- seq_len(n_obs)

  ppe_long <- ppe_mat |>
    as.data.frame() |>
    stats::setNames(obs_names) |>
    dplyr::mutate(draw = dplyr::row_number()) |>
    tidyr::pivot_longer(
      -"draw", names_to = ".row", values_to = "ppe"
    ) |>
    dplyr::mutate(.row = as.integer(.data$.row))

  yrep_long <- yrep_mat |>
    as.data.frame() |>
    stats::setNames(obs_names) |>
    dplyr::mutate(draw = dplyr::row_number()) |>
    tidyr::pivot_longer(
      -"draw", names_to = ".row", values_to = "yrep"
    ) |>
    dplyr::mutate(.row = as.integer(.data$.row))

  yrep_ppe_long <- yrep_ppe_mat |>
    as.data.frame() |>
    stats::setNames(obs_names) |>
    dplyr::mutate(draw = dplyr::row_number()) |>
    tidyr::pivot_longer(
      -"draw", names_to = ".row", values_to = "yrep_ppe"
    ) |>
    dplyr::mutate(.row = as.integer(.data$.row))

  # --- Combine with model data ---
  model_data <- model$data |>
    dplyr::mutate(.row = dplyr::row_number())

  result <- ppe_long |>
    dplyr::left_join(yrep_long, by = c("draw", ".row")) |>
    dplyr::left_join(yrep_ppe_long, by = c("draw", ".row")) |>
    dplyr::left_join(model_data, by = ".row") |>
    dplyr::mutate(
      crit     = criterion(.data[[resp_var]], .data$ppe),
      crit_rep = criterion(.data$yrep, .data$yrep_ppe)
    ) |>
    dplyr::group_by(!!group, .data$draw) |>
    dplyr::summarise(
      crit      = sum(.data$crit),
      crit_rep  = sum(.data$crit_rep),
      crit_diff = .data$crit_rep - .data$crit,
      .groups   = "drop_last"
    ) |>
    dplyr::arrange(!!group, .data$draw)

  result
}


#' Posterior Predictive Item Fit Statistic for Binary Bayesian IRT Models
#'
#' Computes posterior predictive item (or person) fit statistics for
#' dichotomous Bayesian IRT models fitted with \pkg{brms}. For each
#' posterior draw, observed and replicated data are compared via a
#' user-supplied criterion function, grouped by item, person, or any other
#' variable. Posterior predictive p-values can then be derived from the
#' output to assess fit.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object with a binary
#'   response (e.g., \code{family = bernoulli()}).
#' @param criterion A function with signature \code{function(y, p)} that
#'   computes a pointwise fit criterion, where \code{y} is the binary
#'   response (0 or 1) and \code{p} is the predicted probability of
#'   success. A common choice is the Bernoulli log-likelihood:
#'   \code{function(y, p) y * log(p) + (1 - y) * log(1 - p)}.
#' @param group An unquoted variable name (e.g., \code{item} or \code{id})
#'   indicating the grouping variable over which the fit statistic is
#'   aggregated. Typically \code{item} for item fit or \code{id} for person
#'   fit.
#' @param ndraws_use Optional positive integer. If specified, a random subset
#'   of posterior draws of this size is used, which can speed up computation
#'   for large models. If \code{NULL} (the default), all draws are used.
#'
#' @return A \code{\link[tibble]{tibble}} with the following columns:
#' \describe{
#'   \item{\code{group}}{The grouping variable (e.g., item name or person id).}
#'   \item{draw}{Integer index of the posterior draw.}
#'   \item{crit}{The observed fit statistic (criterion applied to observed
#'     data) summed within each group and draw.}
#'   \item{crit_rep}{The replicated fit statistic (criterion applied to
#'     posterior predicted data) summed within each group and draw.}
#'   \item{crit_diff}{The difference \code{crit_rep - crit}.}
#' }
#' The output is grouped by the grouping variable. Posterior predictive
#' p-values can be obtained by computing
#' \code{mean(crit_rep > crit)} within each group.
#'
#' @details
#' This function is the binary-response counterpart of
#' \code{\link{fit_statistic_pcm}}, which handles polytomous (ordinal /
#' categorical) models. For dichotomous models, \code{posterior_epred()}
#' returns a 2D matrix (S x N) of success probabilities, so the criterion
#' function receives the observed binary response and the corresponding
#' probability directly.
#'
#' The procedure follows the posterior predictive checking approach
#' described in Bürkner (2020):
#'
#' \enumerate{
#'   \item Draw posterior expected success probabilities via
#'     \code{\link[brms]{posterior_epred}} and posterior predicted binary
#'     responses via \code{\link[brms]{posterior_predict}}.
#'   \item Apply the user-supplied \code{criterion} function pointwise
#'     to both observed and replicated data paired with the predicted
#'     probabilities.
#'   \item Aggregate (sum) the criterion values within each level of
#'     \code{group} and each posterior draw.
#' }
#'
#' The standard criterion for binary models is the Bernoulli log-likelihood:
#' \deqn{\ell(y, p) = y \log(p) + (1 - y) \log(1 - p).}
#'
#' @references
#' Bürkner, P.-C. (2020). Analysing Standard Progressive Matrices (SPM-LS)
#' with Bayesian Item Response Models. \emph{Journal of Intelligence},
#' \emph{8}(1). \doi{10.3390/jintelligence8010005}
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with brms and
#' Stan. \emph{Journal of Statistical Software}, \emph{100}, 1--54.
#' \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{fit_statistic_pcm}} for polytomous (ordinal/categorical) models,
#' \code{\link[brms]{posterior_epred}} for expected predictions,
#' \code{\link[brms]{posterior_predict}} for posterior predictive samples,
#' \code{\link[brms]{pp_check}} for graphical posterior predictive checks.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Dichotomous Rasch Model ---
#'
#' # Prepare binary response data in long format
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' # Fit a dichotomous Rasch model
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4,
#'   cores   = 1, # use more cores if you have
#'   iter    = 500 # use at least 2000 
#' )
#'
#' # Bernoulli log-likelihood criterion
#' ll_bernoulli <- function(y, p) y * log(p) + (1 - y) * log(1 - p)
#'
#' # Compute item fit statistics
#' item_fit <- fit_statistic_rm(
#'   model      = fit_rm,
#'   criterion  = ll_bernoulli,
#'   group      = item,
#'   ndraws_use = 100 # use at least 500
#' )
#'
#' # Summarise: posterior predictive p-values per item
#' item_fit %>%
#'   group_by(item) %>%
#'   summarise(
#'     observed   = mean(crit),
#'     replicated = mean(crit_rep),
#'     ppp        = mean(crit_rep > crit)
#'   )
#'
#' # Use ggplot2 to make a histogram
#' library(ggplot2)
#' item_fit %>%
#'   ggplot(aes(crit_diff)) +
#'   geom_histogram(aes(fill = ifelse(crit_diff > 0, "above","below"))) +
#'   facet_wrap("item") +
#'   theme_bw() +
#'   theme(legend.position = "none")
#'
#' # Compute person fit statistics
#' person_fit <- fit_statistic_rm(
#'   model      = fit_rm,
#'   criterion  = ll_bernoulli,
#'   group      = id,
#'   ndraws_use = 100 # use at least 500
#' )
#'
#' person_fit %>%
#'   group_by(id) %>%
#'   summarise(
#'     observed   = mean(crit),
#'     replicated = mean(crit_rep),
#'     ppp        = mean(crit_rep > crit)
#'   )
#'
#' # --- 1PL model with item-specific intercepts ---
#'
#' # Alternative parameterisation with fixed item effects
#' fit_1pl <- brm(
#'   response ~ 0 + item + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4,
#'   cores   = 1, # use more cores if you have
#'   iter    = 500 # use at least 2000 
#' )
#'
#' item_fit_1pl <- fit_statistic_rm(
#'   model      = fit_1pl,
#'   criterion  = ll_bernoulli,
#'   group      = item,
#'   ndraws_use = 100 # use at least 500
#' )
#'
#' item_fit_1pl %>%
#'   group_by(item) %>%
#'   summarise(
#'     observed   = mean(crit),
#'     replicated = mean(crit_rep),
#'     ppp        = mean(crit_rep > crit)
#'   )
#' }
#'
#' @importFrom brms posterior_epred posterior_predict ndraws
#' @importFrom dplyr mutate left_join group_by summarise arrange row_number
#' @importFrom tidyr pivot_longer
#' @importFrom rlang enquo !! .data
#' @importFrom stats formula setNames
#' @export
fit_statistic_rm <- function(model, criterion, group,
                                 ndraws_use = NULL) {
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  if (!is.function(criterion)) {
    stop("'criterion' must be a function with signature function(y, p).",
         call. = FALSE)
  }

  group <- rlang::enquo(group)

  # --- Extract response variable name from the model formula ---
  resp_var <- as.character(formula(model)$formula[[2]])
  if (length(resp_var) > 1) {
    resp_var <- resp_var[2]
  }
  if (!resp_var %in% names(model$data)) {
    stop("Response variable '", resp_var, "' not found in model data.",
         call. = FALSE)
  }

  # --- Determine posterior draw subset ---
  draw_ids <- NULL
  if (!is.null(ndraws_use)) {
    ndraws_use <- as.integer(ndraws_use)
    if (ndraws_use < 1) {
      stop("'ndraws_use' must be a positive integer.", call. = FALSE)
    }
    total_draws <- brms::ndraws(model)
    if (ndraws_use > total_draws) {
      warning("'ndraws_use' (", ndraws_use, ") exceeds available draws (",
              total_draws, "). Using all draws.", call. = FALSE)
      ndraws_use <- total_draws
    }
    draw_ids <- sample(seq_len(total_draws), ndraws_use)
  }

  # --- Posterior expected predictions and predicted responses ---
  # For binary models, posterior_epred returns an S x N matrix of P(Y = 1)
  ppe_mat  <- brms::posterior_epred(model, draw_ids = draw_ids)
  yrep_mat <- brms::posterior_predict(model, draw_ids = draw_ids)

  if (length(dim(ppe_mat)) == 3) {
    stop("Model appears to be ordinal/categorical (3D posterior_epred). ",
         "Use fit_statistic_pcm() instead.", call. = FALSE)
  }

  n_draws <- nrow(ppe_mat)
  n_obs   <- ncol(ppe_mat)

  # --- Reshape matrices to long format ---
  obs_names <- seq_len(n_obs)

  ppe_long <- ppe_mat |>
    as.data.frame() |>
    stats::setNames(obs_names) |>
    dplyr::mutate(draw = dplyr::row_number()) |>
    tidyr::pivot_longer(
      -"draw", names_to = ".row", values_to = "ppe"
    ) |>
    dplyr::mutate(.row = as.integer(.data$.row))

  yrep_long <- yrep_mat |>
    as.data.frame() |>
    stats::setNames(obs_names) |>
    dplyr::mutate(draw = dplyr::row_number()) |>
    tidyr::pivot_longer(
      -"draw", names_to = ".row", values_to = "yrep"
    ) |>
    dplyr::mutate(.row = as.integer(.data$.row))

  # --- Combine with model data ---
  model_data <- model$data |>
    dplyr::mutate(.row = dplyr::row_number())

  result <- ppe_long |>
    dplyr::left_join(yrep_long, by = c("draw", ".row")) |>
    dplyr::left_join(model_data, by = ".row") |>
    dplyr::mutate(
      crit     = criterion(.data[[resp_var]], .data$ppe),
      crit_rep = criterion(.data$yrep, .data$ppe)
    ) |>
    dplyr::group_by(!!group, .data$draw) |>
    dplyr::summarise(
      crit      = sum(.data$crit),
      crit_rep  = sum(.data$crit_rep),
      crit_diff = .data$crit_rep - .data$crit,
      .groups   = "drop_last"
    ) |>
    dplyr::arrange(!!group, .data$draw)

  result
}


#' Posterior Predictive Infit Statistic for Bayesian IRT Models
#'
#' Computes a Bayesian analogue of the conditional item infit statistic
#' (as described in Christensen, Kreiner & Mesbah, 2013) for Rasch-family
#' models fitted with \pkg{brms}. For each posterior draw, expected values
#' and variances are derived from the category probabilities returned by
#' \code{\link[brms]{posterior_epred}}, and variance-weighted standardised
#' residuals are computed for both observed and replicated data. The result
#' can be summarised into posterior predictive p-values to assess item fit.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from an ordinal
#'   IRT model (e.g., \code{family = acat} for a partial credit model or
#'   \code{family = bernoulli()} for a dichotomous Rasch model).
#' @param item_var An unquoted variable name identifying the item grouping
#'   variable in the model data (e.g., \code{item}).
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data (e.g., \code{id}).
#' @param ndraws_use Optional positive integer. If specified, a random subset
#'   of posterior draws of this size is used. If \code{NULL} (the default),
#'   all draws are used.
#' @param outfit Logical. If \code{TRUE}, outfit statistics are computed
#'   alongside infit. Default is \code{FALSE} (infit only), since outfit
#'   is highly sensitive to outliers and rarely recommended for Rasch
#'   diagnostics.
#'
#' @return A \code{\link[tibble]{tibble}} with the following columns:
#' \describe{
#'   \item{item}{The item identifier.}
#'   \item{draw}{Integer index of the posterior draw.}
#'   \item{infit}{The observed infit statistic for that item and draw.}
#'   \item{infit_rep}{The replicated infit statistic (based on posterior
#'     predicted data) for that item and draw.}
#'   \item{outfit}{(Only if \code{outfit = TRUE}) The observed outfit
#'     statistic for that item and draw.}
#'   \item{outfit_rep}{(Only if \code{outfit = TRUE}) The replicated
#'     outfit statistic for that item and draw.}
#' }
#' The output is grouped by the item variable. Posterior predictive
#' p-values can be obtained by computing, e.g.,
#' \code{mean(infit_rep > infit)} within each item.
#'
#' @details
#' The procedure adapts the conditional infit/outfit statistics
#' (Christensen et al., 2013; Kreiner & Christensen, 2011; Müller, 2020)
#' to the Bayesian framework:
#'
#' \enumerate{
#'   \item For each posterior draw \eqn{s}, category probabilities
#'     \eqn{P^{(s)}(X_{vi} = c)} are obtained from
#'     \code{\link[brms]{posterior_epred}}.
#'   \item The conditional expected value and variance for each
#'     observation are computed as:
#'     \deqn{E^{(s)}_{vi} = \sum_c c \cdot P^{(s)}(X_{vi} = c)}
#'     \deqn{Var^{(s)}_{vi} = \sum_c (c - E^{(s)}_{vi})^2 \cdot
#'       P^{(s)}(X_{vi} = c)}
#'   \item Standardised squared residuals are:
#'     \deqn{Z^{2(s)}_{vi} = (X_{vi} - E^{(s)}_{vi})^2 / Var^{(s)}_{vi}}
#'   \item The infit statistic for item \eqn{i} is the variance-weighted
#'     mean of \eqn{Z^2} across persons:
#'     \deqn{Infit_i^{(s)} = \frac{\sum_v Var_{vi}^{(s)} Z^{2(s)}_{vi}}
#'       {\sum_v Var_{vi}^{(s)}}}
#'   \item If requested, the outfit is the unweighted mean of \eqn{Z^2}.
#' }
#'
#' @references
#' Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
#' \emph{Rasch Models in Health}. Iste and Wiley, pp. 86--90.
#'
#' Kreiner, S. & Christensen, K. B. (2011). Exact evaluation of Bias in
#' Rasch model residuals. \emph{Advances in Mathematics Research}, 12,
#' 19--40.
#'
#' Müller, M. (2020). Item fit statistics for Rasch analysis: can we
#' trust them? \emph{Journal of Statistical Distributions and
#' Applications}, \emph{7}(1).
#' \doi{10.1186/s40488-020-00108-7}
#'
#' @seealso
#' \code{\link{fit_statistic_pcm}} for a general-purpose posterior predictive
#' fit statistic with user-supplied criterion functions,
#' \code{\link{fit_statistic_rm}} for a general-purpose posterior predictive
#' fit statistic with user-supplied criterion functions,
#' \code{\link[brms]{posterior_epred}},
#' \code{\link[brms]{posterior_predict}}.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Partial Credit Model (polytomous) ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' # Compute infit per item
#' item_infit <- infit_statistic(
#'   model      = fit_pcm,
#'   ndraws_use = 100 # use at least 500
#' )
#'
#' # Post-process draws
#' infit_results <- infit_post(item_infit)
#' infit_results$summary
#' infit_results$hdi
#' infit_results$plot
#'
#' # --- Dichotomous Rasch Model ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' item_infit_rm <- infit_statistic(
#'   model      = fit_rm,
#'   ndraws_use = 100 # use at least 500
#' )
#' 
#' # Post-process draws
#' infit_results <- infit_post(item_infit_rm)
#' infit_results$summary
#' infit_results$hdi
#' infit_results$plot
#' 
#' }
#'
#' @importFrom brms posterior_epred posterior_predict ndraws
#' @importFrom dplyr mutate group_by summarise arrange row_number n
#' @importFrom tidyr pivot_longer
#' @importFrom rlang enquo !! .data as_name
#' @importFrom stats formula setNames
#' @export
infit_statistic <- function(model, item_var = item, person_var = id,
                            ndraws_use = NULL, outfit = FALSE) {
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  
  item_quo   <- rlang::enquo(item_var)
  person_quo <- rlang::enquo(person_var)
  item_name   <- rlang::as_name(item_quo)
  person_name <- rlang::as_name(person_quo)
  
  # --- Extract response variable name from model formula ---
  resp_var <- as.character(formula(model)$formula[[2]])
  if (length(resp_var) > 1) {
    resp_var <- resp_var[2]
  }
  if (!resp_var %in% names(model$data)) {
    stop("Response variable '", resp_var, "' not found in model data.",
         call. = FALSE)
  }
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  # --- Determine posterior draw subset ---
  draw_ids <- NULL
  if (!is.null(ndraws_use)) {
    ndraws_use <- as.integer(ndraws_use)
    if (ndraws_use < 1) {
      stop("'ndraws_use' must be a positive integer.", call. = FALSE)
    }
    total_draws <- brms::ndraws(model)
    if (ndraws_use > total_draws) {
      warning("'ndraws_use' (", ndraws_use, ") exceeds available draws (",
              total_draws, "). Using all draws.", call. = FALSE)
      ndraws_use <- total_draws
    }
    draw_ids <- sample(seq_len(total_draws), ndraws_use)
  }
  
  # --- Posterior predictions ---
  epred_array <- brms::posterior_epred(model, draw_ids = draw_ids)
  yrep_mat    <- brms::posterior_predict(model, draw_ids = draw_ids)
  
  n_draws <- dim(epred_array)[1]
  n_obs   <- dim(epred_array)[2]
  obs_response <- model$data[[resp_var]]
  
  # =================================================================
  # Compute E_mat [S x N] and Var_mat [S x N]  — VECTORIZED
  # =================================================================
  if (length(dim(epred_array)) == 3) {
    # Ordinal/categorical: epred_array is S x N x C
    n_cat <- dim(epred_array)[3]
    cat_values <- seq_len(n_cat)  # 1, 2, ..., C
    
    # E[X | params] = sum_c  c * P(X=c)
    # Reshape epred to (S*N) x C, multiply by cat_values, reshape back
    dim_orig <- dim(epred_array)
    ep_2d <- matrix(epred_array, nrow = dim_orig[1] * dim_orig[2],
                    ncol = dim_orig[3])
    E_vec <- ep_2d %*% cat_values          # (S*N) x 1
    E_mat <- matrix(E_vec, nrow = n_draws, ncol = n_obs)
    
    # Var[X | params] = sum_c (c - E)^2 * P(X=c)
    #                 = sum_c c^2 * P(X=c) - E^2   (computational formula)
    E2_vec <- ep_2d %*% (cat_values^2)     # (S*N) x 1
    Var_vec <- E2_vec - E_vec^2
    Var_mat <- matrix(Var_vec, nrow = n_draws, ncol = n_obs)
  } else {
    # Binary: S x N matrix with P(Y = 1)
    E_mat   <- epred_array
    Var_mat <- epred_array * (1 - epred_array)
  }
  
  # Clamp variance to avoid division by zero
  Var_mat[Var_mat < 1e-12] <- 1e-12
  
  # =================================================================
  # Squared residuals: (X - E)^2  — compute once, divide later
  # =================================================================
  obs_mat <- matrix(obs_response, nrow = n_draws, ncol = n_obs, byrow = TRUE)
  resid2_obs <- (obs_mat - E_mat)^2
  resid2_rep <- (yrep_mat - E_mat)^2
  
  # Z^2 = resid^2 / Var  (only needed for outfit)
  # Infit numerator = sum(Var * Z^2) = sum(resid^2)  — no division needed!
  # Infit denominator = sum(Var)
  
  # --- Retrieve item identifiers and pre-compute indices ---
  items <- model$data[[item_name]]
  unique_items <- unique(items)
  k <- length(unique_items)
  
  # Pre-compute column indices for each item (avoids repeated which())
  item_col_idx <- vector("list", k)
  for (idx in seq_len(k)) {
    item_col_idx[[idx]] <- which(items == unique_items[idx])
  }
  
  # =================================================================
  # Compute infit (and optionally outfit) per item per draw
  # =================================================================
  # Pre-allocate output matrices: rows = draws, cols = items
  infit_obs_mat <- matrix(NA_real_, nrow = n_draws, ncol = k)
  infit_rep_mat <- matrix(NA_real_, nrow = n_draws, ncol = k)
  if (outfit) {
    outfit_obs_mat <- matrix(NA_real_, nrow = n_draws, ncol = k)
    outfit_rep_mat <- matrix(NA_real_, nrow = n_draws, ncol = k)
  }
  
  for (idx in seq_len(k)) {
    cols <- item_col_idx[[idx]]
    
    # Infit = sum(resid^2) / sum(Var)
    #       = sum(Var * Z^2) / sum(Var)    [algebraically identical]
    # This avoids computing Z^2 entirely for infit!
    sum_var <- rowSums(Var_mat[, cols, drop = FALSE])
    sum_var[sum_var < 1e-12] <- 1e-12
    
    infit_obs_mat[, idx] <- rowSums(resid2_obs[, cols, drop = FALSE]) /
      sum_var
    infit_rep_mat[, idx] <- rowSums(resid2_rep[, cols, drop = FALSE]) /
      sum_var
    
    if (outfit) {
      # Outfit = mean(Z^2) = mean(resid^2 / Var)
      Z2_obs_i <- resid2_obs[, cols, drop = FALSE] /
        Var_mat[, cols, drop = FALSE]
      Z2_rep_i <- resid2_rep[, cols, drop = FALSE] /
        Var_mat[, cols, drop = FALSE]
      outfit_obs_mat[, idx] <- rowMeans(Z2_obs_i, na.rm = TRUE)
      outfit_rep_mat[, idx] <- rowMeans(Z2_rep_i, na.rm = TRUE)
    }
  }
  
  # =================================================================
  # Assemble output tibble
  # =================================================================
  draw_seq <- seq_len(n_draws)
  item_rep <- rep(unique_items, each = n_draws)
  draw_rep <- rep(draw_seq, times = k)
  
  if (outfit) {
    result <- tibble::tibble(
      item       = item_rep,
      draw       = draw_rep,
      infit      = as.vector(infit_obs_mat),
      infit_rep  = as.vector(infit_rep_mat),
      outfit     = as.vector(outfit_obs_mat),
      outfit_rep = as.vector(outfit_rep_mat)
    )
  } else {
    result <- tibble::tibble(
      item      = item_rep,
      draw      = draw_rep,
      infit     = as.vector(infit_obs_mat),
      infit_rep = as.vector(infit_rep_mat)
    )
  }
  
  # Rename the item column to match the user's variable name
  names(result)[names(result) == "item"] <- item_name
  
  result <- dplyr::group_by(result, .data[[item_name]])
  result <- dplyr::arrange(result, .data[[item_name]], .data$draw)
  
  result
}


#' Posterior Predictive Item-Restscore Association for Bayesian IRT Models
#'
#' Computes a Bayesian analogue of the item-restscore association test
#' (Kreiner, 2011) for Rasch-family models fitted with \pkg{brms}. For
#' each posterior draw, the Goodman-Kruskal gamma coefficient between
#' each item's score and the rest-score (total score minus that item)
#' is computed for both observed and replicated data. Posterior
#' predictive p-values indicate whether the observed association is
#' stronger than the model predicts, which signals violations of
#' local independence or unidimensionality.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object from an
#'   ordinal IRT model (e.g., \code{family = acat} for a partial credit
#'   model) or a dichotomous model (\code{family = bernoulli()}).
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data (e.g., \code{item}).
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data (e.g., \code{id}).
#' @param ndraws_use Optional positive integer. If specified, a random
#'   subset of posterior draws of this size is used. If \code{NULL} (the
#'   default), all draws are used.
#'
#' @return A \code{\link[tibble]{tibble}} with the following columns:
#' \describe{
#'   \item{item}{The item identifier.}
#'   \item{gamma_obs}{Posterior mean of the observed Goodman-Kruskal
#'     gamma between this item and the rest-score.}
#'   \item{gamma_rep}{Posterior mean of the replicated gamma.}
#'   \item{gamma_diff}{Posterior mean of \code{gamma_obs - gamma_rep}.
#'     Positive values indicate the observed item-restscore association
#'     is stronger than the model expects.}
#'   \item{ppp}{Posterior predictive p-value:
#'     \code{mean(gamma_obs > gamma_rep)} across draws.
#'     Values close to 1 indicate the item discriminates more than the
#'     model predicts (too high discrimination). Values close to 0
#'     indicate the item discriminates less than expected (too low
#'     discrimination, e.g., noise or miskeyed item).}
#'   \item{gamma_obs_q025, gamma_obs_q975}{95\% credible interval for
#'     the observed gamma.}
#'   \item{gamma_obs_q005, gamma_obs_q995}{99\% credible interval for
#'     the observed gamma.}
#'   \item{gamma_diff_q025, gamma_diff_q975}{95\% credible interval for
#'     the gamma difference.}
#'   \item{gamma_diff_q005, gamma_diff_q995}{99\% credible interval for
#'     the gamma difference.}
#' }
#'
#' @details
#' The item-restscore association is a key diagnostic in Rasch
#' measurement. Under the Rasch model, each item should relate to the
#' latent trait (and hence the rest-score) only through the modelled
#' relationship. Goodman-Kruskal's gamma is a rank-based measure of
#' association for ordinal cross-tabulations that is well-suited for
#' this purpose (Kreiner, 2011).
#'
#' The procedure for each posterior draw \eqn{s} is:
#'
#' \enumerate{
#'   \item Obtain replicated responses \eqn{Y^{rep(s)}} from
#'     \code{\link[brms]{posterior_predict}}.
#'   \item For each item \eqn{i} and each person \eqn{v}, compute the
#'     rest-score: \eqn{R^{obs}_{vi} = \sum_{j \neq i} X_{vj}} for
#'     observed data and \eqn{R^{rep(s)}_{vi} = \sum_{j \neq i}
#'     Y^{rep(s)}_{vj}} for replicated data.
#'   \item Cross-tabulate item score \eqn{\times} rest-score and compute
#'     the Goodman-Kruskal gamma for both observed and replicated data.
#'   \item Compare the two gammas across draws.
#' }
#'
#' Items with \code{ppp} close to 1 have observed item-restscore
#' association that is consistently stronger than the model predicts.
#' This typically indicates that the item discriminates more than
#' assumed under the equal-discrimination Rasch model (i.e., a
#' violation of the Rasch assumption). Items with \code{ppp} close to
#' 0 discriminate less than expected.
#'
#' @references
#' Kreiner, S. (2011). A note on item-restscore association in Rasch
#' models. \emph{Applied Psychological Measurement}, \emph{35}(7),
#' 557--561.
#'
#' Goodman, L. A. & Kruskal, W. H. (1954). Measures of association for
#' cross classifications. \emph{Journal of the American Statistical
#' Association}, \emph{49}(268), 732--764.
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
#' brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
#' 1--54. \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{fit_statistic_pcm}} for posterior predictive fit statistics,
#' \code{\link{fit_statistic_rm}} for posterior predictive fit statistics,
#' \code{\link{infit_statistic}} for Bayesian infit/outfit,
#' \code{\link{q3_statistic}} for Bayesian Q3 residual correlations,
#' \code{\link[brms]{posterior_predict}}.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' # Item-restscore association
#' irs <- item_restscore_statistic(
#'   model      = fit_pcm,
#'   ndraws_use = 100 # use at least 500
#' )
#' 
#' # Post-process draws
#' irs_results <- item_restscore_post(irs)
#' irs_results$summary
#' irs_results$plot
#'
#' # --- Dichotomous Rasch Model ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4,
#'   cores  = 1, # use more cores if you have
#'   iter   = 500 # use at least 2000 
#' )
#'
#' irs_rm <- item_restscore_statistic(
#'   model      = fit_rm,
#'   ndraws_use = 100 # use at least 500
#' )
#'
#' # Post-process draws
#' irs_results <- item_restscore_post(irs_rm)
#' irs_results$summary
#' irs_results$plot
#' }
#'
#' @importFrom brms posterior_predict ndraws
#' @importFrom dplyr arrange desc filter everything
#' @importFrom rlang enquo !! .data as_name
#' @importFrom tibble as_tibble
#' @importFrom stats formula quantile
#' @export
item_restscore_statistic <- function(model, item_var = item, person_var = id,
                                     ndraws_use = NULL) {
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  
  item_quo    <- rlang::enquo(item_var)
  person_quo  <- rlang::enquo(person_var)
  item_name   <- rlang::as_name(item_quo)
  person_name <- rlang::as_name(person_quo)
  
  # --- Extract response variable name ---
  resp_var <- as.character(formula(model)$formula[[2]])
  if (length(resp_var) > 1) {
    resp_var <- resp_var[2]
  }
  if (!resp_var %in% names(model$data)) {
    stop("Response variable '", resp_var, "' not found in model data.",
         call. = FALSE)
  }
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  # --- Determine posterior draw subset ---
  draw_ids <- NULL
  if (!is.null(ndraws_use)) {
    ndraws_use <- as.integer(ndraws_use)
    if (ndraws_use < 1) {
      stop("'ndraws_use' must be a positive integer.", call. = FALSE)
    }
    total_draws <- brms::ndraws(model)
    if (ndraws_use > total_draws) {
      warning("'ndraws_use' (", ndraws_use, ") exceeds available draws (",
              total_draws, "). Using all draws.", call. = FALSE)
      ndraws_use <- total_draws
    }
    draw_ids <- sample(seq_len(total_draws), ndraws_use)
  }
  
  # --- Posterior predicted responses ---
  yrep_mat <- brms::posterior_predict(model, draw_ids = draw_ids)
  
  n_draws <- nrow(yrep_mat)
  n_obs   <- ncol(yrep_mat)
  obs_response <- model$data[[resp_var]]
  
  # --- Map observations to person x item structure ---
  items   <- model$data[[item_name]]
  persons <- model$data[[person_name]]
  unique_items   <- sort(unique(items))
  unique_persons <- sort(unique(persons))
  k <- length(unique_items)
  n_persons <- length(unique_persons)
  
  person_idx <- match(persons, unique_persons)
  item_idx   <- match(items, unique_items)
  
  # Pre-compute linear index for vectorized wide-matrix construction
  lin_idx <- (item_idx - 1L) * n_persons + person_idx
  
  # --- Build observed wide matrix (person x item) ONCE ---
  obs_wide <- matrix(NA_real_, nrow = n_persons, ncol = k)
  obs_wide[lin_idx] <- obs_response
  obs_total <- rowSums(obs_wide, na.rm = TRUE)
  
  # --- Compute observed gamma ONCE (constant across draws) ---
  gamma_obs <- numeric(k)
  for (i in seq_len(k)) {
    item_score_obs <- obs_wide[, i]
    rest_score_obs <- obs_total - item_score_obs
    valid_obs <- !is.na(item_score_obs)
    if (sum(valid_obs) > 2L) {
      gamma_obs[i] <- gk_gamma_vec(item_score_obs[valid_obs],
                                   rest_score_obs[valid_obs])
    } else {
      gamma_obs[i] <- NA_real_
    }
  }
  
  # Broadcast observed gamma to a draws x items matrix (for summary stats)
  gamma_obs_draws <- matrix(gamma_obs, nrow = n_draws, ncol = k, byrow = TRUE)
  
  # --- Compute replicated gamma for each draw ---
  gamma_rep_draws <- matrix(NA_real_, nrow = n_draws, ncol = k)
  
  for (s in seq_len(n_draws)) {
    # Vectorized wide-matrix construction
    rep_wide <- matrix(NA_real_, nrow = n_persons, ncol = k)
    rep_wide[lin_idx] <- yrep_mat[s, ]
    rep_total <- rowSums(rep_wide, na.rm = TRUE)
    
    for (i in seq_len(k)) {
      item_score_rep <- rep_wide[, i]
      rest_score_rep <- rep_total - item_score_rep
      valid_rep <- !is.na(item_score_rep)
      if (sum(valid_rep) > 2L) {
        gamma_rep_draws[s, i] <- gk_gamma_vec(item_score_rep[valid_rep],
                                              rest_score_rep[valid_rep])
      }
    }
  }
  
  # --- Summarise across draws ---
  gamma_diff_draws <- gamma_obs_draws - gamma_rep_draws
  
  result <- data.frame(
    item = unique_items,
    stringsAsFactors = FALSE
  )
  names(result)[1] <- item_name
  
  result$gamma_obs  <- gamma_obs
  result$gamma_rep  <- colMeans(gamma_rep_draws, na.rm = TRUE)
  result$gamma_diff <- colMeans(gamma_diff_draws, na.rm = TRUE)
  result$ppp        <- colMeans(gamma_obs_draws > gamma_rep_draws,
                                na.rm = TRUE)
  
  # 95% credible intervals
  result$gamma_obs_q025  <- apply(gamma_obs_draws, 2, stats::quantile,
                                  probs = 0.025, na.rm = TRUE)
  result$gamma_obs_q975  <- apply(gamma_obs_draws, 2, stats::quantile,
                                  probs = 0.975, na.rm = TRUE)
  result$gamma_diff_q025 <- apply(gamma_diff_draws, 2, stats::quantile,
                                  probs = 0.025, na.rm = TRUE)
  result$gamma_diff_q975 <- apply(gamma_diff_draws, 2, stats::quantile,
                                  probs = 0.975, na.rm = TRUE)
  
  # 99% credible intervals
  result$gamma_obs_q005  <- apply(gamma_obs_draws, 2, stats::quantile,
                                  probs = 0.005, na.rm = TRUE)
  result$gamma_obs_q995  <- apply(gamma_obs_draws, 2, stats::quantile,
                                  probs = 0.995, na.rm = TRUE)
  result$gamma_diff_q005 <- apply(gamma_diff_draws, 2, stats::quantile,
                                  probs = 0.005, na.rm = TRUE)
  result$gamma_diff_q995 <- apply(gamma_diff_draws, 2, stats::quantile,
                                  probs = 0.995, na.rm = TRUE)
  
  result <- tibble::as_tibble(result)
  result <- dplyr::arrange(result, dplyr::desc(.data$gamma_diff))
  
  # --- Assign real item names to matrix columns BEFORE pivoting ---
  colnames(gamma_rep_draws) <- unique_items
  colnames(gamma_obs_draws) <- unique_items
  
  gamma_rep_draws_df <- gamma_rep_draws |>
    as.data.frame() |>
    tidyr::pivot_longer(dplyr::everything(),
                        names_to = "item", values_to = "gamma_rep")
  gamma_obs_draws_df <- gamma_obs_draws |>
    as.data.frame() |>
    tidyr::pivot_longer(dplyr::everything(),
                        names_to = "item", values_to = "gamma_obs")
  
  gamma_draws_df <- dplyr::bind_cols(
    gamma_rep_draws_df,
    gamma = gamma_obs_draws_df$gamma_obs
  )
  
  # Rename the "item" column to match the user's item variable name
  names(gamma_draws_df)[names(gamma_draws_df) == "item"] <- item_name
  
  list(
    result = result,
    draws = tibble::as_tibble(gamma_draws_df)
  )
}

#' Goodman-Kruskal Gamma from Two Vectors (No table() Overhead)
#'
#' Computes the Goodman-Kruskal gamma coefficient directly from two
#' numeric vectors, bypassing \code{table()} for speed. Uses manual
#' tabulation with integer indexing and cumulative sums for efficient
#' concordant/discordant pair counting.
#'
#' @param x Integer or numeric vector (e.g., item scores).
#' @param y Integer or numeric vector of the same length (e.g., rest scores).
#'
#' @return A scalar: the gamma coefficient in \eqn{[-1, 1]}, or
#'   \code{NA_real_} if the table has fewer than 2 rows or 2 columns.
#'
#' @references
#' Goodman, L. A. & Kruskal, W. H. (1954). Measures of association for
#' cross classifications. \emph{Journal of the American Statistical
#' Association}, \emph{49}(268), 732--764.
#'
#' @keywords internal
gk_gamma_vec <- function(x, y) {
  # Fast manual tabulation
  ux <- sort.int(unique.default(x), method = "quick")
  uy <- sort.int(unique.default(y), method = "quick")
  nr <- length(ux)
  nc <- length(uy)
  if (nr < 2L || nc < 2L) return(NA_real_)
  
  xi <- match(x, ux)
  yi <- match(y, uy)
  
  # Build contingency table via integer indexing
  tab <- matrix(0L, nrow = nr, ncol = nc)
  for (idx in seq_along(xi)) {
    tab[xi[idx], yi[idx]] <- tab[xi[idx], yi[idx]] + 1L
  }
  
  .gk_gamma_from_tab(tab, nr, nc)
}


#' Goodman-Kruskal Gamma from a Contingency Table
#'
#' Computes the Goodman-Kruskal gamma coefficient for an ordinal
#' contingency table. This is an internal helper function.
#'
#' @param tab A matrix or table representing a contingency table with
#'   rows and columns in ordinal order.
#'
#' @return A scalar: the gamma coefficient in \eqn{[-1, 1]}.
#'
#' @details
#' Gamma is defined as \eqn{(C - D) / (C + D)}, where \eqn{C} is the
#' number of concordant pairs and \eqn{D} is the number of discordant
#' pairs. Uses cumulative suffix sums for \eqn{O(nr \times nc)}
#' complexity.
#'
#' @references
#' Goodman, L. A. & Kruskal, W. H. (1954). Measures of association for
#' cross classifications. \emph{Journal of the American Statistical
#' Association}, \emph{49}(268), 732--764.
#'
#' @keywords internal
gk_gamma <- function(tab) {
  tab <- as.matrix(tab)
  nr <- nrow(tab)
  nc <- ncol(tab)
  if (nr < 2L || nc < 2L) return(NA_real_)
  
  .gk_gamma_from_tab(tab, nr, nc)
}


#' Compute Gamma from a Pre-built Contingency Table
#'
#' Shared engine for \code{gk_gamma()} and \code{gk_gamma_vec()}.
#' Uses cumulative suffix sums and prefix column sums for
#' \eqn{O(nr \times nc)} concordant/discordant pair counting.
#'
#' @param tab Integer or numeric matrix (contingency table).
#' @param nr Number of rows.
#' @param nc Number of columns.
#'
#' @return A scalar: the gamma coefficient, or \code{NA_real_}.
#'
#' @keywords internal
.gk_gamma_from_tab <- function(tab, nr, nc) {
  
  
  # --- Suffix sum: suffix[i,j] = sum of tab[i:nr, j:nc] ---
  # Padded to (nr+1) x (nc+1) so out-of-bounds reads give 0
  suffix <- matrix(0, nrow = nr + 1L, ncol = nc + 1L)
  for (i in seq.int(nr, 1L)) {
    for (j in seq.int(nc, 1L)) {
      suffix[i, j] <- tab[i, j] +
        suffix[i + 1L, j] +
        suffix[i, j + 1L] -
        suffix[i + 1L, j + 1L]
    }
  }
  
  # --- Prefix-column sum: pcol[i,j] = sum of tab[i:nr, 1:j] ---
  # Padded to (nr+1) x (nc+1); column 0 stays 0 as the base case
  pcol <- matrix(0, nrow = nr + 1L, ncol = nc + 1L)
  for (i in seq.int(nr, 1L)) {
    # j runs over columns 1..nc, stored in columns 2..(nc+1)
    # so "column j-1" is always >= 1 and never hits index 0
    for (j in seq.int(2L, nc + 1L)) {
      pcol[i, j] <- tab[i, j - 1L] +
        pcol[i + 1L, j] +
        pcol[i, j - 1L] -
        pcol[i + 1L, j - 1L]
    }
  }
  
  C <- 0
  D <- 0
  for (i in seq_len(nr - 1L)) {
    for (j in seq_len(nc)) {
      n_ij <- tab[i, j]
      if (n_ij == 0L) next
      # Concordant: cells strictly below-right
      if (j < nc) {
        C <- C + n_ij * suffix[i + 1L, j + 1L]
      }
      # Discordant: cells strictly below-left
      # pcol column index is shifted by +1 (column j in pcol = index j+1)
      if (j > 1L) {
        D <- D + n_ij * pcol[i + 1L, j]
      }
    }
  }
  
  denom <- C + D
  if (denom == 0) return(NA_real_)
  (C - D) / denom
}

#' Extract Item Parameters from a Bayesian Rasch Model
#'
#' Extracts item difficulty (threshold) parameters from a fitted
#' Bayesian Rasch model. Returns a simple location table in both long
#' and wide formats, a full summary with posterior SEs and HDCIs,
#' item-level information, threshold ordering diagnostics, and
#' optionally the full posterior draws matrix.
#'
#' @param model A fitted \code{\link[brms]{brmsfit}} object. Supported
#'   parameterisations:
#'   \describe{
#'     \item{Polytomous ordinal (PCM)}{e.g., \code{family = acat} with
#'       \code{thres(gr = item)}, producing item-specific thresholds.}
#'     \item{Dichotomous Rasch (random items)}{e.g.,
#'       \code{response ~ 1 + (1 | item) + (1 | id)} with
#'       \code{family = bernoulli()}.}
#'     \item{Dichotomous 1PL (fixed items)}{e.g.,
#'       \code{response ~ 0 + item + (1 | id)} with
#'       \code{family = bernoulli()}.}
#'   }
#' @param item_var An unquoted variable name identifying the item
#'   grouping variable in the model data. Default is \code{item}.
#' @param person_var An unquoted variable name identifying the person
#'   grouping variable in the model data. Default is \code{id}.
#' @param draws Logical. If \code{TRUE}, a draws matrix of full
#'   posterior draws is included in the output. Default is
#'   \code{FALSE}.
#' @param center Logical. If \code{TRUE} (the default), item
#'   parameters are shifted so that the grand mean of all threshold
#'   locations is zero, matching the frequentist CML convention and
#'   the centering used in \code{\link{plot_targeting}}.
#' @param prob Numeric in \eqn{(0, 1)}. Width of the highest density
#'   continuous interval (HDCI) reported in the summary. Default is
#'   0.95.
#'
#' @return A list with the following elements:
#' \describe{
#'   \item{locations}{A \code{\link[tibble]{tibble}} in long format
#'     with one row per item (dichotomous) or per item-threshold
#'     (polytomous), containing \code{item}, \code{threshold}
#'     (integer, always 1 for dichotomous), and \code{location}
#'     (posterior mean on the logit scale).}
#'   \item{locations_wide}{A \code{\link[tibble]{tibble}} in wide
#'     format with one row per item, sorted by mean location. For
#'     polytomous models, threshold columns are named \code{t1},
#'     \code{t2}, etc., and a \code{location} column gives the mean
#'     across thresholds. For dichotomous models, only \code{item}
#'     and \code{location} columns are present.}
#'   \item{summary}{A \code{\link[tibble]{tibble}} extending the long
#'     \code{locations} with: \code{se} (posterior SD),
#'     \code{hdci_lower} and \code{hdci_upper} (highest density
#'     continuous interval bounds at the level specified by
#'     \code{prob}), and \code{n_eff} (effective sample size for the
#'     parameter).}
#'   \item{item_information}{A \code{\link[tibble]{tibble}} with one
#'     row per item containing: \code{item}, \code{location} (mean
#'     item location, i.e. mean of thresholds for polytomous items),
#'     \code{info_at_location} (Fisher information at the item's own
#'     location), and \code{max_info} (maximum Fisher information
#'     across theta). For Rasch dichotomous items, both are 0.25.
#'     For polytomous items, information depends on the number and
#'     spacing of thresholds.}
#'   \item{threshold_order}{(Polytomous models only) A
#'     \code{\link[tibble]{tibble}} with one row per item containing:
#'     \code{item}, \code{n_thresholds}, \code{ordered} (logical:
#'     are all thresholds in ascending order?), and
#'     \code{prob_disordered} (posterior probability that at least
#'     one pair of adjacent thresholds is disordered, i.e.
#'     \eqn{\tau_{k+1} \le \tau_k}). \code{NULL} for dichotomous
#'     models.}
#'   \item{person_sd}{A \code{\link[tibble]{tibble}} with one row
#'     containing: \code{mean}, \code{sd}, \code{hdci_lower},
#'     \code{hdci_upper} — the posterior summary of the person-level
#'     standard deviation parameter \eqn{\sigma_\theta}.}
#'   \item{draws_matrix}{(Only if \code{draws = TRUE}) A numeric
#'     matrix with rows = thresholds (named
#'     \code{"item[threshold]"}) and columns = posterior draws. For
#'     dichotomous models, row names are item labels only.}
#' }
#'
#' @details
#' \strong{Dichotomous models} with random item effects
#' (\code{response ~ 1 + (1 | item) + (1 | id)}) parameterise item
#' difficulty as \eqn{\delta_i = -(b_0 + r_i)} where \eqn{b_0} is
#' the global intercept and \eqn{r_i} is the item random effect.
#' Models with fixed item effects (\code{response ~ 0 + item +
#' (1 | id)}) parameterise difficulty as \eqn{\delta_i = -b_i}.
#'
#' \strong{Polytomous (acat/PCM) models} with grouped thresholds
#' (\code{thres(gr = item)}) directly estimate item-specific
#' threshold parameters. Each row in the long output represents one
#' threshold within one item; each row in the wide output represents
#' one item.
#'
#' \strong{Item information} is computed from the posterior mean
#' item parameters using the standard Rasch/PCM information
#' formulae:
#' \describe{
#'   \item{Dichotomous}{
#'     \eqn{I_i(\theta) = P_i(\theta) Q_i(\theta)} where
#'     \eqn{P_i = \text{logistic}(\theta - \delta_i)}.}
#'   \item{Polytomous (PCM)}{
#'     \eqn{I_i(\theta) = \sum_c (c - E_i)^2 P_{ic}(\theta)} where
#'     \eqn{E_i = \sum_c c \cdot P_{ic}} is the expected score for
#'     item \eqn{i}.}
#' }
#'
#' \strong{Threshold ordering}: In the partial credit model,
#' disordered thresholds (\eqn{\tau_{k+1} \le \tau_k}) indicate that
#' the probability of responding in the intermediate category never
#' exceeds both adjacent categories — the category is empirically
#' "absorbed". This does not necessarily indicate misfit (see
#' Adams et al., 2012), but may suggest response categories should
#' be collapsed. The \code{prob_disordered} column reports the
#' posterior probability of at least one disordered pair per item,
#' providing a Bayesian alternative to post-hoc threshold checks.
#'
#' @references
#' Adams, R. J., Wu, M. L., & Wilson, M. (2012). The Rasch rating
#' model and the disordered threshold controversy. \emph{Educational
#' and Psychological Measurement}, \emph{72}(4), 547--573.
#' \doi{10.1177/0013164411432166}
#'
#' Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R
#' with brms and Stan. \emph{Journal of Statistical Software},
#' \emph{100}, 1--54. \doi{10.18637/jss.v100.i05}
#'
#' @seealso
#' \code{\link{person_parameters}}, \code{\link{plot_targeting}},
#' \code{\link{plot_ipf}}, \code{\link{posterior_to_prior}}.
#'
#' @examples
#' \donttest{
#' library(brms)
#' library(dplyr)
#' library(tidyr)
#' library(tibble)
#'
#' # --- Partial Credit Model ---
#'
#' df_pcm <- eRm::pcmdat2 %>%
#'   mutate(across(everything(), ~ .x + 1)) %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_pcm <- brm(
#'   response | thres(gr = item) ~ 1 + (1 | id),
#'   data   = df_pcm,
#'   family = acat,
#'   chains = 4, cores = 2, iter = 1000 # use more iter and cores
#' )
#'
#' ip <- item_parameters(fit_pcm)
#'
#' # Long format: one row per threshold
#' ip$locations
#'
#' # Wide format: one row per item, easy to scan
#' ip$locations_wide
#'
#' # Full summary with SE and HDCI
#' ip$summary
#'
#' # Item information
#' ip$item_information
#'
#' # Threshold ordering diagnostic
#' ip$threshold_order
#'
#' # Person SD
#' ip$person_sd
#'
#' # With full posterior draws
#' ip_draws <- item_parameters(fit_pcm, draws = TRUE)
#' dim(ip_draws$draws_matrix)
#'
#' # --- Dichotomous Rasch ---
#'
#' df_rm <- eRm::raschdat3 %>%
#'   as.data.frame() %>%
#'   rownames_to_column("id") %>%
#'   pivot_longer(!id, names_to = "item", values_to = "response")
#'
#' fit_rm <- brm(
#'   response ~ 1 + (1 | item) + (1 | id),
#'   data   = df_rm,
#'   family = bernoulli(),
#'   chains = 4, cores = 2, iter = 1000 # use more iter and cores
#' )
#'
#' ip_rm <- item_parameters(fit_rm)
#' # Wide and long are equivalent for dichotomous:
#' ip_rm$locations
#' ip_rm$locations_wide
#' ip_rm$summary
#' }
#'
#' @importFrom brms as_draws_df ndraws
#' @importFrom ggdist mean_hdci
#' @importFrom rlang enquo as_name .data
#' @importFrom stats family sd plogis optimize var acf
#' @importFrom tibble tibble as_tibble
#' @importFrom dplyr arrange
#' @export
item_parameters <- function(
    model,
    item_var   = item,
    person_var = id,
    draws      = FALSE,
    center     = TRUE,
    prob       = 0.95
) {
  
  # ================================================================
  # INPUT VALIDATION
  # ================================================================
  if (!inherits(model, "brmsfit")) {
    stop("'model' must be a brmsfit object.", call. = FALSE)
  }
  if (!is.numeric(prob) || prob <= 0 || prob >= 1) {
    stop("'prob' must be a numeric value between 0 and 1.",
         call. = FALSE)
  }
  
  item_name   <- rlang::as_name(rlang::enquo(item_var))
  person_name <- rlang::as_name(rlang::enquo(person_var))
  
  if (!item_name %in% names(model$data)) {
    stop("Item variable '", item_name, "' not found in model data.",
         call. = FALSE)
  }
  if (!person_name %in% names(model$data)) {
    stop("Person variable '", person_name, "' not found in model data.",
         call. = FALSE)
  }
  
  all_draws   <- tibble::as_tibble(brms::as_draws_df(model))
  family_name <- stats::family(model)$family
  is_ordinal  <- grepl("acat|cumul|sratio|cratio",
                       family_name, ignore.case = TRUE)
  
  unique_items <- sort(unique(model$data[[item_name]]))
  
  # ================================================================
  # DETECT MODEL PARAMETERISATION
  # ================================================================
  fe_pattern <- paste0("^b_", item_name)
  fe_cols    <- grep(fe_pattern, names(all_draws), value = TRUE)
  fe_cols    <- setdiff(fe_cols, "b_Intercept")
  
  has_fixed_items  <- !is_ordinal && length(fe_cols) > 0
  has_random_items <- !is_ordinal && any(grepl(
    paste0("^r_", item_name, "\\["), names(all_draws)
  ))
  
  if (!is_ordinal && !has_fixed_items && !has_random_items) {
    stop("Could not detect item parameters in the model. ",
         "Supported parameterisations:\n",
         "  - response ~ 0 + item + (1 | id)    [fixed items]\n",
         "  - response ~ 1 + (1 | item) + (1 | id)  [random items]\n",
         "  - response | thres(gr = item) ~ 1 + (1 | id) [PCM/acat]",
         call. = FALSE)
  }
  
  # ================================================================
  # EXTRACT DRAW-LEVEL ITEM PARAMETERS
  # ================================================================
  param_draws <- .extract_item_param_draws(
    all_draws, unique_items, item_name,
    is_ordinal, has_fixed_items
  )
  
  # ================================================================
  # CENTERING
  # ================================================================
  all_means <- vapply(
    unlist(param_draws, recursive = FALSE),
    mean, numeric(1)
  )
  shift <- if (center) mean(all_means) else 0
  
  param_draws <- lapply(param_draws, function(item_list) {
    lapply(item_list, function(draw_vec) draw_vec - shift)
  })
  
  # ================================================================
  # BUILD LOCATIONS TABLE — LONG FORMAT
  # ================================================================
  loc_rows <- list()
  for (item_label in names(param_draws)) {
    thresh_list <- param_draws[[item_label]]
    for (k in seq_along(thresh_list)) {
      loc_rows[[length(loc_rows) + 1]] <- data.frame(
        item      = item_label,
        threshold = as.integer(k),
        location  = round(mean(thresh_list[[k]]), 4),
        stringsAsFactors = FALSE
      )
    }
  }
  locations_long <- tibble::as_tibble(do.call(rbind, loc_rows))
  
  # ================================================================
  # BUILD LOCATIONS TABLE — WIDE FORMAT
  # ================================================================
  max_thresh <- max(vapply(param_draws, length, integer(1)))
  wide_rows  <- list()
  
  for (item_label in names(param_draws)) {
    thresh_list  <- param_draws[[item_label]]
    thresh_means <- vapply(thresh_list, mean, numeric(1))
    
    row <- list(item = item_label)
    
    if (max_thresh == 1) {
      # Dichotomous: single "location" column
      row[["location"]] <- round(thresh_means[1], 4)
    } else {
      # Polytomous: t1, t2, ... + mean location
      for (k in seq_len(max_thresh)) {
        col_name <- paste0("t", k)
        if (k <= length(thresh_means)) {
          row[[col_name]] <- round(thresh_means[k], 4)
        } else {
          row[[col_name]] <- NA_real_
        }
      }
      row[["location"]] <- round(mean(thresh_means), 4)
    }
    
    wide_rows[[length(wide_rows) + 1]] <- as.data.frame(
      row, stringsAsFactors = FALSE
    )
  }
  locations_wide <- tibble::as_tibble(do.call(rbind, wide_rows))
  locations_wide <- dplyr::arrange(locations_wide, .data$location)
  
  # ================================================================
  # BUILD SUMMARY TABLE (with SE, HDCI, n_eff)
  # ================================================================
  sum_rows  <- list()
  param_idx <- 0
  for (item_label in names(param_draws)) {
    thresh_list <- param_draws[[item_label]]
    for (k in seq_along(thresh_list)) {
      param_idx <- param_idx + 1
      vals <- thresh_list[[k]]
      
      hdci  <- ggdist::mean_hdci(vals, .width = prob)
      n_eff <- .compute_ess(vals)
      
      sum_rows[[param_idx]] <- data.frame(
        item       = item_label,
        threshold  = as.integer(k),
        location   = round(mean(vals), 4),
        se         = round(stats::sd(vals), 4),
        hdci_lower = round(hdci$ymin, 4),
        hdci_upper = round(hdci$ymax, 4),
        n_eff      = round(n_eff),
        stringsAsFactors = FALSE
      )
    }
  }
  summary_tbl <- tibble::as_tibble(do.call(rbind, sum_rows))
  
  # ================================================================
  # ITEM INFORMATION
  # ================================================================
  info_rows <- list()
  for (item_label in names(param_draws)) {
    thresh_list  <- param_draws[[item_label]]
    thresh_means <- vapply(thresh_list, mean, numeric(1))
    item_loc     <- mean(thresh_means)
    
    if (length(thresh_means) == 1) {
      p_at_loc    <- stats::plogis(item_loc - thresh_means[1])
      info_at_loc <- p_at_loc * (1 - p_at_loc)
      max_info    <- 0.25
    } else {
      info_at_loc <- .pcm_item_info(item_loc, thresh_means)
      opt <- stats::optimize(
        function(th) .pcm_item_info(th, thresh_means),
        interval = c(min(thresh_means) - 3, max(thresh_means) + 3),
        maximum  = TRUE
      )
      max_info <- opt$objective
    }
    
    info_rows[[length(info_rows) + 1]] <- data.frame(
      item             = item_label,
      location         = round(item_loc, 4),
      info_at_location = round(info_at_loc, 4),
      max_info         = round(max_info, 4),
      stringsAsFactors = FALSE
    )
  }
  item_info <- tibble::as_tibble(do.call(rbind, info_rows))
  
  # ================================================================
  # THRESHOLD ORDERING (polytomous only)
  # ================================================================
  threshold_order <- NULL
  if (is_ordinal) {
    ord_rows <- list()
    for (item_label in names(param_draws)) {
      thresh_list <- param_draws[[item_label]]
      n_thresh    <- length(thresh_list)
      
      if (n_thresh < 2) {
        ord_rows[[length(ord_rows) + 1]] <- data.frame(
          item            = item_label,
          n_thresholds    = n_thresh,
          ordered         = TRUE,
          prob_disordered = 0,
          stringsAsFactors = FALSE
        )
        next
      }
      
      thresh_means <- vapply(thresh_list, mean, numeric(1))
      is_ordered   <- all(diff(thresh_means) > 0)
      
      n_draws_total    <- length(thresh_list[[1]])
      disordered_count <- 0
      for (s in seq_len(n_draws_total)) {
        draw_thresholds <- vapply(
          thresh_list, function(x) x[s], numeric(1)
        )
        if (any(diff(draw_thresholds) <= 0)) {
          disordered_count <- disordered_count + 1
        }
      }
      prob_disord <- disordered_count / n_draws_total
      
      ord_rows[[length(ord_rows) + 1]] <- data.frame(
        item            = item_label,
        n_thresholds    = n_thresh,
        ordered         = is_ordered,
        prob_disordered = round(prob_disord, 4),
        stringsAsFactors = FALSE
      )
    }
    threshold_order <- tibble::as_tibble(do.call(rbind, ord_rows))
  }
  
  # ================================================================
  # PERSON SD
  # ================================================================
  person_sd_col <- grep(
    paste0("^sd_", person_name, "__Intercept$"),
    names(all_draws), value = TRUE
  )
  if (length(person_sd_col) == 1) {
    sd_vals <- all_draws[[person_sd_col]]
    sd_hdci <- ggdist::mean_hdci(sd_vals, .width = prob)
    person_sd <- tibble::tibble(
      mean       = round(mean(sd_vals), 4),
      sd         = round(stats::sd(sd_vals), 4),
      hdci_lower = round(sd_hdci$ymin, 4),
      hdci_upper = round(sd_hdci$ymax, 4)
    )
  } else {
    warning("Could not find person-level SD parameter '",
            paste0("sd_", person_name, "__Intercept"),
            "' in the posterior draws.", call. = FALSE)
    person_sd <- NULL
  }
  
  # ================================================================
  # FULL POSTERIOR DRAWS (optional)
  # ================================================================
  out <- list(
    locations        = locations_long,
    locations_wide   = locations_wide,
    summary          = summary_tbl,
    item_information = item_info,
    threshold_order  = threshold_order,
    person_sd        = person_sd
  )
  
  if (draws) {
    draw_list <- list()
    row_names <- character(0)
    for (item_label in names(param_draws)) {
      thresh_list <- param_draws[[item_label]]
      for (k in seq_along(thresh_list)) {
        draw_list[[length(draw_list) + 1]] <- thresh_list[[k]]
        if (length(thresh_list) == 1) {
          row_names <- c(row_names, item_label)
        } else {
          row_names <- c(row_names,
                         paste0(item_label, "[", k, "]"))
        }
      }
    }
    draws_mat <- do.call(rbind, draw_list)
    rownames(draws_mat) <- row_names
    out$draws_matrix <- draws_mat
  }
  
  out
}


# ── Internal: extract draw-level item threshold parameters ───────
#' @keywords internal
.extract_item_param_draws <- function(draws, unique_items, item_name,
                                      is_ordinal, has_fixed_items) {
  
  result <- vector("list", length(unique_items))
  names(result) <- unique_items
  
  if (is_ordinal) {
    has_grouped <- any(grepl("^b_Intercept\\[.+,\\d+\\]$", names(draws)))
    
    for (item_label in unique_items) {
      if (has_grouped) {
        thresh_pattern <- paste0(
          "^b_Intercept\\[",
          gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
          ","
        )
        thresh_cols <- grep(thresh_pattern, names(draws), value = TRUE)
      } else {
        thresh_cols <- grep("^b_Intercept\\[\\d+\\]$", names(draws),
                            value = TRUE)
      }
      
      if (length(thresh_cols) == 0) {
        warning("No threshold parameters found for item '",
                item_label, "'.", call. = FALSE)
        result[[item_label]] <- list()
        next
      }
      
      thresh_nums <- as.numeric(
        gsub(".*,(\\d+)\\]$|.*\\[(\\d+)\\]$", "\\1\\2", thresh_cols)
      )
      thresh_cols <- thresh_cols[order(thresh_nums)]
      
      result[[item_label]] <- lapply(
        thresh_cols,
        function(col) as.numeric(draws[[col]])
      )
    }
    
  } else if (has_fixed_items) {
    for (item_label in unique_items) {
      col_pattern <- paste0("^b_", item_name,
                            gsub("([.|()\\^{}+$*?])", "\\\\\\1",
                                 item_label), "$")
      col_name <- grep(col_pattern, names(draws), value = TRUE)
      
      if (length(col_name) == 0) {
        warning("Could not find fixed effect for item '",
                item_label, "'.", call. = FALSE)
        result[[item_label]] <- list(rep(NA_real_, nrow(draws)))
        next
      }
      
      result[[item_label]] <- list(-as.numeric(draws[[col_name[1]]]))
    }
    
  } else {
    intercept_col <- grep("^b_Intercept$", names(draws), value = TRUE)
    intercept_draws <- if (length(intercept_col) == 1) {
      as.numeric(draws[[intercept_col]])
    } else {
      rep(0, nrow(draws))
    }
    
    for (item_label in unique_items) {
      re_col <- paste0("r_", item_name, "[", item_label, ",Intercept]")
      if (!re_col %in% names(draws)) {
        re_col_alt <- grep(
          paste0("^r_", item_name, "\\[",
                 gsub("([.|()\\^{}+$*?])", "\\\\\\1", item_label),
                 ",Intercept\\]$"),
          names(draws), value = TRUE
        )
        if (length(re_col_alt) == 1) {
          re_col <- re_col_alt
        } else {
          warning("Could not find random effect for item '",
                  item_label, "'.", call. = FALSE)
          result[[item_label]] <- list(rep(NA_real_, nrow(draws)))
          next
        }
      }
      
      result[[item_label]] <- list(
        -(intercept_draws + as.numeric(draws[[re_col]]))
      )
    }
  }
  
  result
}


# ── Internal: PCM item information at a given theta ──────────────
#' @keywords internal
.pcm_item_info <- function(theta, thresholds) {
  n_cat <- length(thresholds) + 1
  cats  <- seq(0, n_cat - 1)
  
  cum_eta   <- c(0, cumsum(theta - thresholds))
  max_eta   <- max(cum_eta)
  log_denom <- max_eta + log(sum(exp(cum_eta - max_eta)))
  p_cat     <- exp(cum_eta - log_denom)
  
  E_i <- sum(cats * p_cat)
  V_i <- sum((cats - E_i)^2 * p_cat)
  V_i
}


# ── Internal: compute effective sample size ──────────────────────
#' @keywords internal
.compute_ess <- function(x) {
  x <- x[!is.na(x)]
  n <- length(x)
  if (n < 4) return(NA_real_)
  
  v <- stats::var(x)
  if (v < .Machine$double.eps) return(n)
  
  acf_vals <- stats::acf(x, lag.max = n - 1, plot = FALSE,
                         demean = TRUE)$acf[, 1, 1]
  
  sum_rho <- 0
  lag <- 2
  while (lag < length(acf_vals)) {
    pair_sum <- acf_vals[lag] + acf_vals[lag + 1]
    if (pair_sum < 0) break
    sum_rho <- sum_rho + pair_sum
    lag <- lag + 2
  }
  
  ess <- n / (1 + 2 * sum_rho)
  max(1, ess)
}

test_that("exported functions exist and are functions", {
  expect_true(is.function(RMUreliability))
  expect_true(is.function(dif_statistic))
  expect_true(is.function(fit_statistic_pcm))
  expect_true(is.function(fit_statistic_rm))
  expect_true(is.function(infit_post))
  expect_true(is.function(infit_statistic))
  expect_true(is.function(item_parameters))
  expect_true(is.function(item_restscore_post))
  expect_true(is.function(item_restscore_statistic))
  expect_true(is.function(person_parameters))
  expect_true(is.function(plot_bars))
  expect_true(is.function(plot_icc))
  expect_true(is.function(plot_ipf))
  expect_true(is.function(plot_residual_pca))
  expect_true(is.function(plot_stackedbars))
  expect_true(is.function(plot_targeting))
  expect_true(is.function(plot_tile))
  expect_true(is.function(posterior_to_prior))
  expect_true(is.function(q3_post))
  expect_true(is.function(q3_statistic))
})

---
title: "Rasch Partial Credit Model with easyRaschBayes"
output: rmarkdown::html_vignette
vignette: >
  %\VignetteIndexEntry{Rasch Partial Credit Model}
  %\VignetteEngine{knitr::rmarkdown}
  %\VignetteEncoding{UTF-8}
---



## Overview

This vignette demonstrates a Partial Credit Model (PCM) Rasch analysis
workflow using `easyRaschBayes`. The analysis uses the `eRm::pcmdat2` dataset, a
small polytomous item response data set included in the `eRm` package.

All functions in this package work with a Bayesian `brms` model object fitted with the
`acat` (adjacent categories) family, which parameterises the PCM. Dichotomous 
Rasch models can also be fit using `brms` and analyzed with the functions in
this package. A code example is available 
[here](https://pgmj.github.io/reliability.html#rasch-dichotomous-model), and more 
detail is available in [Bürkner, 2021](https://doi.org/10.18637/jss.v100.i05).

Below, there is some brief texts explaining the output and interpretation of key functions.
For a more extensive treatment of various Rasch analysis aspects, please see the
[`easyRasch` vignette](https://pgmj.github.io/raschrvignette/RaschRvign.html). 
Also, each function includes documentation and example code, use `?function` in your
console.

## Data Preparation


``` r
library(easyRaschBayes)
library(brms)
library(dplyr)
library(tidyr)
library(tibble)
library(ggplot2)
```

`eRm::pcmdat2` is in wide format with item responses coded 0, 1, 2, …. The
`brms` `acat` family expects response categories starting at **1**, so we add 1
to every response before reshaping to long format.


``` r
df_pcm <- eRm::pcmdat2 %>%
  mutate(across(everything(), ~ .x + 1)) %>%
  rownames_to_column("id") %>%
  pivot_longer(!id, # don't include the id variable in the wide->long transformation
               names_to = "item", 
               values_to = "response")

head(df_pcm)
#> # A tibble: 6 × 3
#>   id    item  response
#>   <chr> <chr>    <dbl>
#> 1 1     I1           2
#> 2 1     I2           2
#> 3 1     I3           2
#> 4 1     I4           2
#> 5 2     I1           1
#> 6 2     I2           1
```

## Visualizing response data

Before analyzing data, it is good to review the response distributions. There are 
three simple plotting functions included in the package. All make use of wide 
format datasets, where items are separate columns and respondents are rows.


``` r
plot_stackedbars(eRm::pcmdat2)
```

<div class="figure">
<img src="figures/pcm-data-viz-1.png" alt="Stacked bars" width="100%" />
<p class="caption">Stacked bars</p>
</div>

The other two visualization functions are `plot_tile()` and `plot_bars()`

## Fitting the PCM

The model is fitted once and saved to disk. The code chunk below shows the
fitting call (not evaluated during `R CMD check`). A pre-fitted model stored at
`fits/fit_pcm.rds` is loaded instead.


``` r
prior_pcm <- prior("normal(0, 3)", class = "Intercept") +
  prior("normal(0, 3)", class = "sd", group = "id")
```


``` r
fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data    = df_pcm,
  family  = acat,
  prior   = prior_pcm,
  chains  = 4,
  cores   = 4,
  iter    = 2000
)
saveRDS(fit_pcm, "fits/fit_pcm.rds")
```


``` r
fit_pcm <- readRDS("fits/fit_pcm.rds")
```

## Item Fit: Conditional Infit Statistics

`infit_statistic()` computes posterior predictive infit (and outfit) statistics
for each item. Values near 1.0 indicate good fit; values above 1
suggest underfit (unexpected responses, often indicating multidimensionality), 
values below 1 suggest overfit (too predictable, often coinciding with local
dependence issues).

The `ndraws_use` argument limits the number of posterior draws used, which
speeds up computation during exploration. For final reporting, use all draws
(set `ndraws_use = NULL` or omit it).


``` r
fit_stats <- infit_statistic(fit_pcm, ndraws_use = 500)

# Post-process Infit
infit_results <- infit_post(fit_stats)
infit_results$summary
#> # A tibble: 4 × 4
#>   item  infit_obs infit_rep infit_ppp
#>   <chr>     <dbl>     <dbl>     <dbl>
#> 1 I1        1.04      1.00      0.292
#> 2 I2        1.08      1         0.11 
#> 3 I3        0.917     1.00      0.89 
#> 4 I4        1.03      0.999     0.364
infit_results$plot
```

<div class="figure">
<img src="figures/pcm-infit-1.png" alt="Conditional infit figure" width="100%" />
<p class="caption">Conditional infit figure</p>
</div>

`infit_obs` indicates the observed conditional infit, which can be compared to `infit_rep`, 
which is akin to the model expected value. Posterior predictive p-values (`*_ppp`) 
close to 0.5 indicate that the observed statistic falls near the centre of the 
posterior predictive distribution, implying good fit. Values near 0 or 1 warrant 
further investigation.

## Item–Rest Score Association

`item_restscore_statistic()` computes Goodman-Kruskal's gamma between each
item's observed responses and the rest score (total score minus the focal item).
In a well-fitting Rasch model, gamma should be positive and moderate and of similar
magnitude for all items; high values may indicate redundancy, low values suggest the item
does not relate well to the latent trait.


``` r
rest_stats <- item_restscore_statistic(fit_pcm, ndraws_use = 500)

rest_results <- item_restscore_post(rest_stats)
rest_results$summary
#> # A tibble: 4 × 5
#>   item  gamma_obs gamma_rep gamma_diff   ppp
#>   <chr>     <dbl>     <dbl>      <dbl> <dbl>
#> 1 I1        0.473     0.541     -0.068 0.118
#> 2 I2        0.441     0.543     -0.102 0.056
#> 3 I3        0.643     0.53       0.112 0.962
#> 4 I4        0.535     0.537     -0.002 0.492
rest_results$plot
```

<div class="figure">
<img src="figures/pcm-restscore-1.png" alt="Item-restscore figure" width="100%" />
<p class="caption">Item-restscore figure</p>
</div>

## Conditional ICC Plot

Shows item fit across the latent continuum, dividing the sample into n class 
intervals (default is 5).


``` r
plot_icc(fit_pcm)
```

<div class="figure">
<img src="figures/pcm-ciccplot-1.png" alt="Conditional Item Characteristic Curves figure" width="100%" />
<p class="caption">Conditional Item Characteristic Curves figure</p>
</div>

## Dimensionality: Residual PCA

`plot_residual_pca()` performs a principal components analysis on the
person-item residuals and plots the standardized loadings on the first residual 
contrast factor together with item locations and the uncertainty of both.


``` r
pca <- plot_residual_pca(fit_pcm, ndraws_use = 500)
pca$plot
```

<div class="figure">
<img src="figures/pcm-pca-plot-1.png" alt="1st residual contrast loadings &amp; locations" width="100%" />
<p class="caption">1st residual contrast loadings & locations</p>
</div>

If the observed largest eigenvalue is smaller than the replicated, unidimensionality
is supported. The ppp should not be close to 1.

## Local Dependence: Q3 Residual Correlations

`q3_statistic()` computes Yen's Q3 statistic — the correlation between
person-item residuals for every item pair. After conditioning on the latent
trait, residuals should be uncorrelated; elevated Q3 values indicate local
dependence (LD). Our primary metric here is the ppp, that should not be close to 1.


``` r
q3_stats <- q3_statistic(fit_pcm, ndraws_use = 500)

q3_results <- q3_post(q3_stats)
q3_results$summary
#> # A tibble: 6 × 7
#>   item_pair item_1 item_2 q3_obs q3_rep q3_diff q3_ppp
#>   <chr>     <chr>  <chr>   <dbl>  <dbl>   <dbl>  <dbl>
#> 1 I3 : I4   I3     I4      0.34   0.001   0.339  1    
#> 2 I1 : I2   I1     I2      0.102  0.002   0.1    0.994
#> 3 I1 : I3   I1     I3     -0.072 -0.004  -0.068  0.016
#> 4 I2 : I3   I2     I3     -0.088 -0.003  -0.085  0    
#> 5 I1 : I4   I1     I4     -0.132  0      -0.132  0    
#> 6 I2 : I4   I2     I4     -0.163 -0.002  -0.161  0
q3_results$plot
```

<div class="figure">
<img src="figures/pcm-q3-1.png" alt="Q3 residual correlations" width="100%" />
<p class="caption">Q3 residual correlations</p>
</div>

## Item Category Probabilities

This plot shows the probability of using a response category on the y axis and
the latent score on the x axis. The crossover points, where lines meet, are the 
item category threshold locations. Uncertainty is shown with the shaded area around
each line.


``` r
plot_ipf(fit_pcm, theta_range = c(-6,5))
```

<div class="figure">
<img src="figures/pcm-ipf-plot-1.png" alt="Item Probability Functions" width="100%" />
<p class="caption">Item Probability Functions</p>
</div>


## Person–Item Targeting

`plot_targeting()` produces a Wright map (person–item targeting plot) showing
the distribution of person locations alongside the item threshold locations on
the same logit scale. Good targeting occurs when person and item distributions
overlap substantially.


``` r
plot_targeting(fit_pcm)
```

<div class="figure">
<img src="figures/pcm-targeting-1.png" alt="Targeting figure" width="100%" />
<p class="caption">Targeting figure</p>
</div>

## Reliability: Relative Measurement Uncertainty

`RMUreliability()` provides a Bayesian reliability estimate via Relative
Measurement Uncertainty (RMU, see Bignardi et al., 2025). It requires a matrix 
of person location draws with dimensions \[persons × draws\]. The output is a 
point estimate and lower/upper 95% highest density continuous intervals (HDCI).


``` r
person_draws <- fit_pcm %>%
  as_draws_df() %>%
  as_tibble() %>% 
  select(starts_with("r_id")) %>%
  t()
rmu <- RMUreliability(person_draws)
rmu
#>   rmu_estimate hdci_lowerbound hdci_upperbound .width .point .interval
#> 1    0.6727799       0.6137229       0.7280385   0.95   mean      hdci
```

RMU values range from 0 to 1, with higher values indicating higher reliability, 
similarly to traditional reliability metrics such as Cronbach's alpha.

## Item Parameters


``` r
ipar <- item_parameters(fit_pcm)
knitr::kable(ipar$summary)
```



|item | threshold| location|     se| hdci_lower| hdci_upper| n_eff|
|:----|---------:|--------:|------:|----------:|----------:|-----:|
|I1   |         1|  -0.4792| 0.1819|    -0.8505|    -0.1457|  3734|
|I1   |         2|   1.8135| 0.1840|     1.4532|     2.1753|  3503|
|I2   |         1|   0.2591| 0.1751|    -0.0743|     0.6055|  3423|
|I2   |         2|   1.6465| 0.1908|     1.3032|     2.0377|  3350|
|I3   |         1|  -2.5798| 0.3347|    -3.2377|    -1.9333|  4000|
|I3   |         2|   0.2413| 0.1569|    -0.0609|     0.5475|  3255|
|I4   |         1|  -1.4880| 0.2529|    -1.9889|    -1.0082|  4000|
|I4   |         2|   0.5866| 0.1608|     0.2573|     0.8902|  4000|



``` r
knitr::kable(ipar$locations_wide)
```



|item |      t1|     t2| location|
|:----|-------:|------:|--------:|
|I3   | -2.5798| 0.2413|  -1.1693|
|I4   | -1.4880| 0.5866|  -0.4507|
|I1   | -0.4792| 1.8135|   0.6672|
|I2   |  0.2591| 1.6465|   0.9528|



## Person Parameters

This estimates latent scores.


``` r
ppar <- person_parameters(fit_pcm)
knitr::kable(ppar$score_table)
```



| sum_score|  n|     eap| eap_se|     wle| wle_se|
|---------:|--:|-------:|------:|-------:|------:|
|         0|  7| -1.9948| 0.8723| -7.0000|    NaN|
|         1|  4| -1.3413| 0.7962| -3.0165| 1.3597|
|         2| 23| -0.7713| 0.7451| -1.5860| 0.9883|
|         3| 35| -0.2446| 0.7225| -0.6911| 0.8665|
|         4| 80|  0.2582| 0.7107|  0.0616| 0.8151|
|         5| 35|  0.7609| 0.7140|  0.7901| 0.8208|
|         6| 51|  1.2865| 0.7428|  1.6205| 0.9128|
|         7| 28|  1.8658| 0.8013|  2.9849| 1.3554|
|         8| 37|  2.5625| 0.8963|  7.0000|    NaN|



``` r
hist(ppar$person_estimates$eap, col = "lightblue", main = "Histogram of EAP scores")
```

<div class="figure">
<img src="figures/pcm-ppar-1.png" alt="EAP score histogram" width="100%" />
<p class="caption">EAP score histogram</p>
</div>

## References 

- Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with brms and
Stan. *Journal of Statistical Software*, *100*, 1–54.
<https://doi.org/10.18637/jss.v100.i05>

- Bignardi, G., Kievit, R., & Bürkner, P.-C. (2025). A general method for 
estimating reliability using Bayesian Measurement Uncertainty. OSF. 
<https://doi.org/10.31234/osf.io/h54k8_v1>

- Levy, R., Mislevy, R. J., & Sinharay, S. (2009). Posterior Predictive Model 
Checking for Multidimensionality in Item Response Theory. Applied Psychological 
Measurement, 33(7), 519–537. <https://doi.org/10.1177/0146621608329504>

- Sinharay, S. (2006). Bayesian item fit analysis for unidimensional item 
response theory models. British Journal of Mathematical and Statistical 
Psychology, 59(2), 429–449. <https://doi.org/10.1348/000711005X66888>

- Müller, M. (2020). Item fit statistics for Rasch analysis: Can we trust them? 
Journal of Statistical Distributions and Applications, 7(1), 5. 
<https://doi.org/10.1186/s40488-020-00108-7>

- Kreiner, S. (2011). A Note on Item–Restscore Association in Rasch Models. 
Applied Psychological Measurement, 35(7), 557–561. 
<https://doi.org/10.1177/0146621611410227>

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/cicc_plot.R
\name{plot_icc}
\alias{plot_icc}
\title{Item Characteristic Curves with Class Intervals}
\usage{
plot_icc(
  model,
  item_var = item,
  person_var = id,
  items = NULL,
  n_intervals = 5,
  theta_range = c(-4, 4),
  n_points = 200,
  center = TRUE,
  prob = 0.95,
  ncol = NULL,
  line_size = 0.8,
  ribbon_alpha = 0.3,
  point_size = 2.5,
  dif_var = NULL,
  dif_data = NULL,
  dif_labels = NULL,
  dif_stats = TRUE,
  min_n = 5,
  palette = NULL
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from
an ordinal IRT model (e.g., \code{family = acat}) or a
dichotomous model (\code{family = bernoulli()}).}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the
person grouping variable in the model data. Default is
\code{id}.}

\item{items}{An optional character vector of item names to plot.
If \code{NULL} (the default), all items are plotted.}

\item{n_intervals}{Integer. The number of class intervals into
which persons are binned along the sum score. Default is 5.}

\item{theta_range}{A numeric vector of length 2 specifying the
range of theta for the expected curve. Default is
\code{c(-4, 4)}.}

\item{n_points}{Integer. Number of evenly spaced theta values
for computing the expected curve. Default is 200.}

\item{center}{Logical. If \code{TRUE} (the default), the scale
is recentered so the grand mean of item thresholds = 0,
consistent with \code{\link{plot_targeting}}.}

\item{prob}{Numeric in \eqn{(0, 1)}. Width of the credible
interval ribbon around the expected curve. Default is 0.95.}

\item{ncol}{Integer. Number of columns in the faceted layout.
If \code{NULL}, chosen automatically.}

\item{line_size}{Numeric. Line width for the expected curve.
Default is 0.8.}

\item{ribbon_alpha}{Numeric in \eqn{[0, 1]}. Transparency of
the credible interval ribbon. Default is 0.3.}

\item{point_size}{Numeric. Size of observed score points.
Default is 2.5.}

\item{dif_var}{An optional unquoted variable name for a grouping
variable to assess DIF visually. If supplied, observed scores
are computed separately per group and coloured accordingly.}

\item{dif_data}{An optional data frame containing the DIF
variable. Required when \code{dif_var} is specified and the
variable was not part of the model formula (since brms drops
unused variables from \code{model$data}). Must have the same
rows and row order as the original model data.}

\item{dif_labels}{An optional character vector of labels for
the DIF groups. If \code{NULL}, factor levels are used.}

\item{dif_stats}{Logical. If \code{TRUE} (the default when
\code{dif_var} is specified), the partial gamma coefficient
for each item is annotated in the plot panel. The partial
gamma measures the strength of DIF, stratified by total score.}

\item{min_n}{Integer. Minimum number of persons required in a
class interval for the observed mean to be plotted. Intervals
with fewer persons are dropped to avoid unstable estimates.
Default is 5.}

\item{palette}{An optional character vector of colors. If
\code{NULL}, viridis colors are used.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object.
}
\description{
Plots Item Characteristic Curves (ICCs) for Bayesian Rasch-family
models fitted with \pkg{brms}. Each item panel shows the
model-expected item score curve (with a credible interval ribbon)
overlaid with observed average item scores computed within class
intervals. Optionally, observed scores can be split by a grouping
variable to visually assess differential item functioning (DIF).
}
\details{
This is the Bayesian analogue of \code{ICCplot()} from the
\pkg{iarm} package, using the class interval method.

\strong{Expected curve:} For each item, the expected item score
\eqn{E_i(\theta)} is computed for a dense grid of theta values
using the posterior draws of item thresholds. For the adjacent
category (PCM/acat) family:
\deqn{E_i(\theta) = \sum_{c=0}^{K} c \cdot P_{ic}(\theta)}
where \eqn{P_{ic}} are the category probabilities. For
dichotomous items, \eqn{E_i(\theta) = P_i(\theta)}.

The posterior mean of \eqn{E_i(\theta)} is plotted as a line,
with a credible interval ribbon showing the \code{prob} interval
across posterior draws.

\strong{Class intervals:} Following the approach in
\code{iarm::ICCplot()}, persons are binned into class intervals
by their ordinal \emph{sum score} (the sufficient statistic in
Rasch models), not by their EAP theta estimate. Within each
interval, the mean observed item response is computed and
plotted at the theta value corresponding to the mean sum score
in that interval, obtained by inverting the posterior-mean
expected total score function. Persons with extreme sum scores
(0 and maximum) are excluded from the class intervals, as their
theta positions cannot be estimated.

\strong{Uncertainty around observed points:} For each class
interval, the standard error of the mean observed response is
computed and displayed as error bars showing \eqn{\pm 1.96} SE.
These reflect sampling variability of the observed mean within
each bin.

\strong{DIF overlay:} When \code{dif_var} is specified, observed
scores are computed separately for each level of the DIF
variable, producing group-specific points connected by lines.
Deviations between groups that track alongside each other (but
offset from the expected curve) suggest uniform DIF. Crossing
group lines suggest non-uniform DIF.

\strong{Partial gamma:} When \code{dif_stats = TRUE} (default
when DIF is requested), the Goodman-Kruskal partial gamma
coefficient is displayed in each item panel. This measures the
ordinal association between item response and DIF group,
stratified by total score. Values near 0 indicate no DIF;
values near \eqn{\pm 1} indicate strong DIF. The computation
reuses the concordant/discordant pair counting algorithm from
\code{\link{gk_gamma}}.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)
library(ggplot2)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4, cores = 2, iter = 1000 # use more iter and cores
)

# Basic ICC plot with 5 class intervals
plot_icc(fit_pcm)

# Select specific items, more intervals
plot_icc(fit_pcm, items = c("I1", "I2"), n_intervals = 5)

# With DIF overlay and partial gamma annotation

# same dataset, adding a `gender` DIF variable
df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  mutate(gender = sample(c("M", "F"), nrow(.), TRUE)) \%>\% 
  pivot_longer(!c(id,gender), names_to = "item", values_to = "response")
  
plot_icc(fit_pcm, dif_var = gender, dif_data = df_pcm,
         dif_labels = c("Female", "Male"), n_intervals = 5)
}

}
\seealso{
\code{\link{plot_ipf}} for category probability curves,
\code{\link{plot_targeting}} for person-item maps,
\code{\link{dif_statistic}} for formal Bayesian DIF testing,
\code{\link{gk_gamma}} for the underlying gamma algorithm.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/person_parameters.R
\name{person_parameters}
\alias{person_parameters}
\title{Extract Person Parameters from a Bayesian Rasch Model}
\usage{
person_parameters(
  model,
  item_var = item,
  person_var = id,
  draws = FALSE,
  center = TRUE,
  theta_range = c(-7, 7)
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object. Supported
parameterisations:
\describe{
\item{Polytomous ordinal (PCM)}{e.g., \code{family = acat} with
\code{thres(gr = item)}, producing item-specific thresholds.}
\item{Dichotomous Rasch (random items)}{e.g.,
\code{response ~ 1 + (1 | item) + (1 | id)} with
\code{family = bernoulli()}.}
\item{Dichotomous 1PL (fixed items)}{e.g.,
\code{response ~ 0 + item + (1 | id)} with
\code{family = bernoulli()}.}
}}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data. Default is \code{id}.}

\item{draws}{Logical. If \code{TRUE}, a matrix of full posterior
draws (persons x draws) is included in the output. Default is
\code{FALSE}.}

\item{center}{Logical. If \code{TRUE} (the default), person
parameters and item difficulties are recentered so that the
mean item difficulty is zero, matching the convention in
frequentist CML Rasch estimation. If \code{FALSE}, raw brms
parameterisation is used.}

\item{theta_range}{A numeric vector of length 2 specifying the
range for the Newton-Raphson WLE search. Default is
\code{c(-7, 7)}.}
}
\value{
A list with the following elements:
\describe{
\item{person_estimates}{A \code{\link[tibble]{tibble}} with one
row per person containing: the person ID, \code{sum_score},
\code{eap} (posterior mean of person random effect),
\code{eap_se} (posterior SD), \code{wle} (Warm's weighted
likelihood estimate), and \code{wle_se} (asymptotic SE
of the WLE). Rows are ordered to match the original person
order in the model data (i.e., the order of first appearance
of each person ID).}
\item{score_table}{A \code{\link[tibble]{tibble}} mapping each
observed ordinal sum score to its mean EAP, mean EAP SE,
WLE, and WLE SE. Extreme scores (0 and maximum) receive
WLE estimates at the boundary of \code{theta_range} with
\code{NA} standard errors.}
\item{draws_matrix}{(Only if \code{draws = TRUE}) A numeric
matrix with rows = persons and columns = posterior draws.
Row names are person IDs. Rows are ordered to match the
original person order in the model data. Can be passed
directly to \code{\link{RMUreliability}}.}
}
}
\description{
Extracts person ability estimates from a fitted Bayesian Rasch
model. Returns both Bayesian EAP (expected a posteriori) estimates
with posterior SDs and frequentist WLE (Warm's weighted likelihood)
estimates with standard errors, plus a lookup table mapping ordinal
sum scores to both scales.
}
\details{
\strong{EAP estimates} are extracted as the posterior means of
the person random effects (\code{r_id[j, Intercept]}) from
\code{\link[brms]{as_draws_df}}, with the posterior SD serving
as the standard error. These are the standard Bayesian point
estimates and reflect shrinkage toward the population mean.

\strong{WLE estimates} (Warm, 1989) are computed from the
posterior mean item parameters using Newton-Raphson iteration
with adaptive step damping on the Warm-corrected likelihood.
WLE adds a bias correction term \eqn{J(\theta) / (2 I(\theta))}
to the score equations, where \eqn{I(\theta)} is the test
information and \eqn{J(\theta) = \sum_i \sum_c (c - E_i)^3 P_{ic}}
is the sum of third central moments. This produces estimates with
reduced finite-sample bias compared to MLE, especially at
extreme scores (Warm, 1989).

The Newton-Raphson algorithm uses adaptive step damping following
the approach in \pkg{iarm} (Mueller): the maximum allowed step
size shrinks by a factor of 1.05 each iteration, preventing
overshoot and ensuring convergence for near-extreme scores.

For extreme scores (sum score = 0 or maximum possible), the
WLE is not well-defined. These cases are assigned the boundary
values of \code{theta_range} with \code{NA} standard errors.

When \code{center = TRUE} (the default), item difficulty
parameters are shifted so their mean is zero, and EAP person
parameters are shifted by the same constant. WLE is computed
from the centered item parameters. This matches the convention
in frequentist CML Rasch estimation.

\strong{Row ordering:} Both \code{person_estimates} and
\code{draws_matrix} preserve the original person order from
the model data (order of first appearance of each person ID).
This allows direct row-binding with the source data without
re-matching.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Dichotomous Rasch Model (random items) ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4, cores = 2, iter = 1000 # use more iter and cores
)

# Basic usage — rows match original person order
pp <- person_parameters(fit_rm)
pp$person_estimates
pp$score_table

# With full posterior draws (e.g., for RMUreliability)
pp_draws <- person_parameters(fit_rm, draws = TRUE)
RMUreliability(pp_draws$draws_matrix)

# --- Polytomous PCM (acat) ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4, cores = 2, iter = 1000 # use more iter and cores
)

pp_pcm <- person_parameters(fit_pcm)
pp_pcm$score_table
}

}
\references{
Warm, T. A. (1989). Weighted likelihood estimation of ability
in item response theory. \emph{Psychometrika}, \emph{54}(3),
427--450. \doi{10.1007/BF02294627}

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R
with brms and Stan. \emph{Journal of Statistical Software},
\emph{100}, 1--54. \doi{10.18637/jss.v100.i05}

Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
\emph{Rasch Models in Health}. Iste and Wiley, pp. 63--70.
}
\seealso{
\code{\link{RMUreliability}}, \code{\link{plot_targeting}},
\code{\link[brms]{ranef}}, \code{\link[brms]{as_draws_df}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gk_gamma_helpers.R
\name{.gk_gamma_from_tab}
\alias{.gk_gamma_from_tab}
\title{Compute Gamma from a Pre-built Contingency Table}
\usage{
.gk_gamma_from_tab(tab, nr, nc)
}
\arguments{
\item{tab}{Integer or numeric matrix (contingency table).}

\item{nr}{Number of rows.}

\item{nc}{Number of columns.}
}
\value{
A scalar: the gamma coefficient, or \code{NA_real_}.
}
\description{
Shared engine for \code{gk_gamma()} and \code{gk_gamma_vec()}.
Uses cumulative suffix sums and prefix column sums for
\eqn{O(nr \times nc)} concordant/discordant pair counting.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_plots.R
\name{plot_targeting}
\alias{plot_targeting}
\title{Person-Item Map (Targeting Plot) for Bayesian IRT Models}
\usage{
plot_targeting(
  model,
  item_var = item,
  person_var = id,
  robust = FALSE,
  center = TRUE,
  sort_items = c("data", "location"),
  bins = 30,
  prob = 0.95,
  palette = NULL,
  person_fill = "#0072B2",
  threshold_fill = "#D55E00",
  height_ratios = c(3, 2, 5)
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from an
ordinal IRT model (e.g., \code{family = acat}) or a dichotomous
model (\code{family = bernoulli()}).}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data. Default is \code{id}.}

\item{robust}{Logical. If \code{FALSE} (the default), the
histogram annotations use mean ± SD. If \code{TRUE}, median ±
MAD is used instead.}

\item{center}{Logical. If \code{TRUE} (the default), the scale is
recentered so that the grand mean of all item threshold
locations is zero, following the convention in frequentist
Rasch analysis. Person estimates are shifted by the same
constant. If \code{FALSE}, the raw brms parameterisation is
used.}

\item{sort_items}{Character. How to order items on the y-axis of
the bottom panel. \code{"data"} (the default) preserves the
order in which items first appear in the model data, with the
first item at the top. \code{"location"} sorts items by their
mean threshold location (easiest at top, hardest at bottom).}

\item{bins}{Integer. Number of bins for both histograms. Default
is 30.}

\item{prob}{Numeric in \eqn{(0, 1)}. Width of the credible
intervals for the item threshold whiskers. Default is 0.95.}

\item{palette}{An optional character vector of colors for the
response categories. If \code{NULL} (the default), the
\code{viridis} discrete scale is used.}

\item{person_fill}{Fill color for the person histogram. Default is
\code{"#0072B2"} (blue).}

\item{threshold_fill}{Fill color for the threshold histogram.
Default is \code{"#D55E00"} (vermillion).}

\item{height_ratios}{Numeric vector of length 3 specifying the
relative heights of the top (person), middle (threshold), and
bottom (dot-whisker) panels. Default is \code{c(3, 2, 5)}.}
}
\value{
A \code{patchwork} object (combined \code{ggplot}).
}
\description{
Plots a person-item map (also known as a Wright map or targeting
plot) for Bayesian IRT models fitted with \pkg{brms}. The plot
consists of three vertically stacked panels sharing the same
latent variable (theta / logit) x-axis:
}
\details{
\enumerate{
\item \strong{Top}: A histogram of person ability estimates,
with a reference line for the mean (or median) and shading
for ±1 SD (or ±1 MAD).
\item \strong{Middle}: An inverted histogram of item threshold
locations, with a reference line for the mean (or median) and
shading for ±1 SD (or ±1 MAD), mirroring the top panel to
visualise the overlap between person abilities and item
difficulties.
\item \strong{Bottom}: A dot-and-whisker plot of item thresholds
by item, with credible intervals and color-coded response
categories.
}

Together, the top and middle panels form a half-moon (or
back-to-back histogram) display that makes it easy to assess
whether the test is well-targeted to the sample.

\strong{Person estimates} are obtained as the posterior means of
the person random effects from the fitted model via
\code{\link[brms]{ranef}}.

\strong{Item thresholds} are extracted from the posterior draws.
For models with grouped thresholds (\code{thres(gr = item)}),
each item has its own set of threshold parameters. For models
with a single set of thresholds (e.g., dichotomous Rasch with
\code{(1 | item)}), the item random effects are subtracted from
the global thresholds to obtain item-specific locations.

When \code{center = TRUE} (the default), the grand mean of all
item threshold posterior means is computed and subtracted from
every threshold estimate, its credible interval bounds, and every
person estimate. This is a uniform translation of the entire
scale that preserves all relative distances and matches the
zero-centered item difficulty convention used in frequentist CML
estimation.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)
library(ggplot2)
library(patchwork)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

# Default: centered, mean ± SD, items in data order
plot_targeting(fit_pcm)

# Uncentered (raw brms parameterisation)
plot_targeting(fit_pcm, center = FALSE)

# Robust: median ± MAD, items sorted by location
plot_targeting(fit_pcm, robust = TRUE, sort_items = "location")

# --- Dichotomous Rasch Model ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

plot_targeting(fit_rm, sort_items = "location")
}

}
\references{
Wright, B. D. & Stone, M. H. (1979). \emph{Best Test Design}.
MESA Press.

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
1--54. \doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{plot_ipf}} for item category probability curves,
\code{\link[brms]{ranef}},
\code{\link[brms]{as_draws_df}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/reliabilityRMU.R
\name{RMUreliability}
\alias{RMUreliability}
\title{Estimate reliability (Relative Measurement Uncertainty) from Bayesian measurement models}
\usage{
RMUreliability(input_draws, verbose = FALSE, level = 0.95)
}
\arguments{
\item{input_draws}{A matrix or data frame of posterior draws. Rows represent subjects and columns represent draws.}

\item{verbose}{Logical. Print detailed information about the input data. Default is TRUE.}

\item{level}{Numeric. Credibility level for the highest density continuous interval. Default is 0.95.}
}
\value{
A list containing:
\itemize{
\item hdci: A data frame with a point-estimate (posterior mean) and highest density continuous interval for reliability, calculated using the ggdist::mean_hdci function
\item reliability_posterior_draws: A numeric vector of posterior draws for reliability, of length K/2 (K = number of columns/draws in your input_draws matrix)
}
}
\description{
This function measures reliability using posterior draws from a fitted Bayesian model.
}
\details{
To use this function, you will need to provide a matrix (input_draws) that
contains the posterior draws for the parameter you wish to calculate reliability.
The function assumes that rows of input_draws represent subjects and columns represent posterior draws.

For an example of how to apply this function to calculate mean score reliability
using brms, see
\href{https://www.bignardi.co.uk/8_bayes_reliability/tutorial_rmu_sum_score_reliability.html}{this tutorial}.

For an example of how to apply this function to go/go-no task data using brms, see
\href{https://www.bignardi.co.uk/8_bayes_reliability/tutorial_calculating_rmu_gonogo.html}{this tutorial}.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Dichotomous Rasch Model ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

# Extract posterior draws from brms model

posterior_draws = fit_rm \%>\%
  as_draws_df() \%>\%
  as_tibble() \%>\% 
  select(starts_with("r_id")) \%>\%
  t()

# Calculate RMU

RMUreliability(posterior_draws)
}

}
\references{
Bignardi, G., Kievit, R., & Bürkner, P. C. (2025). A general method for estimating reliability using Bayesian Measurement Uncertainty. PsyArXiv. \href{https://osf.io/preprints/psyarxiv/h54k8}{doi:10.31234/osf.io/h54k8}
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_plots.R
\name{plot_stackedbars}
\alias{plot_stackedbars}
\title{Stacked Bar Chart of Item Response Distributions}
\usage{
plot_stackedbars(
  data,
  item_labels = NULL,
  category_labels = NULL,
  show_n = TRUE,
  show_percent = FALSE,
  text_color = "sienna1",
  text_size = 3,
  min_label_n = 0,
  viridis_option = "D",
  viridis_end = 0.99,
  title = "Item responses"
)
}
\arguments{
\item{data}{A data frame in wide format containing only the item
response columns. Each column is one item, each row is one
person. All columns must be numeric (integer-valued). Response
categories may be coded starting from 0 or 1. Do not include
person IDs, grouping variables, or other non-item columns.}

\item{item_labels}{An optional character vector of descriptive
labels for the items (y-axis). Must be the same length as
\code{ncol(data)}. If \code{NULL} (the default), column names
are used.}

\item{category_labels}{An optional character vector of labels
for the response categories (legend). Must be the same length
as the number of response categories spanning from the minimum
to the maximum observed value, ordered from lowest to highest
category. If \code{NULL} (the default), numeric category
values are used.}

\item{show_n}{Logical. If \code{TRUE} (the default), the count
of responses is displayed as a text label inside each bar
segment.}

\item{show_percent}{Logical. If \code{TRUE}, the percentage of
responses is displayed instead of (or in addition to) counts.
Default is \code{FALSE}.}

\item{text_color}{Character. Color for the count/percentage
labels. Default is \code{"sienna1"}.}

\item{text_size}{Numeric. Size of the count/percentage labels.
Default is 3.}

\item{min_label_n}{Integer. Minimum count required for a label
to be displayed within a bar segment. Segments with fewer
responses are left unlabelled to avoid clutter. Default is 0
(all segments labelled).}

\item{viridis_option}{Character. Viridis palette option. One of
\code{"A"} through \code{"H"}. Default is \code{"D"}.}

\item{viridis_end}{Numeric in \eqn{[0, 1]}. End point of the
viridis color scale. Default is 0.99.}

\item{title}{Character. Plot title. Default is
\code{"Item responses"}.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object.
}
\description{
Creates a horizontal stacked bar chart showing the response
distribution for all items. Each bar represents one item, with
segments colored by response category. Counts are displayed as
text labels within each segment. This is a descriptive data
visualization tool intended for use before model fitting.
}
\details{
Items are displayed on the y-axis in the same order as the
columns in \code{data} (first column at the top). Each bar is
divided into segments representing response categories, with
the lowest category on the left and the highest on the right.
The total bar length equals the number of non-missing responses
for that item.

Categories with zero responses still appear in the legend but
produce no visible bar segment, which helps identify gaps in
the response distribution.

\strong{Input requirements:}
\itemize{
\item All columns must be numeric (integer-valued).
\item The data frame must contain at least 2 columns (items)
and at least 1 row (person).
}
}
\examples{
library(ggplot2)
if (requireNamespace("eRm", quietly = TRUE))

# Basic stacked bar chart
plot_stackedbars(eRm::pcmdat2)

# With custom item and category labels
plot_stackedbars(
  eRm::pcmdat2,
  item_labels = c("Mood", "Sleep", "Appetite", "Energy"),
  category_labels = c("Never", "Sometimes", "Often")
)

# Show percentages, suppress small segments
plot_stackedbars(
  eRm::pcmdat2,
  show_percent = TRUE,
  show_n       = FALSE,
  min_label_n  = 5
)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gk_gamma_helpers.R
\name{gk_gamma}
\alias{gk_gamma}
\title{Goodman-Kruskal Gamma from a Contingency Table}
\usage{
gk_gamma(tab)
}
\arguments{
\item{tab}{A matrix or table representing a contingency table with
rows and columns in ordinal order.}
}
\value{
A scalar: the gamma coefficient in \eqn{[-1, 1]}.
}
\description{
Computes the Goodman-Kruskal gamma coefficient for an ordinal
contingency table. This is an internal helper function.
}
\details{
Gamma is defined as \eqn{(C - D) / (C + D)}, where \eqn{C} is the
number of concordant pairs and \eqn{D} is the number of discordant
pairs. Uses cumulative suffix sums for \eqn{O(nr \times nc)}
complexity.
}
\references{
Goodman, L. A. & Kruskal, W. H. (1954). Measures of association for
cross classifications. \emph{Journal of the American Statistical
Association}, \emph{49}(268), 732--764.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post_helpers.R
\name{item_restscore_post}
\alias{item_restscore_post}
\title{Summarize and Plot Posterior Predictive Item-Restscore Associations}
\usage{
item_restscore_post(item_restscore)
}
\arguments{
\item{item_restscore}{A list as returned by
\code{\link{item_restscore_statistic}}, containing at minimum:
\describe{
\item{result}{A data frame with columns including \code{item}
and summary statistics (first 5 columns are used).}
\item{draws}{A data frame with columns \code{item},
\code{gamma} (observed gamma per draw), and
\code{gamma_rep} (replicated gamma per draw).}
}}
}
\value{
A list with two elements:
\describe{
\item{summary}{A \code{\link[tibble]{tibble}} with the first 5
columns of the result table, rounded to 3 decimal places and
sorted by item.}
\item{plot}{A \code{\link[ggplot2]{ggplot}} object showing
the posterior predictive distribution of replicated gamma
(grey filled slab) with the observed gamma values overlaid
as orange diamond points.}
}
}
\description{
Postprocesses the output of \code{\link{item_restscore_statistic}}
to produce a summary table and a slab plot comparing observed
item-restscore gamma associations to the posterior predictive
distribution.
}
\details{
The item-restscore gamma association measures the strength of
the relationship between each item's responses and the rest
score (total score excluding that item). Under good fit, the
observed gamma should fall within the posterior predictive
distribution.

The plot displays:
\describe{
\item{Grey slab}{The posterior predictive distribution of
replicated gamma values, shaded by 84\\% and 95\\% credible
interval levels.}
\item{Orange diamonds}{The observed gamma values per draw,
plotted as points on top of the slab.}
}

Items where the observed gamma (orange) falls consistently
outside the replicated distribution (grey) indicate poor fit
in terms of item discrimination.
}
\examples{
\dontrun{
library(brms)
library(ggplot2)

# Assuming fit_pcm is a fitted brmsfit object
irs <- item_restscore_statistic(fit_pcm)

result <- item_restscore_post(irs)
result$summary
result$plot
}

}
\seealso{
\code{\link{item_restscore_statistic}},
\code{\link{infit_post}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_fitstats.R
\name{fit_statistic_pcm}
\alias{fit_statistic_pcm}
\title{Posterior Predictive Item Fit Statistic for Bayesian IRT Models}
\usage{
fit_statistic_pcm(model, criterion, group, ndraws_use = NULL)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object.}

\item{criterion}{A function with signature \code{function(y, p)} that
computes a pointwise fit criterion. For ordinal and categorical models,
\code{y} is the observed (or replicated) response category and \code{p}
is the model-predicted probability of that category. For binary models,
\code{y} is the binary response and \code{p} is the predicted probability
of success.}

\item{group}{An unquoted variable name (e.g., \code{item} or \code{id})
indicating the grouping variable over which the fit statistic is
aggregated. Typically \code{item} for item fit or \code{id} for person
fit.}

\item{ndraws_use}{Optional positive integer. If specified, a random subset
of posterior draws of this size is used, which can speed up computation
for large models. If \code{NULL} (the default), all draws are used.}
}
\value{
A \code{\link[tibble]{tibble}} with the following columns:
\describe{
\item{\code{group}}{The grouping variable (e.g., item name or person id).}
\item{draw}{Integer index of the posterior draw.}
\item{crit}{The observed fit statistic (criterion applied to observed
data) summed within each group and draw.}
\item{crit_rep}{The replicated fit statistic (criterion applied to
posterior predicted data) summed within each group and draw.}
\item{crit_diff}{The difference \code{crit_rep - crit}.}
}
The output is grouped by the grouping variable. Posterior predictive
p-values can be obtained by computing
\code{mean(crit_rep > crit)} within each group.
}
\description{
Computes posterior predictive item (or person) fit statistics for
Bayesian IRT models fitted with \pkg{brms}. For each posterior draw,
observed and replicated data are compared via a user-supplied criterion
function, grouped by item, person, or any other variable. Posterior
predictive p-values can then be derived from the output to assess fit.
}
\details{
The function implements the posterior predictive checking approach for
item fit described in Bürkner (2020). The procedure works as follows:

\enumerate{
\item Draw posterior expected category probabilities via
\code{\link[brms]{posterior_epred}} and posterior predicted responses
via \code{\link[brms]{posterior_predict}}.
\item For ordinal or categorical models (3D array output from
\code{posterior_epred}), extract the probability assigned to the
observed response category and to the replicated response category
for each draw and observation.
\item Apply the user-supplied \code{criterion} function to compute
pointwise fit values for both observed and replicated data.
\item Aggregate (sum) the criterion values within each level of
\code{group} and each posterior draw.
}

A common choice for ordinal IRT models is the categorical
log-likelihood criterion \code{function(y, p) log(p)}. For binary
(e.g., dichotomous Rasch) models, the Bernoulli log-likelihood
\code{function(y, p) y * log(p) + (1 - y) * log(1 - p)} may be used
instead.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Polytomous Rasch (Partial Credit Model) ---

# Prepare data in long format
df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

# Fit a Partial Credit Model using the adjacent category family
fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data    = df_pcm,
  family  = acat,
  chains  = 4,
  cores   = 1, # use more cores if you have
  iter    = 500 # use at least 2000 
)

# Categorical log-likelihood criterion (for polytomous models)
ll_categorical <- function(y, p) log(p)

# Compute item fit statistics
item_fit <- fit_statistic_pcm(
  model      = fit_pcm,
  criterion  = ll_categorical,
  group      = item,
  ndraws_use = 100 # use at least 500
)

# Summarise: posterior predictive p-values per item
item_fit \%>\%
  group_by(item) \%>\%
  summarise(
    observed   = mean(crit),
    replicated = mean(crit_rep),
    ppp        = mean(crit_rep > crit)
  )

# Use ggplot2 to make a histogram
library(ggplot2)
item_fit \%>\%
  ggplot(aes(crit_diff)) +
  geom_histogram(aes(fill = ifelse(crit_diff > 0, "above","below"))) +
  facet_wrap("item") +
  theme_bw() +
  theme(legend.position = "none")

# Compute person fit statistics
person_fit <- fit_statistic_pcm(
  model      = fit_pcm,
  criterion  = ll_categorical,
  group      = id,
  ndraws_use = 100 # use at least 500
)
}

}
\references{
Bürkner, P.-C. (2020). Analysing Standard Progressive Matrices (SPM-LS)
with Bayesian Item Response Models. \emph{Journal of Intelligence},
\emph{8}(1). \doi{10.3390/jintelligence8010005}

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with brms and
Stan. \emph{Journal of Statistical Software}, \emph{100}, 1--54.
\doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{fit_statistic_rm}} for dichotomous Rasch models,
\code{\link[brms]{posterior_epred}} for expected predictions,
\code{\link[brms]{posterior_predict}} for posterior predictive samples,
\code{\link[brms]{pp_check}} for graphical posterior predictive checks.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post_helpers.R
\name{posterior_to_prior}
\alias{posterior_to_prior}
\title{Extract Informative Priors from a Fitted Bayesian IRT Model}
\usage{
posterior_to_prior(
  model,
  item_var = item,
  person_var = id,
  mult = 1,
  target_link = c("source", "logit", "probit")
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object. Supported
parameterisations:
\describe{
\item{Polytomous ordinal}{e.g., \code{family = acat} with
\code{thres(gr = item)}, producing item-specific thresholds.}
\item{Dichotomous Rasch (random items)}{e.g.,
\code{response ~ 1 + (1 | item) + (1 | id)} with
\code{family = bernoulli()}.}
\item{Dichotomous 1PL (fixed items)}{e.g.,
\code{response ~ 0 + item + (1 | id)} with
\code{family = bernoulli()}.}
}}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data. Default is \code{id}.}

\item{mult}{Numeric multiplier applied to each posterior SD before
it is used as the prior SD. Values > 1 widen the priors
(less informative); values < 1 tighten them. Default is 1
(use posterior SD directly).}

\item{target_link}{Character string specifying the link function of
the model the priors will be used with. One of \code{"logit"},
\code{"probit"}, or \code{"source"} (the default). When
\code{"source"}, the link function of the fitted model is used
and no transformation is applied. When different from the source
model's link, all location and scale parameters are rescaled
using the approximation \eqn{\beta_{\text{probit}} \approx
  \beta_{\text{logit}} / 1.7}. This is useful when transferring
priors from a logit-fitted model to a probit model or vice versa.}
}
\value{
A \code{\link[brms]{brmsprior}} object that can be supplied
to the \code{prior} argument of \code{\link[brms]{brm}} or
\code{\link[stats]{update}}.
}
\description{
Takes a fitted \code{\link[brms]{brmsfit}} object and constructs a
\code{\link[brms]{brmsprior}} object in which each item parameter
receives a \code{normal(mean, sd)} prior derived from its posterior
distribution. The person-level random effect SD prior is also
updated. The returned prior can be passed directly to
\code{\link[stats]{update}} (or \code{\link[brms]{brm}}) to refit
the model with empirical Bayes / informative priors — useful for
anchoring scales, warm-starting a model on new data, or
regularising estimation with small samples.
}
\details{
The function extracts all posterior draws via
\code{\link[brms]{as_draws_df}}, computes the mean and SD of each
parameter's marginal posterior, and constructs
\code{normal(mean, sd * mult)} priors.

\strong{Polytomous ordinal models} with grouped thresholds
(\code{thres(gr = item)}): each threshold receives its own prior
via \code{brms::set_prior("normal(...)", class = "Intercept",
group = item, coef = threshold_index)}.

\strong{Dichotomous Rasch models} parameterised as
\code{response ~ 1 + (1 | item) + (1 | id)}: priors are set on
the global intercept (\code{class = "Intercept"}), the item-level
SD (\code{class = "sd", group = item_var}), and the person-level
SD.

\strong{Dichotomous 1PL models} parameterised as
\code{response ~ 0 + item + (1 | id)}: each item-specific fixed
effect (e.g., \code{b_itemI1}) receives its own
\code{normal(mean, sd)} prior via
\code{brms::set_prior(..., class = "b", coef = "itemI1")}.

In all cases the person-level SD receives a
\code{normal(mean, sd * mult)} prior (brms applies the lower
bound of zero automatically for SD parameters).

\strong{Link function transformation:} When \code{target_link}
differs from the source model's link function, all parameters
(means and SDs) are rescaled by a factor of approximately 1.7.
This uses the well-known approximation that
\eqn{\Phi(x) \approx \text{logistic}(1.7 \, x)}, so
logit-scale parameters can be converted to probit-scale by
dividing by 1.7, and vice versa. The approximation is excellent
for parameters in the range \eqn{|\beta| < 3} and adequate
beyond that range.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4, cores = 2, iter = 1000 # use more iter (and cores if you have)
)

# Extract posterior-informed priors (same link)
new_priors <- posterior_to_prior(fit_pcm)
new_priors

# Narrow the prior's sd by a factor of 0.5
wide_priors <- posterior_to_prior(fit_pcm, mult = 0.5)

# Extract priors for use with a probit model
probit_priors <- posterior_to_prior(fit_pcm, target_link = "probit")

# --- Dichotomous 1PL (fixed item effects) ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_1pl <- brm(
  response ~ 0 + item + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4, cores = 2, iter = 1000 # use more iter (and cores if you have)
)

priors_1pl <- posterior_to_prior(fit_1pl)
priors_1pl

# Transfer logit priors to a probit refit
priors_probit <- posterior_to_prior(fit_1pl, target_link = "probit")
}

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/easyRaschBayes-package.R
\docType{package}
\name{easyRaschBayes-package}
\alias{easyRaschBayes}
\alias{easyRaschBayes-package}
\title{easyRaschBayes: Bayesian Rasch Analysis Using 'brms'}
\description{
Reproduces classic Rasch psychometric analysis features using Bayesian item response theory models fitted with 'brms' following Bürkner (2021) \doi{10.18637/jss.v100.i05} and Bürkner (2020) \doi{10.3390/jintelligence8010005}. Supports both dichotomous and polytomous Rasch models. Features include posterior predictive item fit, conditional infit, item-restscore associations, person fit, differential item functioning, local dependence assessment via Q3 residual correlations, dimensionality assessment with residual principal components analysis, person-item targeting plots, item category probability curves, and reliability using relative measurement uncertainty following Bignardi et al. (2025) \doi{10.31234/osf.io/h54k8_v1}.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/pgmj/easyRaschBayes}
  \item \url{https://pgmj.github.io/easyRaschBayes/}
  \item Report bugs at \url{https://github.com/pgmj/easyRaschBayes/issues}
}

}
\author{
\strong{Maintainer}: Magnus Johansson \email{pgmj@pm.me} (\href{https://orcid.org/0000-0003-1669-592X}{ORCID})

Other contributors:
\itemize{
  \item Giacomo Bignardi (RMU reliability code) [contributor]
}

}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_dif.R
\name{dif_statistic}
\alias{dif_statistic}
\title{Differential Item Functioning (DIF) Analysis for Bayesian IRT Models}
\usage{
dif_statistic(
  model,
  group_var,
  item_var = item,
  person_var = id,
  data = NULL,
  dif_type = c("uniform", "non-uniform"),
  prob = 0.95,
  rope = 0.5,
  refit = TRUE,
  ...
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from
the baseline (no-DIF) model.}

\item{group_var}{An unquoted variable name identifying the
grouping variable for DIF testing (e.g., \code{gender}).
Must be a factor or character variable with exactly 2 levels
in the current implementation.}

\item{item_var}{An unquoted variable name identifying the item
grouping variable. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the
person grouping variable. Default is \code{id}.}

\item{data}{An optional data frame containing all variables
needed for the DIF model, including the group variable. If
\code{NULL} (the default), the function attempts to use
\code{model$data}. Since the baseline model formula typically
does not include the group variable, brms will have dropped
it from the stored model data. In that case, you must supply
the original data frame here.}

\item{dif_type}{Character. For polytomous ordinal models only.
\code{"uniform"} (the default) tests for a uniform location
shift per item via a \code{group:item} fixed-effect
interaction. \code{"non-uniform"} fits group-specific
thresholds per item and computes per-threshold DIF effects
as the difference between groups. Ignored for dichotomous
models.}

\item{prob}{Numeric in \eqn{(0, 1)}. Width of the credible
intervals. Default is 0.95.}

\item{rope}{Numeric. Half-width of the Region of Practical
Equivalence (ROPE) around zero for DIF effects, on the logit
scale. Default is 0.5, corresponding to a practically
negligible DIF effect. Set to 0 to skip ROPE analysis.}

\item{refit}{Logical. If \code{TRUE} (the default), the DIF
model is fitted automatically by updating the baseline model
via \code{\link[stats]{update}}, which reuses the compiled
Stan code for faster sampling. If \code{FALSE}, only the DIF
model formula is returned (useful for manual fitting with
custom settings).}

\item{...}{Additional arguments passed to
\code{\link[brms]{update.brmsfit}} when refitting the DIF
model (e.g., \code{cores}, \code{control}).}
}
\value{
A list with the following elements:
\describe{
\item{summary}{A \code{\link[tibble]{tibble}} with one row per
item (for uniform DIF) or per item × threshold (for
non-uniform DIF) containing: \code{item}, optionally
\code{threshold}, \code{dif_estimate} (posterior mean),
\code{dif_lower}, \code{dif_upper} (credible interval),
\code{dif_sd} (posterior SD), \code{pd} (probability of
direction), \code{rope_percentage} (proportion inside ROPE),
and \code{flag} (classification).}
\item{dif_draws}{A matrix of posterior draws for the DIF
effects (draws × effects), for further analysis.}
\item{dif_model}{The fitted DIF \code{brmsfit} object (if
\code{refit = TRUE}), or \code{NULL}.}
\item{dif_formula}{The \code{brmsformula} used for the DIF
model.}
\item{baseline_model}{The original baseline model.}
\item{plot}{A \code{\link[ggplot2]{ggplot}} forest plot of
DIF effects with credible intervals and ROPE.}
}
}
\description{
Tests for differential item functioning (DIF) in Bayesian
Rasch-family models fitted with \pkg{brms} by comparing item
parameters across subgroups defined by an exogenous variable.
The function fits a DIF model that includes group-by-item
interactions and summarizes the posterior distribution of the
DIF effects.
}
\details{
For polytomous models, two types of DIF can be tested:
\describe{
\item{Uniform DIF (\code{dif_type = "uniform"}, default)}{
A single location shift per item across groups, modelled as
a \code{group:item} fixed-effect interaction. This tests
whether the average item difficulty differs between groups.}
\item{Non-uniform / threshold-level DIF
(\code{dif_type = "non-uniform"})}{Each item receives
group-specific thresholds via
\code{thres(gr = interaction(item, group))}. DIF effects
are computed as the difference in each threshold between
groups, revealing whether DIF affects specific response
categories.}
}

The function constructs a DIF model by adding a group-by-item
interaction to the baseline model:

\itemize{
\item \strong{Dichotomous models}
(\code{family = bernoulli()}): The baseline
\code{response ~ 1 + (1 | item) + (1 | id)} becomes
\code{response ~ 1 + group + (1 + group | item) + (1 | id)},
where the group slope varying by item captures item-specific
DIF.
\item \strong{Polytomous uniform DIF}
(\code{dif_type = "uniform"}): The baseline
\code{response | thres(gr = item) ~ 1 + (1 | id)} becomes
\code{response | thres(gr = item) ~ 1 + group:item + (1 | id)}.
\item \strong{Polytomous non-uniform DIF}
(\code{dif_type = "non-uniform"}): The baseline becomes
\code{response | thres(gr = item_group) ~ 1 + (1 | id)},
where \code{item_group = interaction(item, group)}. Each
item × group combination gets its own thresholds. DIF
effects are the differences between group-specific
thresholds for each item, computed draw-by-draw from the
posterior.
}

DIF effects are summarized using:
\describe{
\item{Probability of Direction (pd)}{The proportion of the
posterior on the dominant side of zero. Values > 0.975
indicate strong directional evidence.}
\item{ROPE}{The Region of Practical Equivalence (Kruschke,
2018). If > 95\\% of the posterior falls inside the ROPE,
the DIF effect is practically negligible. If > 95\\% falls
outside, the effect is practically significant.}
\item{Credible Interval}{If the CI excludes zero, there is
evidence of DIF at the specified credibility level.}
}
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Dichotomous Rasch with DIF testing ---

set.seed(123)
df <- expand.grid(id = 1:200, item = paste0("I", 1:10)) \%>\%
  mutate(
    gender = rep(sample(c("M", "F"), 200, TRUE), each = 10),
    theta  = rep(rnorm(200), each = 10),
    delta  = rep(seq(-2, 2, length.out = 10), 200),
    dif    = ifelse(item == "I3" & gender == "F", 1.0,
             ifelse(item == "I7" & gender == "F", -0.8, 0)),
    p      = plogis(theta - delta - dif),
    response = rbinom(n(), 1, p)
  )

fit_base <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df,
  family = bernoulli(),
  chains = 4, 
  cores  = 2, # use more cores if you have
  iter   = 1000 # use at least 2000 
)

dif_result <- dif_statistic(
  model     = fit_base,
  group_var = gender,
  data      = df
)

dif_result$summary
dif_result$plot

# --- Partial Credit Model: uniform DIF ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  mutate(gender = sample(c("M", "F"), n(), TRUE)) \%>\%
  pivot_longer(!c(id, gender),
               names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4, 
  cores  = 2, # use more cores if you have
  iter   = 1000 # use at least 2000 
)

# Uniform DIF (default): one shift per item
dif_uni <- dif_statistic(fit_pcm, group_var = gender, data = df_pcm)
dif_uni$plot

# Non-uniform DIF: threshold-level effects
dif_nu <- dif_statistic(fit_pcm, group_var = gender, data = df_pcm,
                         dif_type = "non-uniform")
dif_nu$summary
dif_nu$plot
}

}
\references{
Kruschke, J. K. (2018). Rejecting or accepting parameter values
in Bayesian estimation. \emph{Advances in Methods and Practices
in Psychological Science}, \emph{1}(2), 270--280.

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
1--54. \doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{infit_statistic}} for item fit,
\code{\link{q3_statistic}} for local dependence,
\code{\link[brms]{brm}},
\code{\link[brms]{hypothesis}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/item_parameters.R
\name{item_parameters}
\alias{item_parameters}
\title{Extract Item Parameters from a Bayesian Rasch Model}
\usage{
item_parameters(
  model,
  item_var = item,
  person_var = id,
  draws = FALSE,
  center = TRUE,
  prob = 0.95
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object. Supported
parameterisations:
\describe{
\item{Polytomous ordinal (PCM)}{e.g., \code{family = acat} with
\code{thres(gr = item)}, producing item-specific thresholds.}
\item{Dichotomous Rasch (random items)}{e.g.,
\code{response ~ 1 + (1 | item) + (1 | id)} with
\code{family = bernoulli()}.}
\item{Dichotomous 1PL (fixed items)}{e.g.,
\code{response ~ 0 + item + (1 | id)} with
\code{family = bernoulli()}.}
}}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data. Default is \code{id}.}

\item{draws}{Logical. If \code{TRUE}, a draws matrix of full
posterior draws is included in the output. Default is
\code{FALSE}.}

\item{center}{Logical. If \code{TRUE} (the default), item
parameters are shifted so that the grand mean of all threshold
locations is zero, matching the frequentist CML convention and
the centering used in \code{\link{plot_targeting}}.}

\item{prob}{Numeric in \eqn{(0, 1)}. Width of the highest density
continuous interval (HDCI) reported in the summary. Default is
0.95.}
}
\value{
A list with the following elements:
\describe{
\item{locations}{A \code{\link[tibble]{tibble}} in long format
with one row per item (dichotomous) or per item-threshold
(polytomous), containing \code{item}, \code{threshold}
(integer, always 1 for dichotomous), and \code{location}
(posterior mean on the logit scale).}
\item{locations_wide}{A \code{\link[tibble]{tibble}} in wide
format with one row per item, sorted by mean location. For
polytomous models, threshold columns are named \code{t1},
\code{t2}, etc., and a \code{location} column gives the mean
across thresholds. For dichotomous models, only \code{item}
and \code{location} columns are present.}
\item{summary}{A \code{\link[tibble]{tibble}} extending the long
\code{locations} with: \code{se} (posterior SD),
\code{hdci_lower} and \code{hdci_upper} (highest density
continuous interval bounds at the level specified by
\code{prob}), and \code{n_eff} (effective sample size for the
parameter).}
\item{item_information}{A \code{\link[tibble]{tibble}} with one
row per item containing: \code{item}, \code{location} (mean
item location, i.e. mean of thresholds for polytomous items),
\code{info_at_location} (Fisher information at the item's own
location), and \code{max_info} (maximum Fisher information
across theta). For Rasch dichotomous items, both are 0.25.
For polytomous items, information depends on the number and
spacing of thresholds.}
\item{threshold_order}{(Polytomous models only) A
\code{\link[tibble]{tibble}} with one row per item containing:
\code{item}, \code{n_thresholds}, \code{ordered} (logical:
are all thresholds in ascending order?), and
\code{prob_disordered} (posterior probability that at least
one pair of adjacent thresholds is disordered, i.e.
\eqn{\tau_{k+1} \le \tau_k}). \code{NULL} for dichotomous
models.}
\item{person_sd}{A \code{\link[tibble]{tibble}} with one row
containing: \code{mean}, \code{sd}, \code{hdci_lower},
\code{hdci_upper} — the posterior summary of the person-level
standard deviation parameter \eqn{\sigma_\theta}.}
\item{draws_matrix}{(Only if \code{draws = TRUE}) A numeric
matrix with rows = thresholds (named
\code{"item[threshold]"}) and columns = posterior draws. For
dichotomous models, row names are item labels only.}
}
}
\description{
Extracts item difficulty (threshold) parameters from a fitted
Bayesian Rasch model. Returns a simple location table in both long
and wide formats, a full summary with posterior SEs and HDCIs,
item-level information, threshold ordering diagnostics, and
optionally the full posterior draws matrix.
}
\details{
\strong{Dichotomous models} with random item effects
(\code{response ~ 1 + (1 | item) + (1 | id)}) parameterise item
difficulty as \eqn{\delta_i = -(b_0 + r_i)} where \eqn{b_0} is
the global intercept and \eqn{r_i} is the item random effect.
Models with fixed item effects (\code{response ~ 0 + item +
(1 | id)}) parameterise difficulty as \eqn{\delta_i = -b_i}.

\strong{Polytomous (acat/PCM) models} with grouped thresholds
(\code{thres(gr = item)}) directly estimate item-specific
threshold parameters. Each row in the long output represents one
threshold within one item; each row in the wide output represents
one item.

\strong{Item information} is computed from the posterior mean
item parameters using the standard Rasch/PCM information
formulae:
\describe{
\item{Dichotomous}{
\eqn{I_i(\theta) = P_i(\theta) Q_i(\theta)} where
\eqn{P_i = \text{logistic}(\theta - \delta_i)}.}
\item{Polytomous (PCM)}{
\eqn{I_i(\theta) = \sum_c (c - E_i)^2 P_{ic}(\theta)} where
\eqn{E_i = \sum_c c \cdot P_{ic}} is the expected score for
item \eqn{i}.}
}

\strong{Threshold ordering}: In the partial credit model,
disordered thresholds (\eqn{\tau_{k+1} \le \tau_k}) indicate that
the probability of responding in the intermediate category never
exceeds both adjacent categories — the category is empirically
"absorbed". This does not necessarily indicate misfit (see
Adams et al., 2012), but may suggest response categories should
be collapsed. The \code{prob_disordered} column reports the
posterior probability of at least one disordered pair per item,
providing a Bayesian alternative to post-hoc threshold checks.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4, cores = 2, iter = 1000 # use more iter and cores
)

ip <- item_parameters(fit_pcm)

# Long format: one row per threshold
ip$locations

# Wide format: one row per item, easy to scan
ip$locations_wide

# Full summary with SE and HDCI
ip$summary

# Item information
ip$item_information

# Threshold ordering diagnostic
ip$threshold_order

# Person SD
ip$person_sd

# With full posterior draws
ip_draws <- item_parameters(fit_pcm, draws = TRUE)
dim(ip_draws$draws_matrix)

# --- Dichotomous Rasch ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4, cores = 2, iter = 1000 # use more iter and cores
)

ip_rm <- item_parameters(fit_rm)
# Wide and long are equivalent for dichotomous:
ip_rm$locations
ip_rm$locations_wide
ip_rm$summary
}

}
\references{
Adams, R. J., Wu, M. L., & Wilson, M. (2012). The Rasch rating
model and the disordered threshold controversy. \emph{Educational
and Psychological Measurement}, \emph{72}(4), 547--573.
\doi{10.1177/0013164411432166}

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R
with brms and Stan. \emph{Journal of Statistical Software},
\emph{100}, 1--54. \doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{person_parameters}}, \code{\link{plot_targeting}},
\code{\link{plot_ipf}}, \code{\link{posterior_to_prior}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post_helpers.R
\name{infit_post}
\alias{infit_post}
\title{Summarize and Plot Posterior Predictive Infit Statistics}
\usage{
infit_post(infit_draws, ci = 0.84)
}
\arguments{
\item{infit_draws}{A data frame (or tibble) as returned by
\code{\link{infit_statistic}} containing at minimum the columns
\code{item}, \code{infit} (observed infit per draw), and
\code{infit_rep} (replicated infit per draw).}

\item{ci}{Numeric in \eqn{(0, 1)}. Width of the credible
interval used for the posterior predictive HDI and the slab
display. Default is 0.84.}
}
\value{
A list with three elements:
\describe{
\item{summary}{A \code{\link[tibble]{tibble}} with one row per
item containing: \code{item}, \code{infit_obs} (posterior
mean of observed infit), \code{infit_rep} (posterior mean of
replicated infit), and \code{infit_ppp} (posterior predictive
p-value: proportion of draws where the replicated infit
exceeds the observed infit). Values near 0.5 indicate good
fit; values near 0 suggest underfit; values near 1 suggest
overfit.}
\item{hdi}{A \code{\link[tibble]{tibble}} with one row per
item containing: \code{item}, \code{underfit} (posterior
probability that the observed infit exceeds the upper HDI
bound of the replicated distribution), and \code{overfit}
(posterior probability that the observed infit falls below
the lower HDI bound).}
\item{plot}{A \code{\link[ggplot2]{ggplot}} object showing
the posterior predictive distribution of replicated infit
(grey filled slab) overlaid with the observed infit
distribution (coloured slab + interval), with a dashed
reference line at 1 (perfect fit).}
}
}
\description{
Postprocesses the output of \code{\link{infit_statistic}} to produce summary tables
of posterior predictive infit statistics and a combined slab +
interval plot comparing observed infit values to the posterior
predictive distribution.
}
\details{
Two complementary summary tables are provided:

\describe{
\item{\code{summary}}{Reports the posterior mean observed and
replicated infit values alongside the posterior predictive
p-value (ppp). The ppp is the proportion of draws where the
replicated infit exceeds the observed infit. Under good fit,
the ppp should be near 0.5. A ppp near 0 indicates the
observed infit is consistently larger than expected
(underfit); a ppp near 1 indicates it is consistently
smaller (overfit).}
\item{\code{hdi}}{Reports the probability that the observed
infit falls above (underfit) or below (overfit) the HDI of
the replicated distribution. This provides a more
distributional assessment than the ppp alone.}
}

The plot uses two layers from the \pkg{ggdist} package:
\describe{
\item{\code{stat_slab}}{Displays the posterior predictive
(replicated) infit distribution as a filled density slab
per item, shaded by credible interval level.}
\item{\code{stat_slabinterval}}{Displays the observed infit
distribution per item as a semi-transparent slab with
point and interval summaries.}
}

Under good model fit, the observed infit distribution should
overlap substantially with the replicated distribution. Items
where the observed distribution sits systematically above the
replicated HDI indicate underfit (more variation than expected);
items below indicate overfit (less variation than expected).
}
\examples{
\dontrun{
library(brms)
library(ggplot2)

# Assuming fit_pcm is a fitted brmsfit object
infit_draws <- infit_statistic(fit_pcm)

result <- infit_post(infit_draws)
result$summary
result$hdi
result$plot
}

}
\seealso{
\code{\link{infit_statistic}},
\code{\link{item_restscore_post}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_globalfit.R
\name{plot_residual_pca}
\alias{plot_residual_pca}
\title{Residual PCA Contrast Plot for Bayesian IRT Models}
\usage{
plot_residual_pca(
  model,
  item_var = item,
  person_var = id,
  center = TRUE,
  prob = 0.95,
  ndraws_use = NULL,
  style = c("both", "density", "crosshair"),
  density_alpha = 0.3,
  density_bins = 6,
  density_palette = NULL,
  label_items = TRUE,
  point_size = 2.5,
  point_color = "#0072B2"
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from an
ordinal IRT model (e.g., \code{family = acat}) or a dichotomous
model (\code{family = bernoulli()}).}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data. Default is \code{id}.}

\item{center}{Logical. If \code{TRUE} (the default), item
locations are mean-centered to zero, matching the convention
used in \code{\link{plot_targeting}}.}

\item{prob}{Numeric in \eqn{(0, 1)}. Width of the credible
intervals for both loading and location whiskers. Default is
0.95.}

\item{ndraws_use}{Optional positive integer. Number of posterior
draws to use. If \code{NULL} (the default), up to 500 draws
are used.}

\item{style}{Character. Visual style for displaying uncertainty.
\code{"density"} (the default) overlays filled 2D density
contours per item computed from the draw-level location and
loading values, showing the full joint posterior uncertainty.
\code{"crosshair"} shows point estimates with horizontal and
vertical credible interval bars.
\code{"both"} displays density contours with crosshairs on top.}

\item{density_alpha}{Numeric in \eqn{[0, 1]}. Maximum opacity
of the density contours when \code{style} is \code{"density"}
or \code{"both"}. Default is 0.3.}

\item{density_bins}{Integer. Number of contour bins for
\code{\link[ggplot2]{geom_density_2d_filled}}. Default is 6.}

\item{density_palette}{An optional character vector of colors for
the density contour fills (from low to high density). If
\code{NULL} (the default), a blue sequential ramp is used. The
length should match \code{density_bins}; the lowest
(background) level is always transparent.}

\item{label_items}{Logical. If \code{TRUE} (the default), item
names are displayed next to points.}

\item{point_size}{Numeric. Size of the item points. Default is
2.5.}

\item{point_color}{Color for the item points and error bars.
Default is \code{"#0072B2"}.}
}
\value{
A list with three elements:
\describe{
\item{plot}{A \code{\link[ggplot2]{ggplot}} object showing item
loadings on the first residual contrast (y-axis) versus item
locations (x-axis).}
\item{data}{A \code{\link[tibble]{tibble}} with columns:
\code{item}, \code{location} (posterior mean item location),
\code{location_lower}, \code{location_upper} (location CI),
\code{loading} (posterior mean loading on first contrast),
\code{loading_lower}, \code{loading_upper} (loading CI).}
\item{eigenvalue}{A \code{\link[tibble]{tibble}} with columns:
\code{eigenvalue_obs} (posterior mean observed eigenvalue),
\code{eigenvalue_rep} (posterior mean replicated eigenvalue),
\code{eigenvalue_diff} (posterior mean difference),
\code{ppp} (posterior predictive p-value),
\code{var_explained_obs}, \code{var_explained_rep} (posterior
mean proportions of residual variance explained).}
}
}
\description{
Assesses dimensionality of Bayesian IRT models by performing a
principal component analysis (PCA) of standardized residuals for
each posterior draw. The item loadings on the first residual
contrast are plotted against item locations, with posterior
uncertainty displayed as 2D density contours, crosshairs, or
both. A posterior predictive p-value for the first-contrast
eigenvalue tests whether the observed residual structure exceeds
what the model predicts under unidimensionality.
}
\details{
The procedure for each posterior draw \eqn{s} is:

\enumerate{
\item Obtain category probabilities from
\code{\link[brms]{posterior_epred}}. Compute expected values
\eqn{E^{(s)}_{vi}} and variances \eqn{Var^{(s)}_{vi}}.
\item Compute standardized residuals for observed data:
\deqn{z^{obs(s)}_{vi} = \frac{X_{vi} - E^{(s)}_{vi}}
      {\sqrt{Var^{(s)}_{vi}}}}
\item Generate replicated data \eqn{Y^{rep(s)}} from
\code{\link[brms]{posterior_predict}} and compute standardized
residuals:
\deqn{z^{rep(s)}_{vi} = \frac{Y^{rep(s)}_{vi} -
      E^{(s)}_{vi}}{\sqrt{Var^{(s)}_{vi}}}}
\item Reshape both sets of residuals into person \eqn{\times}
item matrices and perform SVD on each.
\item Extract the first-contrast eigenvalue and item loadings
from both observed and replicated SVDs.
\item Compare eigenvalues across draws: the posterior predictive
p-value \code{ppp = mean(eigenvalue_obs > eigenvalue_rep)}
tests whether the observed residual structure is stronger than
what the model produces under its own assumptions.
}

When \code{style = "density"} or \code{style = "both"}, the
draw-level (location, loading) pairs for each item are used to
construct filled 2D kernel density contours via
\code{\link[ggplot2]{geom_density_2d_filled}}. The lowest
contour level (outside all contours) is set to transparent so
the white panel background shows through. Higher density regions
use progressively darker fills.

Item loadings are aligned across draws using majority-sign
alignment to resolve the sign indeterminacy of eigenvectors.
}
\examples{
\dontrun{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)
library(ggplot2)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4,
  cores  = 4,
  iter   = 2000
)

# 2D density contours (default)
result <- plot_residual_pca(fit_pcm)
result$plot

# Crosshair style
result_c <- plot_residual_pca(fit_pcm, style = "crosshair")
result_c$plot

# Both combined
result_b <- plot_residual_pca(fit_pcm, style = "both")
result_b$plot

# Custom warm palette
result_w <- plot_residual_pca(
  fit_pcm,
  density_palette = c("#FEE8C8", "#FDBB84", "#E34A33",
                      "#B30000", "#7F0000", "#4A0000"),
  point_color = "#B30000"
)
result_w$plot
}

}
\references{
Smith, E. V. (2002). Detecting and evaluating the impact of
multidimensionality using item fit statistics and principal
component analysis of residuals. \emph{Journal of Applied
Measurement}, \emph{3}, 205--231.

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
1--54. \doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{plot_targeting}} for person-item maps,
\code{\link{plot_ipf}} for item category probability curves,
\code{\link{q3_statistic}} for Q3 residual correlations (another
local dependence / dimensionality diagnostic).
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_localdep.R
\name{q3_statistic}
\alias{q3_statistic}
\title{Posterior Predictive Q3 Residual Correlations for Bayesian IRT Models}
\usage{
q3_statistic(model, item_var = item, person_var = id, ndraws_use = NULL)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from an
ordinal IRT model (e.g., \code{family = acat} for a partial credit
model) or a dichotomous model (\code{family = bernoulli()}).}

\item{item_var}{An unquoted variable name identifying the item grouping
variable in the model data. Default is \code{item}.}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data. Default is \code{id}.}

\item{ndraws_use}{Optional positive integer. If specified, a random
subset of posterior draws of this size is used. If \code{NULL} (the
default), all draws are used.}
}
\value{
A \code{\link[tibble]{tibble}} in long format with one row
per draw per item pair, containing:
\describe{
\item{draw}{Integer draw index.}
\item{item_pair}{Character label of the item pair
(\code{"item1 : item2"}).}
\item{item_1}{First item in the pair.}
\item{item_2}{Second item in the pair.}
\item{q3}{Observed Q3 residual correlation for this draw.}
\item{q3_rep}{Replicated Q3 residual correlation for this draw.}
}

This long-format output parallels the structure of
\code{\link{infit_statistic}} and can be passed directly to
\code{\link{q3_post}} for summary tables and plots.
}
\description{
Computes a Bayesian analogue of Yen's Q3 statistic (Yen, 1984) for
detecting local dependence between item pairs in Rasch-family models
fitted with \pkg{brms}. For each posterior draw, residual correlations
are computed for both observed and replicated data, yielding draw-level
Q3 values that can be summarized and visualized via
\code{\link{q3_post}}.
}
\details{
The procedure works as follows for each posterior draw \eqn{s}:

\enumerate{
\item Compute expected values \eqn{E^{(s)}_{vi}} from the category
probabilities returned by \code{\link[brms]{posterior_epred}}.
For ordinal models: \eqn{E^{(s)}_{vi} = \sum_c c \cdot
    P^{(s)}(X_{vi} = c)}. For binary models: \eqn{E^{(s)}_{vi} =
    P^{(s)}(X_{vi} = 1)}.
\item Compute observed residuals: \eqn{d^{(s)}_{vi} = X_{vi} -
    E^{(s)}_{vi}}.
\item Compute replicated residuals: \eqn{d^{rep(s)}_{vi} =
    Y^{rep(s)}_{vi} - E^{(s)}_{vi}}, where \eqn{Y^{rep}} is drawn
via \code{\link[brms]{posterior_predict}}.
\item For each item pair \eqn{(i, j)}, compute Q3 as the Pearson
correlation of residuals across all persons who responded to
both items.
}
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

# Q3 residual correlations
q3_draws <- q3_statistic(fit_pcm, ndraws_use = 500)

# Postprocess
result <- q3_post(q3_draws)
result$summary
result$hdi
result$plot

# --- Dichotomous Rasch Model ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

q3_draws <- q3_statistic(fit_rm, ndraws_use = 500)

# Postprocess
result <- q3_post(q3_draws)
result$summary
result$hdi
result$plot
}

}
\references{
Yen, W. M. (1984). Effects of local item dependence on the fit and
equating performance of the three-parameter logistic model.
\emph{Applied Psychological Measurement}, \emph{8}(2), 125--145.
\doi{10.1177/014662168400800201}

Christensen, K. B., Makransky, G. & Horton, M. (2017). Critical
values for Yen's Q3: Identification of local dependence in the
Rasch model using residual correlations.
\emph{Applied Psychological Measurement}, \emph{41}(3), 178--194.
\doi{10.1177/0146621616677520}

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
1--54. \doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{q3_post}} for postprocessing summaries and plots,
\code{\link{infit_statistic}} for Bayesian infit/outfit,
\code{\link[brms]{posterior_epred}},
\code{\link[brms]{posterior_predict}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_plots.R
\name{plot_ipf}
\alias{plot_ipf}
\title{Item Category Probability Function Curves for Polytomous IRT Models}
\usage{
plot_ipf(
  model,
  item_var = item,
  person_var = id,
  items = NULL,
  theta_range = c(-4, 4),
  n_points = 100,
  ncol = NULL,
  line_size = 0.8,
  ribbon_alpha = 0.15,
  prob = 0.95,
  category_labels = NULL,
  palette = NULL
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from a
polytomous IRT model (e.g., \code{family = acat} for a partial
credit model or \code{family = cumulative} for a graded response
model).}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data (e.g., \code{item}).}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data (e.g., \code{id}).}

\item{items}{An optional character vector of item names to plot.
If \code{NULL} (the default), all items in the model are plotted.}

\item{theta_range}{A numeric vector of length 2 specifying the range
of the latent variable (theta) for the x-axis. Default is
\code{c(-4, 4)}.}

\item{n_points}{Integer. Number of evenly spaced theta values at
which to evaluate the category probabilities. Default is 100.}

\item{ncol}{Integer. Number of columns in the faceted plot layout.
If \code{NULL} (the default), an appropriate number is chosen
automatically.}

\item{line_size}{Numeric. Line width for the probability curves.
Default is 0.8.}

\item{ribbon_alpha}{Numeric in \eqn{[0, 1]}. Transparency of the
credible interval ribbons. Default is 0.15. Set to 0 to hide
ribbons.}

\item{prob}{Numeric in \eqn{(0, 1)}. Width of the credible interval
for the ribbons. Default is 0.95.}

\item{category_labels}{An optional character vector of labels for
the response categories. If \code{NULL} (the default), categories
are labelled as integers starting from 1.}

\item{palette}{An optional character vector of colors, one per
response category. If \code{NULL} (the default), the
\code{viridis} discrete scale from \pkg{ggplot2} is used.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object. The plot can be
further customised using standard \pkg{ggplot2} functions.
}
\description{
Plots item category probability functions (ICPFs) for polytomous
Bayesian IRT models fitted with \pkg{brms}. For each item, the
probability of endorsing each response category is plotted as a
function of the latent variable (theta), with separate colored
curves per category. All items are displayed in a combined faceted
plot.
}
\details{
The function computes category probabilities directly from the
posterior draws of the item threshold parameters. For the brms
\code{acat} (adjacent category / partial credit) family with
logit link, the density is:
\deqn{P(Y = y | \eta) = \frac{\exp\bigl(\sum_{k=1}^{y}(\eta -
  \tau_k)\bigr)}{\sum_{k=0}^{K} \exp\bigl(\sum_{j=1}^{k}(\eta -
  \tau_j)\bigr)}}
where \eqn{\eta} is the linear predictor (i.e., theta for a Rasch
model with no additional fixed effects) and \eqn{\tau_k} are the
item thresholds. Analogous formulas are used for the
\code{cumulative}, \code{sratio}, and \code{cratio} families.

Posterior uncertainty in the thresholds propagates into credible
interval ribbons around the category probability curves — a
Bayesian advantage over point-estimate-based plots.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)
library(ggplot2)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

# Plot all items
plot_ipf(fit_pcm, item_var = item, person_var = id)

# Plot a subset of items
plot_ipf(fit_pcm, item_var = item, person_var = id,
         items = c("I1", "I2", "I3"))

# Customise appearance
plot_ipf(fit_pcm, item_var = item, person_var = id,
         theta_range = c(-6, 6), ncol = 3, prob = 0.90) +
  theme_minimal() +
  labs(title = "Item Category Probability Functions")
}

}
\references{
Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
1--54. \doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link[brms]{posterior_epred}},
\code{\link[brms]{conditional_effects}},
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/gk_gamma_helpers.R
\name{gk_gamma_vec}
\alias{gk_gamma_vec}
\title{Goodman-Kruskal Gamma from Two Vectors (No table() Overhead)}
\usage{
gk_gamma_vec(x, y)
}
\arguments{
\item{x}{Integer or numeric vector (e.g., item scores).}

\item{y}{Integer or numeric vector of the same length (e.g., rest scores).}
}
\value{
A scalar: the gamma coefficient in \eqn{[-1, 1]}, or
\code{NA_real_} if the table has fewer than 2 rows or 2 columns.
}
\description{
Computes the Goodman-Kruskal gamma coefficient directly from two
numeric vectors, bypassing \code{table()} for speed. Uses manual
tabulation with integer indexing and cumulative sums for efficient
concordant/discordant pair counting.
}
\references{
Goodman, L. A. & Kruskal, W. H. (1954). Measures of association for
cross classifications. \emph{Journal of the American Statistical
Association}, \emph{49}(268), 732--764.
}
\keyword{internal}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_fitstats.R
\name{infit_statistic}
\alias{infit_statistic}
\title{Posterior Predictive Infit Statistic for Bayesian IRT Models}
\usage{
infit_statistic(
  model,
  item_var = item,
  person_var = id,
  ndraws_use = NULL,
  outfit = FALSE
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from an ordinal
IRT model (e.g., \code{family = acat} for a partial credit model or
\code{family = bernoulli()} for a dichotomous Rasch model).}

\item{item_var}{An unquoted variable name identifying the item grouping
variable in the model data (e.g., \code{item}).}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data (e.g., \code{id}).}

\item{ndraws_use}{Optional positive integer. If specified, a random subset
of posterior draws of this size is used. If \code{NULL} (the default),
all draws are used.}

\item{outfit}{Logical. If \code{TRUE}, outfit statistics are computed
alongside infit. Default is \code{FALSE} (infit only), since outfit
is highly sensitive to outliers and rarely recommended for Rasch
diagnostics.}
}
\value{
A \code{\link[tibble]{tibble}} with the following columns:
\describe{
\item{item}{The item identifier.}
\item{draw}{Integer index of the posterior draw.}
\item{infit}{The observed infit statistic for that item and draw.}
\item{infit_rep}{The replicated infit statistic (based on posterior
predicted data) for that item and draw.}
\item{outfit}{(Only if \code{outfit = TRUE}) The observed outfit
statistic for that item and draw.}
\item{outfit_rep}{(Only if \code{outfit = TRUE}) The replicated
outfit statistic for that item and draw.}
}
The output is grouped by the item variable. Posterior predictive
p-values can be obtained by computing, e.g.,
\code{mean(infit_rep > infit)} within each item.
}
\description{
Computes a Bayesian analogue of the conditional item infit statistic
(as described in Christensen, Kreiner & Mesbah, 2013) for Rasch-family
models fitted with \pkg{brms}. For each posterior draw, expected values
and variances are derived from the category probabilities returned by
\code{\link[brms]{posterior_epred}}, and variance-weighted standardised
residuals are computed for both observed and replicated data. The result
can be summarised into posterior predictive p-values to assess item fit.
}
\details{
The procedure adapts the conditional infit/outfit statistics
(Christensen et al., 2013; Kreiner & Christensen, 2011; Müller, 2020)
to the Bayesian framework:

\enumerate{
\item For each posterior draw \eqn{s}, category probabilities
\eqn{P^{(s)}(X_{vi} = c)} are obtained from
\code{\link[brms]{posterior_epred}}.
\item The conditional expected value and variance for each
observation are computed as:
\deqn{E^{(s)}_{vi} = \sum_c c \cdot P^{(s)}(X_{vi} = c)}
\deqn{Var^{(s)}_{vi} = \sum_c (c - E^{(s)}_{vi})^2 \cdot
      P^{(s)}(X_{vi} = c)}
\item Standardised squared residuals are:
\deqn{Z^{2(s)}_{vi} = (X_{vi} - E^{(s)}_{vi})^2 / Var^{(s)}_{vi}}
\item The infit statistic for item \eqn{i} is the variance-weighted
mean of \eqn{Z^2} across persons:
\deqn{Infit_i^{(s)} = \frac{\sum_v Var_{vi}^{(s)} Z^{2(s)}_{vi}}
      {\sum_v Var_{vi}^{(s)}}}
\item If requested, the outfit is the unweighted mean of \eqn{Z^2}.
}
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Partial Credit Model (polytomous) ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

# Compute infit per item
item_infit <- infit_statistic(
  model      = fit_pcm,
  ndraws_use = 100 # use at least 500
)

# Post-process draws
infit_results <- infit_post(item_infit)
infit_results$summary
infit_results$hdi
infit_results$plot

# --- Dichotomous Rasch Model ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

item_infit_rm <- infit_statistic(
  model      = fit_rm,
  ndraws_use = 100 # use at least 500
)

# Post-process draws
infit_results <- infit_post(item_infit_rm)
infit_results$summary
infit_results$hdi
infit_results$plot

}

}
\references{
Christensen, K. B., Kreiner, S. & Mesbah, M. (Eds.) (2013).
\emph{Rasch Models in Health}. Iste and Wiley, pp. 86--90.

Kreiner, S. & Christensen, K. B. (2011). Exact evaluation of Bias in
Rasch model residuals. \emph{Advances in Mathematics Research}, 12,
19--40.

Müller, M. (2020). Item fit statistics for Rasch analysis: can we
trust them? \emph{Journal of Statistical Distributions and
Applications}, \emph{7}(1).
\doi{10.1186/s40488-020-00108-7}
}
\seealso{
\code{\link{fit_statistic_pcm}} for a general-purpose posterior predictive
fit statistic with user-supplied criterion functions,
\code{\link{fit_statistic_rm}} for a general-purpose posterior predictive
fit statistic with user-supplied criterion functions,
\code{\link[brms]{posterior_epred}},
\code{\link[brms]{posterior_predict}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_fitstats.R
\name{fit_statistic_rm}
\alias{fit_statistic_rm}
\title{Posterior Predictive Item Fit Statistic for Binary Bayesian IRT Models}
\usage{
fit_statistic_rm(model, criterion, group, ndraws_use = NULL)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object with a binary
response (e.g., \code{family = bernoulli()}).}

\item{criterion}{A function with signature \code{function(y, p)} that
computes a pointwise fit criterion, where \code{y} is the binary
response (0 or 1) and \code{p} is the predicted probability of
success. A common choice is the Bernoulli log-likelihood:
\code{function(y, p) y * log(p) + (1 - y) * log(1 - p)}.}

\item{group}{An unquoted variable name (e.g., \code{item} or \code{id})
indicating the grouping variable over which the fit statistic is
aggregated. Typically \code{item} for item fit or \code{id} for person
fit.}

\item{ndraws_use}{Optional positive integer. If specified, a random subset
of posterior draws of this size is used, which can speed up computation
for large models. If \code{NULL} (the default), all draws are used.}
}
\value{
A \code{\link[tibble]{tibble}} with the following columns:
\describe{
\item{\code{group}}{The grouping variable (e.g., item name or person id).}
\item{draw}{Integer index of the posterior draw.}
\item{crit}{The observed fit statistic (criterion applied to observed
data) summed within each group and draw.}
\item{crit_rep}{The replicated fit statistic (criterion applied to
posterior predicted data) summed within each group and draw.}
\item{crit_diff}{The difference \code{crit_rep - crit}.}
}
The output is grouped by the grouping variable. Posterior predictive
p-values can be obtained by computing
\code{mean(crit_rep > crit)} within each group.
}
\description{
Computes posterior predictive item (or person) fit statistics for
dichotomous Bayesian IRT models fitted with \pkg{brms}. For each
posterior draw, observed and replicated data are compared via a
user-supplied criterion function, grouped by item, person, or any other
variable. Posterior predictive p-values can then be derived from the
output to assess fit.
}
\details{
This function is the binary-response counterpart of
\code{\link{fit_statistic_pcm}}, which handles polytomous (ordinal /
categorical) models. For dichotomous models, \code{posterior_epred()}
returns a 2D matrix (S x N) of success probabilities, so the criterion
function receives the observed binary response and the corresponding
probability directly.

The procedure follows the posterior predictive checking approach
described in Bürkner (2020):

\enumerate{
\item Draw posterior expected success probabilities via
\code{\link[brms]{posterior_epred}} and posterior predicted binary
responses via \code{\link[brms]{posterior_predict}}.
\item Apply the user-supplied \code{criterion} function pointwise
to both observed and replicated data paired with the predicted
probabilities.
\item Aggregate (sum) the criterion values within each level of
\code{group} and each posterior draw.
}

The standard criterion for binary models is the Bernoulli log-likelihood:
\deqn{\ell(y, p) = y \log(p) + (1 - y) \log(1 - p).}
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Dichotomous Rasch Model ---

# Prepare binary response data in long format
df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

# Fit a dichotomous Rasch model
fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4,
  cores   = 1, # use more cores if you have
  iter    = 500 # use at least 2000 
)

# Bernoulli log-likelihood criterion
ll_bernoulli <- function(y, p) y * log(p) + (1 - y) * log(1 - p)

# Compute item fit statistics
item_fit <- fit_statistic_rm(
  model      = fit_rm,
  criterion  = ll_bernoulli,
  group      = item,
  ndraws_use = 100 # use at least 500
)

# Summarise: posterior predictive p-values per item
item_fit \%>\%
  group_by(item) \%>\%
  summarise(
    observed   = mean(crit),
    replicated = mean(crit_rep),
    ppp        = mean(crit_rep > crit)
  )

# Use ggplot2 to make a histogram
library(ggplot2)
item_fit \%>\%
  ggplot(aes(crit_diff)) +
  geom_histogram(aes(fill = ifelse(crit_diff > 0, "above","below"))) +
  facet_wrap("item") +
  theme_bw() +
  theme(legend.position = "none")

# Compute person fit statistics
person_fit <- fit_statistic_rm(
  model      = fit_rm,
  criterion  = ll_bernoulli,
  group      = id,
  ndraws_use = 100 # use at least 500
)

person_fit \%>\%
  group_by(id) \%>\%
  summarise(
    observed   = mean(crit),
    replicated = mean(crit_rep),
    ppp        = mean(crit_rep > crit)
  )

# --- 1PL model with item-specific intercepts ---

# Alternative parameterisation with fixed item effects
fit_1pl <- brm(
  response ~ 0 + item + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4,
  cores   = 1, # use more cores if you have
  iter    = 500 # use at least 2000 
)

item_fit_1pl <- fit_statistic_rm(
  model      = fit_1pl,
  criterion  = ll_bernoulli,
  group      = item,
  ndraws_use = 100 # use at least 500
)

item_fit_1pl \%>\%
  group_by(item) \%>\%
  summarise(
    observed   = mean(crit),
    replicated = mean(crit_rep),
    ppp        = mean(crit_rep > crit)
  )
}

}
\references{
Bürkner, P.-C. (2020). Analysing Standard Progressive Matrices (SPM-LS)
with Bayesian Item Response Models. \emph{Journal of Intelligence},
\emph{8}(1). \doi{10.3390/jintelligence8010005}

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with brms and
Stan. \emph{Journal of Statistical Software}, \emph{100}, 1--54.
\doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{fit_statistic_pcm}} for polytomous (ordinal/categorical) models,
\code{\link[brms]{posterior_epred}} for expected predictions,
\code{\link[brms]{posterior_predict}} for posterior predictive samples,
\code{\link[brms]{pp_check}} for graphical posterior predictive checks.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_plots.R
\name{plot_tile}
\alias{plot_tile}
\title{Tile Plot of Item Response Distributions}
\usage{
plot_tile(
  data,
  cutoff = 10,
  highlight = TRUE,
  percent = FALSE,
  text_color = "orange",
  item_labels = NULL,
  category_labels = NULL
)
}
\arguments{
\item{data}{A data frame in wide format containing only the item
response columns. Each column is one item, each row is one
person. All columns must be numeric (integer-valued). Response
categories may be coded starting from 0 or 1. Do not include
person IDs, grouping variables, or other non-item columns.}

\item{cutoff}{Integer. Cells with counts below this value are
highlighted (when \code{highlight = TRUE}). Default is 10.}

\item{highlight}{Logical. If \code{TRUE} (the default), cell
labels with counts below \code{cutoff} are displayed in red.
This includes cells with zero responses (empty categories),
which is useful for identifying gaps in the response
distribution.}

\item{percent}{Logical. If \code{TRUE}, cell labels show
percentages instead of raw counts. Default is \code{FALSE}.}

\item{text_color}{Character. Color for cell label text (when
not highlighted). Default is \code{"orange"}.}

\item{item_labels}{An optional character vector of descriptive
labels for the items (y-axis). Must be the same length as
\code{ncol(data)}. If \code{NULL} (the default), column names
are used.}

\item{category_labels}{An optional character vector of labels
for the response categories (x-axis). Must be the same length
as the number of response categories spanning from the minimum
to the maximum observed value. If \code{NULL} (the default),
numeric category values are used.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object.
}
\description{
Creates a tile (heat map) plot showing the distribution of
responses across all items and response categories. Each cell
displays the count (or percentage) of responses, with optional
conditional highlighting for cells with low counts. This is a
descriptive data visualization tool intended for use before
model fitting.
}
\details{
The plot displays items on the y-axis (in the same order as the
columns in \code{data}, from top to bottom) and response
categories on the x-axis. Cell shading represents the count
of responses (darker = more responses). Cell labels show either
raw counts or percentages.

Categories with zero responses are explicitly shown (as cells
with \code{n = 0}), which helps identify gaps in the response
distribution — one of the primary purposes of this plot.

\strong{Input requirements:}
\itemize{
\item All columns must be numeric (integer-valued).
\item The data frame must contain at least 2 columns (items)
and at least 1 row (person).
}
}
\examples{
library(ggplot2)
if (requireNamespace("eRm", quietly = TRUE))

# Basic tile plot
plot_tile(eRm::pcmdat2)

# With custom item labels
plot_tile(
  eRm::pcmdat2,
  item_labels = c("Mood", "Sleep", "Appetite", "Energy")
)

# With custom category labels and percentages
plot_tile(
  eRm::pcmdat2,
  category_labels = c("Never", "Sometimes", "Often"),
  percent = TRUE
)

# Adjust cutoff for highlighting
plot_tile(eRm::pcmdat2, cutoff = 20, highlight = TRUE)

}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/post_helpers.R
\name{q3_post}
\alias{q3_post}
\title{Summarize and Plot Posterior Predictive Q3 Residual Correlations}
\usage{
q3_post(
  q3_draws,
  ci = 0.84,
  n_pairs = NULL,
  sort_by = c("q3_diff", "q3_obs", "ppp")
)
}
\arguments{
\item{q3_draws}{A data frame (or tibble) as returned by
\code{\link{q3_statistic}} containing at minimum the columns
\code{item_pair}, \code{q3} (observed Q3 per draw), and
\code{q3_rep} (replicated Q3 per draw).}

\item{ci}{Numeric in \eqn{(0, 1)}. Width of the credible
interval used for the posterior predictive HDI and the slab
display. Default is 0.84.}

\item{n_pairs}{Integer. Maximum number of item pairs to display
in the plot, selected by largest absolute \code{q3_diff}. If
\code{NULL} (the default), all pairs are shown. Useful when
the number of item pairs is large.}

\item{sort_by}{Character. How to order item pairs on the y-axis.
\code{"q3_diff"} (the default) sorts by the posterior mean
difference between observed and replicated Q3 (largest at top).
\code{"q3_obs"} sorts by the posterior mean observed Q3.
\code{"ppp"} sorts by the posterior predictive p-value.}
}
\value{
A list with three elements:
\describe{
\item{summary}{A \code{\link[tibble]{tibble}} with one row per
item pair containing: \code{item_pair}, \code{item_1},
\code{item_2}, \code{q3_obs} (posterior mean observed Q3),
\code{q3_rep} (posterior mean replicated Q3),
\code{q3_diff} (posterior mean difference), and \code{q3_ppp}
(posterior predictive p-value: proportion of draws where the
observed Q3 exceeds the replicated Q3).}
\item{hdi}{A \code{\link[tibble]{tibble}} with one row per
item pair containing: \code{item_pair}, \code{item_1},
\code{item_2}, \code{ld} (local dependence probability:
proportion of draws where observed Q3 exceeds upper HDI
bound of replicated distribution), and \code{lr} (local
repulsion probability: proportion of draws where observed
Q3 falls below lower HDI bound).}
\item{plot}{A \code{\link[ggplot2]{ggplot}} object showing
the posterior predictive distribution of replicated Q3
(grey filled slab) overlaid with the observed Q3
distribution (coloured slab + interval), with a dashed
reference line at 0.}
}
}
\description{
Postprocesses the output of \code{\link{q3_statistic}} to produce
summary tables of posterior predictive Q3 statistics and a combined
slab + interval plot comparing observed Q3 residual correlations
to the posterior predictive distribution for each item pair.
}
\details{
Two complementary summary tables are provided:

\describe{
\item{\code{summary}}{Reports posterior mean observed and
replicated Q3 values alongside the posterior predictive
p-value (ppp). The ppp is the proportion of draws where
the observed Q3 exceeds the replicated Q3. Under good fit
(no local dependence), the ppp should be near 0.5. A ppp
close to 1 indicates the observed correlation is
systematically higher than expected (local dependence);
a ppp close to 0 indicates it is systematically lower
(local repulsion, e.g., speed-accuracy tradeoffs).}
\item{\code{hdi}}{Reports the probability that the observed
Q3 falls above (local dependence) or below (local repulsion)
the HDI of the replicated distribution. This provides a
more distributional assessment than the ppp alone.}
}

The plot uses two layers from the \pkg{ggdist} package:
\describe{
\item{\code{stat_slab}}{Displays the posterior predictive
(replicated) Q3 distribution as a filled density slab per
item pair, shaded by credible interval level.}
\item{\code{stat_slabinterval}}{Displays the observed Q3
distribution per item pair as a semi-transparent slab with
point and interval summaries.}
}
}
\examples{
\dontrun{
library(brms)
library(ggplot2)

# Assuming fit_pcm is a fitted brmsfit object
q3_draws <- q3_statistic(fit_pcm, ndraws_use = 500)

result <- q3_post(q3_draws)
result$summary
result$hdi
result$plot

# Show only top 10 pairs by Q3 difference
result_top <- q3_post(q3_draws, n_pairs = 10)
result_top$plot
}

}
\seealso{
\code{\link{q3_statistic}},
\code{\link{infit_post}},
\code{\link{item_restscore_post}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/brms_fitstats.R
\name{item_restscore_statistic}
\alias{item_restscore_statistic}
\title{Posterior Predictive Item-Restscore Association for Bayesian IRT Models}
\usage{
item_restscore_statistic(
  model,
  item_var = item,
  person_var = id,
  ndraws_use = NULL
)
}
\arguments{
\item{model}{A fitted \code{\link[brms]{brmsfit}} object from an
ordinal IRT model (e.g., \code{family = acat} for a partial credit
model) or a dichotomous model (\code{family = bernoulli()}).}

\item{item_var}{An unquoted variable name identifying the item
grouping variable in the model data (e.g., \code{item}).}

\item{person_var}{An unquoted variable name identifying the person
grouping variable in the model data (e.g., \code{id}).}

\item{ndraws_use}{Optional positive integer. If specified, a random
subset of posterior draws of this size is used. If \code{NULL} (the
default), all draws are used.}
}
\value{
A \code{\link[tibble]{tibble}} with the following columns:
\describe{
\item{item}{The item identifier.}
\item{gamma_obs}{Posterior mean of the observed Goodman-Kruskal
gamma between this item and the rest-score.}
\item{gamma_rep}{Posterior mean of the replicated gamma.}
\item{gamma_diff}{Posterior mean of \code{gamma_obs - gamma_rep}.
Positive values indicate the observed item-restscore association
is stronger than the model expects.}
\item{ppp}{Posterior predictive p-value:
\code{mean(gamma_obs > gamma_rep)} across draws.
Values close to 1 indicate the item discriminates more than the
model predicts (too high discrimination). Values close to 0
indicate the item discriminates less than expected (too low
discrimination, e.g., noise or miskeyed item).}
\item{gamma_obs_q025, gamma_obs_q975}{95\\% credible interval for
the observed gamma.}
\item{gamma_obs_q005, gamma_obs_q995}{99\\% credible interval for
the observed gamma.}
\item{gamma_diff_q025, gamma_diff_q975}{95\\% credible interval for
the gamma difference.}
\item{gamma_diff_q005, gamma_diff_q995}{99\\% credible interval for
the gamma difference.}
}
}
\description{
Computes a Bayesian analogue of the item-restscore association test
(Kreiner, 2011) for Rasch-family models fitted with \pkg{brms}. For
each posterior draw, the Goodman-Kruskal gamma coefficient between
each item's score and the rest-score (total score minus that item)
is computed for both observed and replicated data. Posterior
predictive p-values indicate whether the observed association is
stronger than the model predicts, which signals violations of
local independence or unidimensionality.
}
\details{
The item-restscore association is a key diagnostic in Rasch
measurement. Under the Rasch model, each item should relate to the
latent trait (and hence the rest-score) only through the modelled
relationship. Goodman-Kruskal's gamma is a rank-based measure of
association for ordinal cross-tabulations that is well-suited for
this purpose (Kreiner, 2011).

The procedure for each posterior draw \eqn{s} is:

\enumerate{
\item Obtain replicated responses \eqn{Y^{rep(s)}} from
\code{\link[brms]{posterior_predict}}.
\item For each item \eqn{i} and each person \eqn{v}, compute the
rest-score: \eqn{R^{obs}_{vi} = \sum_{j \neq i} X_{vj}} for
observed data and \eqn{R^{rep(s)}_{vi} = \sum_{j \neq i}
    Y^{rep(s)}_{vj}} for replicated data.
\item Cross-tabulate item score \eqn{\times} rest-score and compute
the Goodman-Kruskal gamma for both observed and replicated data.
\item Compare the two gammas across draws.
}

Items with \code{ppp} close to 1 have observed item-restscore
association that is consistently stronger than the model predicts.
This typically indicates that the item discriminates more than
assumed under the equal-discrimination Rasch model (i.e., a
violation of the Rasch assumption). Items with \code{ppp} close to
0 discriminate less than expected.
}
\examples{
\donttest{
library(brms)
library(dplyr)
library(tidyr)
library(tibble)

# --- Partial Credit Model ---

df_pcm <- eRm::pcmdat2 \%>\%
  mutate(across(everything(), ~ .x + 1)) \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_pcm <- brm(
  response | thres(gr = item) ~ 1 + (1 | id),
  data   = df_pcm,
  family = acat,
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

# Item-restscore association
irs <- item_restscore_statistic(
  model      = fit_pcm,
  ndraws_use = 100 # use at least 500
)

# Post-process draws
irs_results <- item_restscore_post(irs)
irs_results$summary
irs_results$plot

# --- Dichotomous Rasch Model ---

df_rm <- eRm::raschdat3 \%>\%
  as.data.frame() \%>\%
  rownames_to_column("id") \%>\%
  pivot_longer(!id, names_to = "item", values_to = "response")

fit_rm <- brm(
  response ~ 1 + (1 | item) + (1 | id),
  data   = df_rm,
  family = bernoulli(),
  chains = 4,
  cores  = 1, # use more cores if you have
  iter   = 500 # use at least 2000 
)

irs_rm <- item_restscore_statistic(
  model      = fit_rm,
  ndraws_use = 100 # use at least 500
)

# Post-process draws
irs_results <- item_restscore_post(irs_rm)
irs_results$summary
irs_results$plot
}

}
\references{
Kreiner, S. (2011). A note on item-restscore association in Rasch
models. \emph{Applied Psychological Measurement}, \emph{35}(7),
557--561.

Goodman, L. A. & Kruskal, W. H. (1954). Measures of association for
cross classifications. \emph{Journal of the American Statistical
Association}, \emph{49}(268), 732--764.

Bürkner, P.-C. (2021). Bayesian Item Response Modeling in R with
brms and Stan. \emph{Journal of Statistical Software}, \emph{100},
1--54. \doi{10.18637/jss.v100.i05}
}
\seealso{
\code{\link{fit_statistic_pcm}} for posterior predictive fit statistics,
\code{\link{fit_statistic_rm}} for posterior predictive fit statistics,
\code{\link{infit_statistic}} for Bayesian infit/outfit,
\code{\link{q3_statistic}} for Bayesian Q3 residual correlations,
\code{\link[brms]{posterior_predict}}.
}

% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/data_plots.R
\name{plot_bars}
\alias{plot_bars}
\title{Item Response Distribution Bar Chart}
\usage{
plot_bars(
  data,
  item_labels = NULL,
  category_labels = NULL,
  ncol = 1,
  label_wrap = 25,
  text_y = 6,
  viridis_option = "A",
  viridis_end = 0.9,
  font = "sans"
)
}
\arguments{
\item{data}{A data frame in wide format containing only the item
response columns. Each column is one item, each row is one
person. All columns must be numeric (integer-valued). Response
categories may be coded starting from 0 or 1. Do not include
person IDs, grouping variables, or other non-item columns.}

\item{item_labels}{An optional character vector of descriptive
labels for the items (facet strips). Must be the same length
as \code{ncol(data)}. If \code{NULL} (the default), column
names are used. Labels are displayed as
\code{"column_name - label"}.}

\item{category_labels}{An optional character vector of labels
for the response categories (x-axis). Must be the same length
as the number of response categories spanning from the minimum
to the maximum observed value. If \code{NULL} (the default),
numeric category values are used.}

\item{ncol}{Integer. Number of columns in the faceted layout.
Default is 1.}

\item{label_wrap}{Integer. Number of characters per line in
facet strip labels before wrapping. Default is 25.}

\item{text_y}{Numeric. Vertical position (in percent units) for
the count labels on each bar. Adjust upward if bars are tall.
Default is 6.}

\item{viridis_option}{Character. Viridis palette option for the
count text color. One of \code{"A"} through \code{"H"}.
Default is \code{"A"}.}

\item{viridis_end}{Numeric in \eqn{[0, 1]}. End point of the
viridis color scale for count text. Adjust if text is hard to
read against the bar colors. Default is 0.9.}

\item{font}{Character. Font family for all text. Default is
\code{"sans"}.}
}
\value{
A \code{\link[ggplot2]{ggplot}} object.
}
\description{
Creates a faceted bar chart showing the response distribution for
each item, with counts and percentages displayed on each bar.
Each item gets its own panel, with response categories on the
x-axis and percentage of responses on the y-axis. This is a
descriptive data visualization tool intended for use before
model fitting.
}
\details{
Each item is displayed as a separate facet panel with the item
label in the strip on the left side. Bars are colored by response
category using the viridis palette. Each bar shows the count
(\code{n = X}) as text.

\strong{Input requirements:}
\itemize{
\item All columns must be numeric (integer-valued).
\item The data frame must contain at least 2 columns (items)
and at least 1 row (person).
}
}
\examples{
library(ggplot2)
if (requireNamespace("eRm", quietly = TRUE))

# Basic response distribution plot
plot_bars(eRm::pcmdat2)

# With custom item labels
plot_bars(
  eRm::pcmdat2,
  item_labels = c("Mood", "Sleep", "Appetite", "Energy")
)

# Two-column layout with wrapped labels
plot_bars(
  eRm::pcmdat2,
  item_labels = c(
    "General mood and emotional wellbeing",
    "Quality of sleep at night",
    "Appetite and eating habits",
    "Overall energy level during the day"
  ),
  ncol = 2, label_wrap = 20
)

# With custom category labels
plot_bars(
  eRm::pcmdat2,
  category_labels = c("Never", "Sometimes", "Often")
)

}
